/******************************************************************************
TransAccord filter plugin for Avisynth by V.C.Mohan
The LeftClip  funnels out  in the direction specified,unmasking a stationary RightClip clip.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also 
used partly adapted function coding 
from Avisynth and simple sample. 
 shading along roll length are other params
********************************************************************************/  
//#include "windows.h"
//#include "avisynth.h"
//#include "LineZ.cpp"
class TransAccord : public GenericVideoFilter {
		PClip RightClip;
		int overlap;
		const char* dir;
		const bool twin ;
		const bool open ;
		void * abuf;
		
		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		 
  public:
							//Definition of function
    TransAccord(PClip _child, PClip _RightClip,int _overlap, const char* _dir,
					const bool _twin, const bool _open,
				   
				IScriptEnvironment* env) ;	
			
 	~TransAccord();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransAccord::TransAccord(PClip _child, PClip _RightClip,int _overlap, 
						 const char* _dir,
							const bool _twin, const bool _open,		
							IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap(_overlap), dir(_dir),
						twin(_twin), open(_open)
						 
			
{
	char * Tname="TransAccord";	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransAccord:Clips have differing pixel_types");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransAccord: This color format is not supported here");
	if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
		env->ThrowError("TransAccord: The heights/widths of clips are unequal");
	if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
		env->ThrowError("TransAccord: Height/width of frame are not multiple of 4");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 
	if(!(*dir=='v') && !(*dir=='V') && !(*dir=='h') && !(*dir=='H') )
		env->ThrowError("TransAccord:doors are either vertical or Horizontal only ");
  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	
	
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);
}
	
	TransAccord::~TransAccord()
	{
	// include in destructor
	
	}
	
/***************************************************************/	
	PVideoFrame __stdcall TransAccord::GetFrame(int n, IScriptEnvironment* env)
	{
		PVideoFrame LeftFrame = child->GetFrame(n, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& vi = child->GetVideoInfo();
		if(n==vi.num_frames)
			return RightFrame;	// Being last frame 100% transition
		if(n==0)
			return LeftFrame;
				 
		const	int bwd = vi.width;
        const	int bht = LeftFrame->GetHeight();
		int dx =(double) n*bwd/vi.num_frames;
		int dy =(double) n*bht/vi.num_frames;
		int nfoldw= (bwd/40) & 0xfffffffe;
		int nfoldh =(bht/40) & 0xfffffffe;
		int shade;
		const	int pitch = LeftFrame->GetPitch();	
		if(!(pitch==RightFrame->GetPitch()))
			env->ThrowError("TransAccord:unequal pitch values");
		double  taperh, taperw;
		double taper=3.0;						// trial & error arrived at value
		const unsigned char * srcp,*srcpU,*srcpV;
		unsigned char * dstp,*dstpU,*dstpV;
		const int spitchUV=open?LeftFrame->GetPitch(PLANAR_U):RightFrame->GetPitch(PLANAR_U);
		const int dpitchUV=!open?LeftFrame->GetPitch(PLANAR_U):RightFrame->GetPitch(PLANAR_U);
		const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
		const int bhtUV = LeftFrame->GetHeight(PLANAR_U);	
		if (open)
		{
			env->MakeWritable(&RightFrame);
			dstp= RightFrame->GetWritePtr();
			srcp= LeftFrame->GetReadPtr();
			dx=bwd-dx;							// width of visible screen
			dy=bht-dy;							// height of visible screen
			shade=254.0-64.0*(n)/vi.num_frames;	// alternate fold sides to be darkened
			taperw=taper*n/vi.num_frames;		// max width of segment to be blacked out for fold effect
			taperh=taper*n/vi.num_frames;		// max height of segment to be blackened
			if(vi.IsYV12())
			{
				
				srcpU = LeftFrame->GetReadPtr(PLANAR_U);
				srcpV = LeftFrame->GetReadPtr(PLANAR_V);
				dstpU = RightFrame->GetWritePtr(PLANAR_U);
				dstpV = RightFrame->GetWritePtr(PLANAR_V);
				
			}

		}
		if(!open)
		{
			env->MakeWritable(&LeftFrame);
			dstp= LeftFrame->GetWritePtr();
			srcp= RightFrame->GetReadPtr();
			shade=192.0+64.0*n/vi.num_frames;
			taperw=taper*(vi.num_frames-n)/vi.num_frames;
			taperh=taper*(vi.num_frames-n)/vi.num_frames;
			if(vi.IsYV12())
			{
				
				dstpU = LeftFrame->GetWritePtr(PLANAR_U);
				dstpV = LeftFrame->GetWritePtr(PLANAR_V);
				srcpU = RightFrame->GetReadPtr(PLANAR_U);
				srcpV = RightFrame->GetReadPtr(PLANAR_V);

			}
			
		}
		if(twin)
		{
			dx=1+dx/2;
			dy=1+dy/2;
			nfoldh/=2;
			nfoldw/=2;
		}
		dx=dx & 0xfffffffc;	//ensure even number for YUY2 format
		dy=dy & 0xfffffffe;
		if (dy<4 || dx<4)
			if(open)
				return RightFrame;
			else
				return LeftFrame;
		if (bht-dy<4 || bwd-dx<4)
			if(open)
				return LeftFrame;
			else
				return RightFrame;

		int kb=vi.BitsPerPixel()/8;

	
		if((*dir=='h' || *dir=='H'))
			
		{			//Horizontal folding

			if(twin)
			{
				if(vi.IsRGB32())
				{
					for(int w=0;w<bwd;w++)
					{
						ResizeVLineRGB32( srcp+pitch*(bht/4)+kb*w, 
										dstp+(pitch*dy)/2+kb*w,
										dy,pitch,pitch,(double)(bht/2)/dy);
						ResizeVLineRGB32( srcp+(bht-1-bht/4)*pitch+kb*w, 
										dstp+pitch*(bht-1-dy/2)+kb*w,
										dy,pitch,pitch,(double)(bht/2)/dy);
					}
				}
				if(vi.IsRGB24())
				{
					for(int w=0;w<bwd;w++)
					{
						ResizeVLineRGB24( srcp+pitch*(bht/4)+kb*w, 
										dstp+(pitch*dy)/2+kb*w,
										dy,pitch,pitch,(double)(bht/2)/dy);
						ResizeVLineRGB24( srcp+(bht-1-bht/4)*pitch+kb*w, 
										dstp+pitch*(bht-1-dy/2)+kb*w,
										dy,pitch,pitch,(double)(bht/2)/dy);
					}
				}
				if(vi.IsYUY2())
				{
					for(int w=0;w<bwd;w+=2)
					{
						ResizeVLineYUY2( srcp+pitch*(bht/4)+kb*w, 
										dstp+(pitch*dy)/2+kb*w,
										dy,pitch,pitch,(double)(bht/2)/dy);
						ResizeVLineYUY2( srcp+(bht-1-bht/4)*pitch+kb*w, 
										dstp+pitch*(bht-1-dy/2)+kb*w,
										dy,pitch,pitch,(double)(bht/2)/dy);
					}

					for(int f=0; f<nfoldh; f+=2)		// shade alternate segments
						for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
							for(int w=0;w<bwd;w++)
						
								{
									*(dstp+h*pitch+kb*w)
										=(*(dstp+h*pitch+kb*w))*shade/256;
									*(dstp+(bht-1-h)*pitch+kb*w)
										=(*(dstp+(bht-1-h)*pitch+kb*w))*shade/256;
								}

				}

				if(vi.IsYV12())
				{
					for(int w=0;w<bwd;w++)
					{
						ResizeVLineYV12( srcp+pitch*(bht/4)+w, 
										dstp+(pitch*dy)/2+w,
										dy,pitch,pitch,(double)(bht/2)/dy);
						ResizeVLineYV12( srcp+(bht-1-bht/4)*pitch+w, 
										dstp+pitch*(bht-1-dy/2)+w,
										dy,pitch,pitch,(double)(bht/2)/dy);
						if((w & 1) ==1)
							continue;
								// U plane
						ResizeVLineYV12( srcpU+spitchUV*(bht/4)/2+w/2, 
										dstpU+(dpitchUV*dy)/4+w/2,
										dy/2,spitchUV,dpitchUV,(double)(bht/2)/dy);
						ResizeVLineYV12( srcpU+(bht-1-bht/4)/2*spitchUV+w/2, 
										dstpU+dpitchUV*(bht-1-dy/2)/2+w/2,
										dy/2,spitchUV,dpitchUV,(double)(bht/2)/dy);
								// V plane
						ResizeVLineYV12( srcpV+spitchUV*(bht/4)/2+w/2, 
										dstpV+(dpitchUV*dy)/4+w/2,
										dy/2,spitchUV,dpitchUV,(double)(bht/2)/dy);
						ResizeVLineYV12( srcpV+(bht-1-bht/4)/2*spitchUV+w/2, 
										dstpV+dpitchUV*(bht-1-dy/2)/2+w/2,
										dy/2,spitchUV,dpitchUV,(double)(bht/2)/dy);
					}

					for(int f=0; f<nfoldh; f+=2)		// shade alternate segments
						for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
							for(int w=0;w<bwd;w++)
						
								{
									*(dstp+h*pitch+kb*w)
										=(*(dstp+h*pitch+kb*w))*shade/256;
									*(dstp+(bht-1-h)*pitch+kb*w)
										=(*(dstp+(bht-1-h)*pitch+kb*w))*shade/256;
								}

				}

				if(vi.IsRGB32() || vi.IsRGB24())
				

							// shade alternate sections to get fold effect
					for(int f=0; f<nfoldh; f+=2)
						for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
							for(int w=0;w<bwd;w++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+h*pitch+kb*w+k)
										=(*(dstp+h*pitch+kb*w+k))*shade/256;
									*(dstp+(bht-1-h)*pitch+kb*w+k)
										=(*(dstp+(bht-1-h)*pitch+kb*w+k))*shade/256;
								}
								// the following two loops are common toYV12 YUY2, RGB32 and RGB24
				for(int f=0; f<nfoldh; f+=2)	// blacken both sides ends for fold effect
					for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
						for(int w=0;w<taperw*((f+1)*dy/nfoldh-h);w++)
							for(int k=0;k<kb;k++)
							{
									*(dstp+(bht-1-h)*pitch+kb*w+k)=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w)+k)=0;
									*(dstp+h*pitch+kb*w+k)=0;
									*(dstp+h*pitch+kb*(bwd-1-w)+k)=0;
							}
				for( f=1; f<nfoldh; f+=2)
					for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
						for(int w=0;w<taperw*(h-(f)*dy/nfoldh);w++)
							for(int k=0;k<kb;k++)
							{
									*(dstp+(bht-1-h)*pitch+kb*(w)+k)=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w)+k)=0;
									*(dstp+h*pitch+kb*w+k)=0;
									*(dstp+h*pitch+kb*(bwd-1-w)+k)=0;
							}
			}



				
			if(!twin)
			{
				if(vi.IsRGB32())
				{
					for(int w=0;w<bwd;w++)
					{

						ResizeVLineRGB32( srcp+(bht/2)*pitch+kb*w, 
										dstp+(bht-1-dy/2)*pitch+kb*w,
										dy,pitch,pitch,(double)(bht)/dy);
					}
						
				}
				if(vi.IsRGB24())
				{
					for(int w=0;w<bwd;w++)
					{
						ResizeVLineRGB24( srcp+(bht/2)*pitch+kb*w, 
										dstp+(bht-1-dy/2)*pitch+kb*w,
										dy,pitch,pitch,(double)(bht)/dy);
					}
				}
				if(vi.IsRGB24() || vi.IsRGB32())
				{
							// shade alternate sections to get fold effect
					for(int f=0; f<nfoldh; f+=2)
						for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
							for(int w=0;w<bwd;w++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+(bht-1-h)*pitch+kb*w+k)
										=(*(dstp+(bht-1-h)*pitch+kb*w+k))* shade/256;
								}
					for(f=0; f<nfoldh; f+=2)
						for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
							for(int w=0;w<taperw*((f+1)*dy/nfoldh-h);w++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+(bht-1-h)*pitch+kb*w+k)=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w)+k)=0;
								}
					for( f=1; f<nfoldh; f+=2)
						for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
							for(int w=0;w<taperw*(h-(f)*dy/nfoldh);w++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+(bht-1-h)*pitch+kb*(w)+k)=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w)+k)=0;									
								}
				}
				if(vi.IsYUY2()|| vi.IsYV12())
				{
					if(vi.IsYUY2())
						for(int w=0;w<bwd;w+=2)
						{
							ResizeVLineYUY2( srcp+(bht/2)*pitch+kb*w, 
										dstp+dy*pitch/2+kb*w,
											dy,pitch,pitch,(double)(bht)/dy);

						}
					if(vi.IsYV12())
						for(int w=0;w<bwd;w++)
						{
							ResizeVLineYV12( srcp+(bht/2)*pitch+w, 
										dstp+dy*pitch/2+w,
											dy,pitch,pitch,(double)(bht)/dy);
							if((w& 1)==1)
								continue;
									// U plane
							ResizeVLineYV12( srcpU+(bhtUV/2)*spitchUV+w/2, 
										dstpU+dy*dpitchUV/4+w/2,
											dy/2,spitchUV,dpitchUV,(double)(bht)/dy);
									// V plane
							ResizeVLineYV12( srcpV+(bhtUV/2)*spitchUV+w/2, 
										dstpV+dy*dpitchUV/4+w/2,
											dy/2,spitchUV,dpitchUV,(double)(bht)/dy);

						}
							// shade alternate sections to get fold effect
						for(int f=0; f<nfoldh; f+=2)
							for(int h=f*dy/nfoldh;h<(f+1)*dy/nfoldh;h++)
								for(int w=0;w<bwd;w++)
								{
									*(dstp+h*pitch+kb*w)
										=(*(dstp+h*pitch+kb*w))*shade/256;
								}
				}

			
			}
		}

		if(*dir=='v' || *dir =='V')
		{
			if(twin)
			{
				if(vi.IsRGB32())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineRGB32(srcp+h*pitch+kb*bwd/4,
											dstp+h*pitch+kb*dx/2,
											dx,(double)bwd/(2*dx));
						ResizeHLineRGB32(srcp+h*pitch+kb*3*bwd/4,
											dstp+h*pitch+kb*(bwd-1-dx/2),
											dx,(double)bwd/(2*dx));
					}
				if(vi.IsRGB24())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineRGB24(srcp+h*pitch+kb*bwd/4,
											dstp+h*pitch+kb*dx/2,
											dx,(double)bwd/(2*dx));
						ResizeHLineRGB24(srcp+h*pitch+kb*3*bwd/4,
											dstp+h*pitch+kb*(bwd-1-dx/2),
											dx,(double)bwd/(2*dx));
					}
				if(vi.IsYUY2())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineYUY2(srcp+h*pitch+kb*bwd/4,
											dstp+h*pitch+kb*dx/2,
											dx,(double)bwd/(2*dx));
						ResizeHLineYUY2(srcp+h*pitch+kb*3*bwd/4,
											dstp+h*pitch+kb*(bwd-2-dx/2),
											dx,(double)bwd/(2*dx));
					}
				if(vi.IsYV12())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineYV12(srcp+h*pitch+kb*bwd/4,
											dstp+h*pitch+kb*dx/2,
											dx,(double)bwd/(2*dx));
						ResizeHLineYV12(srcp+h*pitch+kb*3*bwd/4,
											dstp+h*pitch+kb*(bwd-2-dx/2),
											dx,(double)bwd/(2*dx));
						if((h& 1)==1)
							continue;
								//U plane
						ResizeHLineYV12(srcpU+h/2*spitchUV+bwdUV/4,
											dstpU+h/2*dpitchUV+dx/4,
											dx/2,(double)bwd/(2*dx));
						ResizeHLineYV12(srcpU+h/2*spitchUV+3*bwdUV/4,
											dstpU+h/2*dpitchUV+(bwd-2-dx/2)/2,
											dx/2,(double)bwd/(2*dx));
								//U plane
						ResizeHLineYV12(srcpV+h/2*spitchUV+bwdUV/4,
											dstpV+h/2*dpitchUV+dx/4,
											dx/2,(double)bwd/(2*dx));
						ResizeHLineYV12(srcpV+h/2*spitchUV+3*bwdUV/4,
											dstpV+h/2*dpitchUV+(bwd-2-dx/2)/2,
											dx/2,(double)bwd/(2*dx));
					}
				if(vi.IsRGB32() || vi.IsRGB24())
				{
				

							// shade alternate sections to get fold effect
					for(int f=0; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<bht;h++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+h*pitch+kb*w+k)
										=(*(dstp+h*pitch+kb*w+k))*shade/256;
									*(dstp+h*pitch+kb*(bwd-1-w)+k)
										=(*(dstp+h*pitch+kb*(bwd-1-w)+k))*shade/256;
								}
					for( f=0; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<taperw*((f+1)*dx/nfoldw-w);h++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+(bht-1-h)*pitch+kb*w+k)=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w)+k)=0;
									*(dstp+h*pitch+kb*w+k)=0;
									*(dstp+h*pitch+kb*(bwd-1-w)+k)=0;
								}
					for( f=1; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<taperw*(w-(f)*dx/nfoldw);h++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+(bht-1-h)*pitch+kb*(w)+k)=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w)+k)=0;
									*(dstp+h*pitch+kb*w+k)=0;
									*(dstp+h*pitch+kb*(bwd-1-w)+k)=0;
								}
				}
				if(vi.IsYUY2()  || vi.IsYV12())
				{
					for(int f=0; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<bht;h++)
								{
									*(dstp+h*pitch+kb*w)
										=(*(dstp+h*pitch+kb*w))*shade/256;
									*(dstp+h*pitch+kb*(bwd-1-w))
										=(*(dstp+h*pitch+kb*(bwd-1-w)))*shade/256;
								}
					for( f=0; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<taperw*((f+1)*dx/nfoldw-w);h++)
								{
									*(dstp+(bht-1-h)*pitch+kb*w)=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w))=0;
									*(dstp+h*pitch+kb*w)=0;
									*(dstp+h*pitch+kb*(bwd-1-w))=0;
								}
					for( f=1; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<taperw*(w-(f)*dx/nfoldw);h++)
								{
									*(dstp+(bht-1-h)*pitch+kb*(w))=0;
									*(dstp+(bht-1-h)*pitch+kb*(bwd-1-w))=0;
									*(dstp+h*pitch+kb*w)=0;
									*(dstp+h*pitch+kb*(bwd-1-w))=0;
								}
				}
			}
			if(!twin)
			{
				if(vi.IsRGB32())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineRGB32(srcp+h*pitch+kb*bwd/2,
											dstp+h*pitch+kb*dx/2,
											dx,(double)bwd/dx);
					}
				if(vi.IsRGB24())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineRGB24(srcp+h*pitch+kb*bwd/2,
											dstp+h*pitch+kb*dx/2,
											dx,(double)bwd/dx);
					}
				if(vi.IsYUY2())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineYUY2(srcp+h*pitch+kb*bwd/2,
											dstp+h*pitch+kb*dx/2,
											dx,(double)bwd/dx);
					}
				if(vi.IsYV12())
					for(int h=0; h<bht;h++)
					{
						ResizeHLineYV12(srcp+h*pitch+bwd/2,
											dstp+h*pitch+dx/2,
											dx,(double)bwd/dx);
						if((h & 1)==1)
							continue;
								// U plane
						ResizeHLineYV12(srcpU+h/2*spitchUV+bwdUV/2,
											dstpU+h/2*dpitchUV+dx/4,
											dx/2,(double)bwd/dx);
								// V plane
						ResizeHLineYV12(srcpV+h/2*spitchUV+bwdUV/2,
											dstpV+h/2*dpitchUV+dx/4,
											dx/2,(double)bwd/dx);
					}
				if(vi.IsRGB32() || vi.IsRGB24())
				{
				

							// shade alternate sections to get fold effect
					for(int f=0; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<bht;h++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+h*pitch+kb*w+k)
										=(*(dstp+h*pitch+kb*w+k))*shade/256;

								}
					for(f=0; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<taperw*((f+1)*dx/nfoldw-w);h++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+(bht-1-h)*pitch+kb*w+k)=0;
									*(dstp+h*pitch+kb*w+k)=0;
								}
					for( f=1; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<taperw*(w-(f)*dx/nfoldw);h++)
								for(int k=0;k<kb;k++)
								{
									*(dstp+(bht-1-h)*pitch+kb*(w)+k)=0;
									*(dstp+h*pitch+kb*w+k)=0;
								}
				}
				if(vi.IsYUY2() || vi.IsYV12())
				{
					for(int f=0; f<nfoldw; f+=2)
						for(int w=f*dx/nfoldw;w<(f+1)*dx/nfoldw;w++)
							for(int h=0;h<bht;h++)
								{
									*(dstp+h*pitch+kb*w)
										=(*(dstp+h*pitch+kb*w))*shade/256;
								}
				}
			}
		}
			



		if(open)
										
			return RightFrame;
		return LeftFrame;

		
	
}

 
void TransAccord::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransAccord::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransAccord(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransAccord(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),		// Clip as RightClip
						args[2].AsInt(),		// overlap frames
 						args[3].AsString("vert"),		// Door vertical or Horizontal "vert" "hor"
						args[4].AsBool(true),	// False Single or true for split doors
						args[5].AsBool(true),	// true open, false close
																							
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransAccord", "ccis[twin]b[open]b", Create_TransAccord, 0);
					
    return "TransAccord  Plugin";
}

*/