
/******************************************************************************
The TransBubbles Transition creates Bubbles from Right Frame fragments 
and floats them on to Left Frame randomly and bursts the Bubbles on landing
started 27-1-05 completed 29-1-05
*************************************************************************************************/  
#define nmaxBubbles 16 
class TransBubbles : public GenericVideoFilter 
{
		PClip RightClip;
		bool stat;
		int  rad;
		int nBubbles;
//		bool drop;

		int overlap;
		int nthbbl[nmaxBubbles*nmaxBubbles+1];
		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransBubbles(PClip _child, PClip _RightClip,
				int _overlap,bool _static,  
			IScriptEnvironment* env) ;	
			
 	~TransBubbles();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/


TransBubbles::TransBubbles(PClip _child, PClip _RightClip,
						   int _overlap,bool _static,  IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
				overlap(_overlap), stat(_static)  
			
{
		  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	char * Tname="TransBubbles";	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransBubbles:Clips have differing pixel_types");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2()&& !vi.IsYV12())
		env->ThrowError("TransBubbles: This color format is not supported here");
	if (vi.width != rvi.width || vi.height != rvi.height)
		env->ThrowError("%s:Clips have differing frame dimensions", Tname);

	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);	
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 


	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
	rad=vi.width>vi.height?vi.width/(2*nmaxBubbles):vi.height/(2*nmaxBubbles);
	rad=rad<8?8:rad;
	nBubbles=(vi.width/(2*rad))*(vi.height/(2*rad));
	//get non repeating number 0 to nbubbles-1
	for(int i=0;i<nBubbles; i++)
	{
		nthbbl[i]=rand()%nBubbles;
		for (int k=0;k<i;k++)			
			if(nthbbl[k]==nthbbl[i])
					i--;
	
	}
	nthbbl[nBubbles]=nthbbl[0];	// for getting last movement correctly


}
// This is where any actual destructor code used goes
TransBubbles::~TransBubbles() {
	
// This is where you can deallocate any memory you might have used.
}
//_____________________________________________________________________
void TransBubbles::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransBubbles::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


	
/***************************************************************/	
	PVideoFrame __stdcall TransBubbles::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		int nframes=video_fade_end-video_fade_start+1;
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		PVideoFrame Frame;
		
		const int bwd = vi.width; 
        const int bht = LeftFrame->GetHeight();	
		const int lpitch = LeftFrame->GetPitch();	
		const int rpitch = RightFrame->GetPitch();
		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	
		const int bhtUV = LeftFrame->GetHeight(PLANAR_U);
		
	
		int mobile=stat?0:1;
		unsigned char* dstp, *dstpU, *dstpV;
		const unsigned char * srcp, *srcpU, *srcpV;
		env->MakeWritable(&LeftFrame);
			dstp= LeftFrame->GetWritePtr();
			srcp= RightFrame->GetReadPtr();
			if(vi.IsYV12())
			{
				dstpU = LeftFrame->GetWritePtr(PLANAR_U);
				dstpV = LeftFrame->GetWritePtr(PLANAR_V);
				srcpU = RightFrame->GetReadPtr(PLANAR_U);
				srcpV = RightFrame->GetReadPtr(PLANAR_V);
			}

		
		double mag1=4.0;
		int kb= vi.BitsPerPixel()/8;
		// number of bubbles along horizontal and vertical ;
		int hbbl=bwd/(2*rad);
		int vbbl=bht/(2*rad);

		double bubblesPerFrame=(double)nBubbles/(7*nframes/8);
		int nthfull=n>nframes/8?(n-nframes/8)*bubblesPerFrame:0;	// Bubbled some time back, formed squares
			// number of bubbles present in any frame
		int nexpanding=(n<nframes/8?n:n>(7*nframes)/8?nframes-n:nframes/8)*bubblesPerFrame;
		int maxbubbles=(nframes/8)*bubblesPerFrame;	// max number of bubbles alive in a frame
			// draw the squares for already burst bubbles
		for(int i=0;i<nthfull;i++)
			for(int h=0;h<2*rad;h++)
				for(int w=0;w<2*rad;w++)
					for(int k=0;k<kb;k++)
					{

						*(dstp+(nthbbl[i]/hbbl*2*rad+h)*lpitch+(nthbbl[i]%hbbl)*2*rad*kb+w*kb+k)
						=*(srcp+(nthbbl[i]/hbbl*2*rad+h)*rpitch+(nthbbl[i]%hbbl)*2*rad*kb+w*kb+k);

					}
		if(vi.IsYV12())
		{
			for(int i=0;i<nthfull;i++)
				for(int h=0;h<rad;h++)
					for(int w=0;w<rad;w++)
						
					{

						*(dstpU+(nthbbl[i]/hbbl*rad+h)*lpitchUV+(nthbbl[i]%hbbl)*rad+w)
							=*(srcpU+(nthbbl[i]/hbbl*rad+h)*rpitchUV+(nthbbl[i]%hbbl)*rad+w);
						*(dstpV+(nthbbl[i]/hbbl*rad+h)*lpitchUV+(nthbbl[i]%hbbl)*rad+w)
							=*(srcpV+(nthbbl[i]/hbbl*rad+h)*rpitchUV+(nthbbl[i]%hbbl)*rad+w);

					}
		}
		for(i=0;i<nexpanding;i++)
		{
				// bubbles while forming grow and move to destination
						//destination and source coordinates
						
			int xcoord0=rad+(nthbbl[nthfull+i]%hbbl)*2*rad;
			int ycoord0=rad+(nthbbl[nthfull+i]/hbbl)*2*rad;
						// start appearance coordinates
						// mobile=0 if static opted otherwise 1
			int xcoord1=rad+(nthbbl[nthfull+i+mobile]%hbbl)*2*rad;
			int ycoord1=rad+(nthbbl[nthfull+i+mobile]/hbbl)*2*rad;
						// current coordinates
			int xcoord=xcoord0-((xcoord0-xcoord1)*i)/maxbubbles;
			int ycoord=ycoord0-((ycoord0-ycoord1)*i)/maxbubbles;
			int radius=((maxbubbles-i)*rad)/maxbubbles;
			if(n<nframes/8)
			{
				radius=(n*radius)/(nframes/8);
				xcoord=xcoord1+((xcoord0-xcoord1)*n)/(nframes/8);
				ycoord=ycoord1+((ycoord0-ycoord1)*n)/(nframes/8);
			}
			if(vi.IsRGB32())

				FastDiscoRGB32( srcp, dstp, radius,
								xcoord0, bht-1-ycoord0, bht, bwd,
								xcoord, bht-1-ycoord, bht, bwd,
								rpitch,lpitch, mag1);
			
		
			if(vi.IsRGB24())
				FastDiscoRGB24( srcp, dstp, radius,
								xcoord0, bht-1-ycoord0, bht, bwd,
								xcoord, bht-1-ycoord, bht, bwd,
								rpitch,lpitch, mag1);
		
			if(vi.IsYUY2())
				FastDiscoYUY2( srcp, dstp, radius,
								xcoord0, ycoord0, bht, bwd,
								xcoord, ycoord, bht, bwd,
								rpitch,lpitch, mag1);

			if(vi.IsYV12())
			{
				FastDiscoYV12( srcp, dstp, radius,
								xcoord0, ycoord0, bht, bwd,
								xcoord, ycoord, bht, bwd,
								rpitch,lpitch, mag1);
				FastDiscoYV12( srcpU, dstpU, radius/2,
								xcoord0/2, ycoord0/2, bhtUV, bwdUV,
								xcoord/2, ycoord/2, bhtUV, bwdUV,
								rpitchUV,lpitchUV, mag1);
				FastDiscoYV12( srcpV, dstpV, radius/2,
								xcoord0/2, ycoord0/2, bhtUV, bwdUV,
								xcoord/2, ycoord/2, bhtUV, bwdUV,
								rpitchUV,lpitchUV, mag1);

			
			}
		}
		
		return LeftFrame;

		
	}

/**************
DiscoRGB32(const char* srcp,  char* dstp, int radius, int srcx, int srcy, int srch, int srcw,
		int dstx, int dsty, int dsth, int dstw, int spitch, int dpitch, bool mag, bool drop);
DiscoRGB24(const char* srcp,  char* dstp, int radius, int srcx, int srcy, int srch, int srcw,
		int dstx, int dsty, int dsth, int dstw, int spitch, int dpitch, bool mag, bool drop);
***********************/
  	 
/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransBubbles(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
/******************************************************************************************/

/**********************************************************************************************/
	
		

	return new TransBubbles(args[0].AsClip(),	// clip as LeftClip
							args[1].AsClip(),	// clip as RightClip
							args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
							args[3].AsBool(false),	//static bubbles?
						//	args[4].AsInt(),	// test index
							env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransBubbles", "cci[static]b", Create_TransBubbles, 0);
				
    return "TransBubbles Transition TransBubbles Plugin";
}

*/