/******************************************************************************
TransCentral filter plugin for Avisynth by V.C.Mohan
This RightClip  emerging from center gradually Masks  a stationary LeftClip. While emerging
options are for resizing to fit, rotating n turns.
Alternately the LeftClip disappears in to the center of  RightFrame. While disappearing 
options are for resizing to fit, and rotate n times .
 The clips should have identical lengths, height, width and same pixel type.
 Only RGB24, RGB32 and YUY2 formats are supported. YUY2 while combining resize
 and rotation may distort colors.
*************************************************************************************************/  

class TransCentral : public GenericVideoFilter {
		PClip RightClip;
		const bool emerge;
		const bool resize;
		const int nturns;
		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		 
  public:
							//Definition of function
    TransCentral(PClip _child, PClip _RightClip, int _overlap,
				const bool _emerge, const bool _resize, const int _nturns,  
				IScriptEnvironment* env) ;	
			
 	~TransCentral();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
	
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransCentral::TransCentral(PClip _child, PClip _RightClip,int _overlap,
						   const bool _emerge, const bool _resize, const int _nturns,
						   IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap(_overlap),
						emerge(_emerge), resize(_resize), nturns(_nturns) 
			
{
char * Tname="TransCentral";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransCentral:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransCentral: This color format is not supported here");
if (vi.width != rvi.width || vi.height != rvi.height)
	env->ThrowError("%s:Clips have differing frame dimensions", Tname);	
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;

}
// This is where any actual destructor code used goes
TransCentral::~TransCentral() {
	
// This is where you can deallocate any memory you might have used.
}
//_______________________________________________________________
void TransCentral::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}

bool TransCentral::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}

//______________________________________________________________________________________________
	
/***************************************************************/	
	PVideoFrame __stdcall TransCentral::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
			return child->GetFrame(en, env);
		if (en > video_fade_end)
			return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		PVideoFrame dst,src,work,work2;
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		
		const	int bwd = lvi.width; 
        const	int bht = LeftFrame->GetHeight();	
		int deltay =((n+1)*bht)/(overlap+1);
		int deltax = ((n+1)*bwd)/(overlap+1);
		const	int pitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		if(!(pitch==rpitch))
			env->ThrowError("TransCentral: The two clips have unequal pitches");
		deltax= deltax & 0xFFFFFFFC;
		deltay= deltay & 0xFFFFFFFC;
		int dstx,dsty, wht,wwd,srcx,srcy ;
		int degree= (((n+1)*nturns*360)/(overlap+1))%360;
		unsigned char* dstp, *dstpU,*dstpV;
		const unsigned char* srcp, *srcpU,*srcpV;
		const	int pitchUV = LeftFrame->GetPitch(PLANAR_U);
		
		env->MakeWritable(&RightFrame);
		
		if(emerge)

		{
					// Right emerges from center of LeftFrame in all directions
			
			dstx =(bwd-deltax)/2;
			dsty =(bht- deltay)/2;
			wht = deltay;
			wwd = deltax;
			srcx=(bwd-deltax)/2;
			srcy=(bht-deltay)/2;
			src=RightFrame;
			dst=LeftFrame;
			
				
		}
/**
bool FastDiscoRotateRGB32(const unsigned char *srcp,unsigned char*dstp,
					  int radius,int srcx,int srcy,int srch,int srcw,
					  int dstx,int dsty,int dsth,int dstw,int spitch, int dpitch,int degree);
inline void CopyHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw);
inline void CopyVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
// ResizeHLineRGB32 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw, double factor);
// ResizeVLineRGB32 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
*/
		if(!emerge)
			
		{	//LeftFrame disappears into center of RightFrame
			dstx =(deltax)/2;
			dsty =(deltay)/2;
			wht =bht-deltay;
			wwd =bwd-deltax;
			srcx= (deltax)/2;
			srcy= (deltay)/2;
			src=LeftFrame;
			dst=RightFrame;
		}
		srcp=src->GetReadPtr();
		env->MakeWritable(&dst);
		dstp= dst->GetWritePtr();
		if(vi.IsYV12())
		{
			srcpU=src->GetReadPtr(PLANAR_U);
			srcpV=src->GetReadPtr(PLANAR_V);
		
			dstpU= dst->GetWritePtr(PLANAR_U);
			dstpV= dst->GetWritePtr(PLANAR_V);
		}



			
		if (wwd==0 || wht==0)
			return LeftFrame; // no work to be done
		double wfactor=(double)(bwd)/(double)(wwd);
		double hfactor=(double)(bht)/(double)(wht);
		int radius=sqrt(wht*wht+wwd*wwd);
		radius=radius & 0xFFFFFFFC;
		if(!resize)
		{
			if(nturns==0)
			{
			if(lvi.IsRGB24())					
				for (int h=dsty; h<dsty+wht; h++)
					CopyHLineRGB24( srcp+h*pitch+3*dstx,dstp+h*pitch+3*dstx,wwd);
								
			if(lvi.IsRGB32())
				for (int h=dsty; h<dsty+wht; h++)
					CopyHLineRGB32( srcp+h*pitch+4*dstx,dstp+h*pitch+4*dstx,wwd);

			if(lvi.IsYUY2())
				for (int h=dsty; h<dsty+wht; h++)
					CopyHLineYUY2( srcp+h*pitch+2*dstx,dstp+h*pitch+2*dstx,wwd);
			if(lvi.IsYV12())
				for (int h=dsty; h<dsty+wht; h++)
				{
					CopyHLineYV12( srcp+h*pitch+dstx,dstp+h*pitch+dstx,wwd);
					if((h & 1)==1)
						continue;
					CopyHLineYV12( srcpU+h/2*pitchUV+dstx/2,dstpU+h/2*pitchUV+dstx/2,wwd/2);
					CopyHLineYV12( srcpV+h/2*pitchUV+dstx/2,dstpV+h/2*pitchUV+dstx/2,wwd/2);
				}


			}
			if(nturns!=0)
			{
				if(lvi.IsRGB24())
					FastDiscoRotateRGB24(srcp+srcy*pitch+3*srcx,dstp+dsty*pitch+3*dstx,
										radius,wwd/2,wht/2,wht,wwd,
										wwd/2,wht/2,wht,wwd,pitch, pitch,degree);

				if(lvi.IsRGB32())
					FastDiscoRotateRGB32(srcp+srcy*pitch+4*srcx,dstp+dsty*pitch+4*dstx,
										radius,wwd/2,wht/2,wht,wwd,
										wwd/2,wht/2,wht,wwd,pitch, pitch,degree);
				if(lvi.IsYUY2())
					FastDiscoRotateYUY2(srcp+srcy*pitch+2*srcx,dstp+dsty*pitch+2*dstx,
										radius,wwd/2,wht/2,wht,wwd,
										wwd/2,wht/2,wht,wwd,pitch, pitch,degree);
				if(lvi.IsYV12())
				{
					FastDiscoRotateYV12(srcp+srcy*pitch+srcx,dstp+dsty*pitch+dstx,
										radius,wwd/2,wht/2,wht,wwd,
										wwd/2,wht/2,wht,wwd,pitch, pitch,degree);
					FastDiscoRotateYV12(srcpU+srcy/2*pitchUV+srcx/2,dstpU+dsty/2*pitchUV+dstx/2,
										radius/2,wwd/4,wht/4,wht/2,wwd/2,
										wwd/4,wht/4,wht/2,wwd/2,pitchUV, pitchUV,degree);
					FastDiscoRotateYV12(srcpV+srcy/2*pitchUV+srcx/2,dstpV+dsty/2*pitchUV+dstx/2,
										radius/2,wwd/4,wht/4,wht/2,wwd/2,
										wwd/4,wht/4,wht/2,wwd/2,pitchUV, pitchUV,degree);
				}
			}
		}
		if(resize)
		{
			work=env-> NewVideoFrame(vi);
			unsigned char* wp= work->GetWritePtr();
			unsigned char* wpU= work->GetWritePtr(PLANAR_U);
			unsigned char* wpV= work->GetWritePtr(PLANAR_V);

			if(nturns==0)
			{
				if(lvi.IsRGB24())
				{
					for (int h=0; h<bht; h++)
					// ResizeHLineRGB32 requires the srcp and dstp to point to middle of window winw/2.
					//The factor is frame width/winw
					//inline void ResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp,
					//							int winw, double factor);
						ResizeHLineRGB24( srcp+h*pitch+3*bwd/2, wp+h*pitch+3*bwd/2,wwd,wfactor);
					for(int w=dstx; w<dstx+wwd;w++)
						ResizeVLineRGB24(wp+bht*pitch/2+3*w, dstp+bht*pitch/2+3*w,
								wht,pitch,pitch,hfactor);
				}


								
				if(lvi.IsRGB32())
				{
					for (int h=0; h<bht; h++)
						ResizeHLineRGB32( srcp+h*pitch+4*bwd/2,wp+h*pitch+4*bwd/2,wwd,wfactor);
					for(int w=dstx; w<dstx+wwd;w++)
						ResizeVLineRGB32(wp+bht*pitch/2+4*w,dstp+bht*pitch/2+4*w,
								wht,pitch,pitch,hfactor);
				}
				if(lvi.IsYUY2())
				{
					for (int h=0; h<bht; h++)
						ResizeHLineYUY2( srcp+h*pitch+2*bwd/2, wp+h*pitch+2*bwd/2,wwd,wfactor);
					for(int w=dstx; w<dstx+wwd;w+=2)
						ResizeVLineYUY2(wp+bht*pitch/2+2*w,dstp+bht*pitch/2+2*w,
								wht,pitch,pitch,hfactor);
				}
				if(lvi.IsYV12())
				{
					for (int h=0; h<bht; h++)
					{
						ResizeHLineYV12( srcp+h*pitch+bwd/2, wp+h*pitch+bwd/2,wwd,wfactor);
						if((h & 1)==1)
							continue;
						ResizeHLineYV12( srcpU+h/2*pitchUV+bwd/4, wpU+h/2*pitchUV+bwd/4,wwd/2,wfactor);
						ResizeHLineYV12( srcpV+h/2*pitchUV+bwd/4, wpV+h/2*pitchUV+bwd/4,wwd/2,wfactor);
					}

					for(int w=dstx; w<dstx+wwd;w++)
					{
						ResizeVLineYV12(wp+bht*pitch/2+w,dstp+bht*pitch/2+w,
								wht,pitch,pitch,hfactor);
						if((w & 1)==1)
							continue;
						ResizeVLineYV12(wpU+bht/2*pitchUV/2+w/2,dstpU+bht/2*pitchUV/2+w/2,
								wht/2,pitchUV,pitchUV,hfactor);
						ResizeVLineYV12(wpV+bht/2*pitchUV/2+w/2,dstpV+bht/2*pitchUV/2+w/2,
								wht/2,pitchUV,pitchUV,hfactor);
					}

				}
			}
			if(nturns!=0)
			{
				work2=env->NewVideoFrame(vi);
				unsigned char* w2p= work2->GetWritePtr();
				unsigned char* w2pU= work2->GetWritePtr(PLANAR_U);
				unsigned char* w2pV= work2->GetWritePtr(PLANAR_V);

				if(lvi.IsRGB24())
				{
					for (int h=0; h<bht; h++)
						ResizeHLineRGB24( srcp+h*pitch+3*bwd/2, wp+h*pitch+3*bwd/2,wwd,wfactor);
					for(int w=dstx; w<dstx+wwd;w++)
						ResizeVLineRGB24(wp+bht*pitch/2+3*w, w2p+bht*pitch/2+3*w,
								wht,pitch,pitch,hfactor);
					FastDiscoRotateRGB24(w2p+srcy*pitch+3*srcx,dstp+dsty*pitch+3*dstx,
					  radius,wwd/2,wht/2,wht,wwd,
					  wwd/2,wht/2,wht,wwd,pitch, pitch,degree);
				}

				if(lvi.IsRGB32())
				{
					for (int h=0; h<bht; h++)
						ResizeHLineRGB32( srcp+h*pitch+4*bwd/2, wp+h*pitch+4*bwd/2,wwd,wfactor);
					for(int w=dstx; w<dstx+wwd;w++)
						ResizeVLineRGB32(wp+bht*pitch/2+4*w, w2p+bht*pitch/2+4*w,
								wht,pitch,pitch,hfactor);
					FastDiscoRotateRGB32(w2p+srcy*pitch+4*srcx,dstp+dsty*pitch+4*dstx,
					  radius,wwd/2,wht/2,wht,wwd,
					  wwd/2,wht/2,wht,wwd,pitch, pitch,degree);
				}
				if(lvi.IsYUY2())
				{
					for (int h=0; h<bht; h++)
						ResizeHLineYUY2( srcp+h*pitch+2*bwd/2, wp+h*pitch+2*bwd/2,wwd,wfactor);
					for(int w=dstx; w<dstx+wwd;w+=2)
						ResizeVLineYUY2(wp+bht*pitch/2+2*w, w2p+bht*pitch/2+2*w,
								wht,pitch,pitch,hfactor);
					FastDiscoRotateYUY2(w2p+srcy*pitch+2*srcx,dstp+dsty*pitch+2*dstx,
					  radius,wwd/2,wht/2,wht,wwd,
					  wwd/2,wht/2,wht,wwd,pitch, pitch,degree);
				}
				if(lvi.IsYV12())
				{
					for (int h=0; h<bht; h++)
					{
						ResizeHLineYV12( srcp+h*pitch+bwd/2, wp+h*pitch+bwd/2,wwd,wfactor);
						if((h & 1)==1)
							continue;
						ResizeHLineYV12( srcpU+h/2*pitchUV+bwd/4, wpU+h/2*pitchUV+bwd/4,wwd/2,wfactor);
						ResizeHLineYV12( srcpV+h/2*pitchUV+bwd/4, wpV+h/2*pitchUV+bwd/4,wwd/2,wfactor);
					}

					for(int w=dstx; w<dstx+wwd;w++)
					{
						ResizeVLineYV12(wp+bht*pitch/2+w, w2p+bht*pitch/2+w,
								wht,pitch,pitch,hfactor);
						if((w & 1)==1)
							continue;
						ResizeVLineYV12(wpU+bht/2*pitchUV/2+w/2, w2pU+bht/2*pitchUV/2+w/2,
								wht/2,pitchUV,pitchUV,hfactor);
						ResizeVLineYV12(wpV+bht/2*pitchUV/2+w/2, w2pV+bht/2*pitchUV/2+w/2,
								wht/2,pitchUV,pitchUV,hfactor);
					}
					FastDiscoRotateYV12(w2p+srcy*pitch+srcx,dstp+dsty*pitch+dstx,
					  radius,wwd/2,wht/2,wht,wwd,
					  wwd/2,wht/2,wht,wwd,pitch, pitch,degree);
					FastDiscoRotateYV12(w2pU+srcy/2*pitchUV+srcx/2,dstpU+dsty/2*pitchUV+dstx/2,
					  radius/2,wwd/4,wht/4,wht/2,wwd/2,
					  wwd/4,wht/4,wht/2,wwd/2,pitchUV, pitchUV,degree);
					FastDiscoRotateYV12(w2pV+srcy/2*pitchUV+srcx/2,dstpV+dsty/2*pitchUV+dstx/2,
					  radius/2,wwd/4,wht/4,wht/2,wwd/2,
					  wwd/4,wht/4,wht/2,wwd/2,pitchUV, pitchUV,degree);
				}
			}
		}
		
		return dst;
	}


			
//*********************************************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransCentral(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	

				
	return new TransCentral(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),		// Clip as RightClip
						args[2].AsInt(),		// overlap of clips
 						args[3].AsBool("true"),	// Transition true: Emerge, false:Disappear
						args[4].AsBool("true"), // true: the emerging/disappearing frame is resized for full view
												// false : The emerging/disappearing frame is cropped
						args[5].AsInt( 0),		// the emerging/ disappearing frame rotates by number of times. -ve
												// number reverses rotation.
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransCentral", "cci[emerge]b[resize]b[revolve]i", Create_TransCentral, 0);
				
    return "TransCentral Transition Mask Plugin";
}

*/