/******************************************************************************
TransCrumple filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip folding or crumpling.or
The Right Clip masks  the Left clip by uncrumpling .
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sampleModified on 21-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "FastResizer.cpp"
#include "LineZ.cpp"
*/
class TransCrumple : public GenericVideoFilter {
		PClip RightClip;
		const char* type;
		const bool crumple;	// true left crumples, false right uncrumples

		int overlap;

		void * abuf;
//		bool first,laudio,raudio;
		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		
  public:
							//Definition of function
    TransCrumple(PClip _child, PClip _RightClip,int _overlap,
				const char* _type, const bool _crump,  
			IScriptEnvironment* env) ;	
			
 	~TransCrumple();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
	
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransCrumple::TransCrumple(PClip _child, PClip _RightClip,int _overlap, 
						 const char* _type, const bool _crump,
						 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap(_overlap),
		type(_type),crumple(_crump) 
			
{
char * Tname="TransCrumple";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransCrumple:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransCrumple: This color format is not yet supported ");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("TransCrumple: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransCrumple: Height/width of frame are not multiple of 4");

if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;

}
// This is where any actual destructor code used goes
TransCrumple::~TransCrumple() {
	
// This is where you can deallocate any memory you might have used.
}
//_______________________________________________________________
void TransCrumple::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}

bool TransCrumple::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


/***************************************************************/	
		PVideoFrame __stdcall TransCrumple::GetFrame(int en, IScriptEnvironment* env)
		{
			if (en < video_fade_start)
				return child->GetFrame(en, env);
			if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
			int n=en-video_fade_start;
		
			PVideoFrame LeftFrame = child->GetFrame(en, env);
			PVideoFrame RightFrame = RightClip->GetFrame(n, env);
			
			
			double deltax,deltay,winw, winh;
			const	int bwd = vi.width;
			const	int bht = LeftFrame->GetHeight();
			int taperx,taperw;
			deltax=((double)(n+1)*bwd)/((overlap+1)*4.0);
			deltay=((double)(n+1)*bht)/((overlap+1)*4.0);
			int dx=((int)deltax)& 0xfffffffc;
			int dy=((int)deltay) & 0xfffffffe;

			const	int pitch = LeftFrame->GetPitch();
			const	int pitchUV = LeftFrame->GetPitch(PLANAR_U);
				
			const	int dpitch = RightFrame->GetPitch();
			const	int dpitchUV = RightFrame->GetPitch(PLANAR_U);
			
			if(!(dpitch==pitch))
				env->ThrowError("TransCrumple:unequal pitches");
			const unsigned char *srcp,*srcpU,*srcpV;
			unsigned char * dstp, *dstpU, *dstpV, *wp, *wpU, *wpV, *w2p, *w2pU, *w2pV;
			if(!crumple)
			{
				srcp=RightFrame->GetReadPtr();
				
				env->MakeWritable(&LeftFrame);
				dstp=LeftFrame->GetWritePtr();
				if(vi.IsYV12())
				{
					srcpU=RightFrame->GetReadPtr(PLANAR_U);
					srcpV=RightFrame->GetReadPtr(PLANAR_V);
					dstpU=LeftFrame->GetWritePtr(PLANAR_U);
					dstpV=LeftFrame->GetWritePtr(PLANAR_V);
				}
				dx=bwd/4-dx;
				dy=bht/4-dy;
				
			}
			if(crumple)
			{
				srcp=LeftFrame->GetReadPtr();
				
				env->MakeWritable(&RightFrame);
				dstp=RightFrame->GetWritePtr();
				if(vi.IsYV12())
				{
					srcpU=LeftFrame->GetReadPtr(PLANAR_U);
					srcpV=LeftFrame->GetReadPtr(PLANAR_V);
					
					dstpU=RightFrame->GetWritePtr(PLANAR_U);
					dstpV=RightFrame->GetWritePtr(PLANAR_V);
				}
			}
			int vwin=bht-2*dy;
			int wwin=bwd-2*dx;

			const int kb=vi.BitsPerPixel()/8;
			
			if(*type=='c' || *type=='C')
			{
				PVideoFrame work= env->NewVideoFrame(vi);
				wp= work->GetWritePtr();
				const int wpitch=work->GetPitch();
			
				PVideoFrame work2= env->NewVideoFrame(vi);
				unsigned char * w2p= work2->GetWritePtr();
				const int w2pitch=work2->GetPitch();

				if(vi.IsRGB32())
				{// Resize the right frame to the correct position on to work2
					VFastResizerRGB32(srcp,wp,bht,bwd,vwin,bwd,pitch,wpitch,false);
					HFastResizerRGB32(wp,w2p+w2pitch*dy+kb*dx,vwin,bwd,vwin,wwin,wpitch,w2pitch,false);
					
						// resize along the contracting window centers. The reduction of 
						// window at final frame for crumple is equal to dx and dy along w and h
						// calculation of window centers and window sizes
					for(int h=dy;h<bht/2;h++)
					{
						double iwin=(double)(h-dy)*(bwd/2-dx)/(bht/2-dy);
						double owin=(double)(h-dy)*(bwd/2-2*dx)/(bht/2-dy);
						int wcenter=dx+iwin/2;
						int owcenter=dx+(double)(h-dy)*dx/(bht/2-dy)+owin/2;
						if(owin>0)
						{
							ResizeHLineRGB32(w2p+w2pitch*(h)+kb*wcenter,
									dstp+dpitch*(h)+kb*owcenter,
									owin,iwin/owin);
							ResizeHLineRGB32(w2p+w2pitch*(h)+kb*(bwd-wcenter),
									dstp+dpitch*(h)+kb*(bwd-owcenter),
									owin,iwin/owin);
							ResizeHLineRGB32(w2p+w2pitch*(bht-1-h)+kb*(wcenter),
									dstp+dpitch*(bht-1-h)+kb*(owcenter),
									owin,iwin/owin);
							ResizeHLineRGB32(w2p+w2pitch*(bht-1-h)+kb*(bwd-wcenter),
									dstp+dpitch*(bht-1-h)+kb*(bwd-owcenter),
									owin,iwin/owin);
						}
					}
					for(int w=dx;w<bwd/2;w++)
					{
						double iwin=(double)(w-dx)*(bht/2-dy)/(bwd/2-dx);
						double owin=(double)(w-dx)*(bht/2-2*dy)/(bwd/2-dx);
						int wcenter=dy+iwin/2;
						int owcenter=dy+(double)(w-dx)*dy/(bwd/2-dx)+owin/2;
						if(owin>0)
						{
							ResizeVLineRGB32(w2p+w2pitch*(wcenter)+kb*w,
									dstp+dpitch*(owcenter)+kb*w,
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineRGB32(w2p+w2pitch*(wcenter)+kb*(bwd-w),
									dstp+dpitch*(owcenter)+kb*(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineRGB32(w2p+w2pitch*(bht-1-wcenter)+kb*(w),
									dstp+dpitch*(bht-1-owcenter)+kb*(w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineRGB32(w2p+w2pitch*(bht-1-wcenter)+kb*(bwd-w),
									dstp+dpitch*(bht-1-owcenter)+kb*(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
						}
					}



				}
				if(vi.IsRGB24())
				{
					VFastResizerRGB24(srcp,wp,bht,bwd,vwin,bwd,pitch,wpitch,false);
					HFastResizerRGB24(wp,w2p+w2pitch*dy+kb*dx,vwin,bwd,vwin,wwin,wpitch,w2pitch,false);
					for(int h=dy;h<bht/2;h++)
					{
						double iwin=(double)(h-dy)*(bwd/2-dx)/(bht/2-dy);
						double owin=(double)(h-dy)*(bwd/2-2*dx)/(bht/2-dy);
						int wcenter=dx+iwin/2;
						int owcenter=dx+(double)(h-dy)*dx/(bht/2-dy)+owin/2;
						if(owin>0)
						{
							ResizeHLineRGB24(w2p+w2pitch*(h)+kb*wcenter,
										dstp+dpitch*(h)+kb*owcenter,
										owin,iwin/owin);
							ResizeHLineRGB24(w2p+w2pitch*(h)+kb*(bwd-wcenter),
										dstp+dpitch*(h)+kb*(bwd-owcenter),
										owin,iwin/owin);
							ResizeHLineRGB24(w2p+w2pitch*(bht-1-h)+kb*(wcenter),
										dstp+dpitch*(bht-1-h)+kb*(owcenter),
										owin,iwin/owin);
							ResizeHLineRGB24(w2p+w2pitch*(bht-1-h)+kb*(bwd-wcenter),
										dstp+dpitch*(bht-1-h)+kb*(bwd-owcenter),
										owin,iwin/owin);
						}
					}
					for(int w=dx;w<bwd/2;w++)
					{
						double iwin=(double)(w-dx)*(bht/2-dy)/(bwd/2-dx);
						double owin=(double)(w-dx)*(bht/2-2*dy)/(bwd/2-dx);
						int wcenter=dy+iwin/2;
						int owcenter=dy+(double)(w-dx)*dy/(bwd/2-dx)+owin/2;
						if(owin>0)
						{
							ResizeVLineRGB24(w2p+w2pitch*(wcenter)+kb*w,
									dstp+dpitch*(owcenter)+kb*w,
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineRGB24(w2p+w2pitch*(wcenter)+kb*(bwd-w),
									dstp+dpitch*(owcenter)+kb*(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineRGB24(w2p+w2pitch*(bht-1-wcenter)+kb*(w),
									dstp+dpitch*(bht-1-owcenter)+kb*(w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineRGB24(w2p+w2pitch*(bht-1-wcenter)+kb*(bwd-w),
									dstp+dpitch*(bht-1-owcenter)+kb*(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
						}
					}

				}
				if(vi.IsYUY2())
				{
			
					VFastResizerYUY2(srcp,wp,bht,bwd,vwin,bwd,pitch,wpitch,false);
					HFastResizerYUY2(wp,w2p+w2pitch*dy+kb*dx,vwin,bwd,vwin,wwin,wpitch,w2pitch,false);
					for(int h=dy;h<bht/2;h++)
					{
						double iwin=(double)(h-dy)*(bwd/2-dx)/(bht/2-dy);
						double owin=(double)(h-dy)*(bwd/2-2*dx)/(bht/2-dy);
						int wcenter=(int)(dx+iwin/2) & 0xfffffffe;
						int owcenter=(int)(dx+(double)(h-dy)*dx/(bht/2-dy)+owin/2) & 0xfffffffe;
						if(owin>0)
						{
							ResizeHLineYUY2(w2p+w2pitch*(h)+kb*wcenter,
									dstp+dpitch*(h)+kb*owcenter,
									owin,iwin/owin);
							ResizeHLineYUY2(w2p+w2pitch*(h)+kb*(bwd-wcenter),
									dstp+dpitch*(h)+kb*(bwd-owcenter),
									owin,iwin/owin);
							ResizeHLineYUY2(w2p+w2pitch*(bht-1-h)+kb*(wcenter),
									dstp+dpitch*(bht-1-h)+kb*(owcenter),
									owin,iwin/owin);
							ResizeHLineYUY2(w2p+w2pitch*(bht-1-h)+kb*(bwd-wcenter),
									dstp+dpitch*(bht-1-h)+kb*(bwd-owcenter),
									owin,iwin/owin);
						}
					}
					for(int w=dx;w<bwd/2;w+=2)
					{
						double iwin=(double)(w-dx)*(bht/2-dy)/(bwd/2-dx);
						double owin=(double)(w-dx)*(bht/2-2*dy)/(bwd/2-dx);
						int wcenter=dy+iwin/2;
						int owcenter=dy+(double)(w-dx)*dy/(bwd/2-dx)+owin/2;
						if(owin>0)
						{
							ResizeVLineYUY2(w2p+w2pitch*(wcenter)+kb*w,
									dstp+dpitch*(owcenter)+kb*w,
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineYUY2(w2p+w2pitch*(wcenter)+kb*(bwd-w),
									dstp+dpitch*(owcenter)+kb*(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineYUY2(w2p+w2pitch*(bht-1-wcenter)+kb*(w),
									dstp+dpitch*(bht-1-owcenter)+kb*(w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineYUY2(w2p+w2pitch*(bht-1-wcenter)+kb*(bwd-w),
									dstp+dpitch*(bht-1-owcenter)+kb*(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
						}
					}

				}

				if(vi.IsYV12())
				{
					wpU= work->GetWritePtr(PLANAR_U);
					wpV= work->GetWritePtr(PLANAR_V);
					const int wpitchUV=work->GetPitch(PLANAR_U);
					const	int bwdUV = LeftFrame->GetRowSize(PLANAR_U);
					const	int bhtUV = LeftFrame->GetHeight(PLANAR_U);
				
					w2pU= work2->GetWritePtr(PLANAR_U);
					w2pV= work2->GetWritePtr(PLANAR_V);
					const int w2pitchUV=work2->GetPitch(PLANAR_U);
			
					VFastResizerYV12(srcp,wp,bht,bwd,vwin,bwd,pitch,wpitch,false);
					HFastResizerYV12(wp,w2p+w2pitch*dy+dx,vwin,bwd,vwin,wwin,wpitch,w2pitch,false);
						// U plane and V planes
					VFastResizerYV12(srcpU,wpU,bhtUV,bwdUV,vwin/2,bwdUV,pitchUV,wpitchUV,false);
					HFastResizerYV12(wpU,w2pU+w2pitchUV*dy/2+dx/2,vwin/2,
									bwdUV,vwin/2,wwin/2,wpitchUV,w2pitchUV,false);
					VFastResizerYV12(srcpV,wpV,bhtUV,bwdUV,vwin/2,bwdUV,pitchUV,wpitchUV,false);
					HFastResizerYV12(wpV,w2pV+w2pitchUV*dy/2+dx/2,vwin/2,
									bwdUV,vwin/2,wwin/2,wpitchUV,w2pitchUV,false);
					for(int h=dy;h<bht/2;h++)
					{
						double iwin=(double)(h-dy)*(bwd/2-dx)/(bht/2-dy);
						double owin=(double)(h-dy)*(bwd/2-2*dx)/(bht/2-dy);
						int wcenter=(int)(dx+iwin/2) & 0xfffffffe;
						int owcenter=(int)(dx+(double)(h-dy)*dx/(bht/2-dy)+owin/2) & 0xfffffffe;
						if(owin>=2)
						{
							ResizeHLineYV12(w2p+w2pitch*(h)+wcenter,
									dstp+dpitch*(h)+owcenter,
									owin,iwin/owin);
							ResizeHLineYV12(w2p+w2pitch*(h)+(bwd-wcenter),
									dstp+dpitch*(h)+(bwd-owcenter),
									owin,iwin/owin);
							ResizeHLineYV12(w2p+w2pitch*(bht-1-h)+(wcenter),
									dstp+dpitch*(bht-1-h)+(owcenter),
									owin,iwin/owin);
							ResizeHLineYV12(w2p+w2pitch*(bht-1-h)+(bwd-wcenter),
									dstp+dpitch*(bht-1-h)+(bwd-owcenter),
									owin,iwin/owin);
						}
					}
					for(h=dy/2;h<bhtUV/2;h++)
					{
						double iwin=(double)(h-dy/2)*(bwdUV/2-dx/2)/(bhtUV/2-dy/2);
						double owin=(double)(h-dy/2)*(bwdUV/2-2*dx/2)/(bhtUV/2-dy/2);
						int wcenter=(int)(dx+iwin/2) & 0xfffffffe;
						int owcenter=(int)(dx/2+(double)(h-dy/2)*dx/2/(bhtUV/2-dy/2)+owin/2) & 0xfffffffe;
						if(owin>=2)
						{
						
									// U and V planes
							
							ResizeHLineYV12(w2pU+w2pitchUV*(h)+wcenter,
									dstpU+dpitchUV*(h)+owcenter,
									owin,iwin/owin);
							ResizeHLineYV12(w2pU+w2pitchUV*(h)+(bwdUV-wcenter),
									dstpU+dpitchUV*(h)+(bwdUV-owcenter),
									owin,iwin/owin);
							ResizeHLineYV12(w2pU+w2pitchUV*(bhtUV-2-h)+(wcenter),
									dstpU+dpitchUV*(bhtUV-2-h)+(owcenter),
									owin,iwin/owin);
							ResizeHLineYV12(w2pU+w2pitchUV*(bhtUV-2-h)+(bwdUV-wcenter),
									dstpU+dpitchUV*(bhtUV-2-h)+(bwdUV-owcenter),
									owin,iwin/owin);
									// V plane
							ResizeHLineYV12(w2pV+w2pitchUV*(h)+wcenter,
									dstpV+dpitchUV*(h)+owcenter,
									owin,iwin/owin);
							ResizeHLineYV12(w2pV+w2pitchUV*(h)+(bwdUV-wcenter),
									dstpV+dpitchUV*(h)+(bwdUV-owcenter),
									owin,iwin/owin);
							ResizeHLineYV12(w2pV+w2pitchUV*(bhtUV-2-h)+(wcenter),
									dstpV+dpitchUV*(bhtUV-2-h)+(owcenter),
									owin,iwin/owin);
							ResizeHLineYV12(w2pV+w2pitchUV*(bhtUV-2-h)+(bwdUV-wcenter),
									dstpV+dpitchUV*(bhtUV-2-h)+(bwdUV-owcenter),
									owin,iwin/owin);						}
					}
					for(int w=dx;w<bwd/2;w++)
					{
						double iwin=(double)(w-dx)*(bht/2-dy)/(bwd/2-dx);
						double owin=(double)(w-dx)*(bht/2-2*dy)/(bwd/2-dx);
						int wcenter=dy+iwin/2;
						int owcenter=dy+(double)(w-dx)*dy/(bwd/2-dx)+owin/2;
						if(owin>=2)
						{
							ResizeVLineYV12(w2p+w2pitch*(wcenter)+w,
									dstp+dpitch*(owcenter)+w,
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineYV12(w2p+w2pitch*(wcenter)+(bwd-w),
									dstp+dpitch*(owcenter)+(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineYV12(w2p+w2pitch*(bht-1-wcenter)+(w),
									dstp+dpitch*(bht-1-owcenter)+(w),
									owin,w2pitch,dpitch,iwin/owin);
							ResizeVLineYV12(w2p+w2pitch*(bht-1-wcenter)+(bwd-w),
									dstp+dpitch*(bht-1-owcenter)+(bwd-w),
									owin,w2pitch,dpitch,iwin/owin);
						}
					}
					for( w=dx/2;w<bwdUV/2;w++)
					{
						double iwin=(double)(w-dx/2)*(bhtUV/2-dy/2)/(bwdUV/2-dx/2);
						double owin=(double)(w-dx/2)*(bhtUV/2-2*dy/2)/(bwdUV/2-dx/2);
						int wcenter=dy/2+iwin/2;
						int owcenter=dy/2+(double)(w-dx/2)*dy/2/(bwdUV/2-dx/2)+owin/2;
						if(owin>=2)
						{
							ResizeVLineYV12(w2pU+w2pitchUV*(wcenter)+w,
									dstpU+dpitchUV*(owcenter)+w,
									owin,w2pitchUV,dpitchUV,iwin/owin);
							ResizeVLineYV12(w2pU+w2pitchUV*(wcenter)+(bwdUV-w),
									dstpU+dpitchUV*(owcenter)+(bwdUV-w),
									owin,w2pitchUV,dpitchUV,iwin/owin);
							ResizeVLineYV12(w2pU+w2pitchUV*(bhtUV-1-wcenter)+(w),
									dstpU+dpitchUV*(bhtUV-1-owcenter)+(w),
									owin,w2pitchUV,dpitchUV,iwin/owin);
							ResizeVLineYV12(w2pU+w2pitchUV*(bhtUV-1-wcenter)+(bwdUV-w),
									dstpU+dpitchUV*(bhtUV-1-owcenter)+(bwdUV-w),
									owin,w2pitchUV,dpitchUV,iwin/owin);
									// V plane
							ResizeVLineYV12(w2pV+w2pitchUV*(wcenter)+w,
									dstpV+dpitchUV*(owcenter)+w,
									owin,w2pitchUV,dpitchUV,iwin/owin);
							ResizeVLineYV12(w2pV+w2pitchUV*(wcenter)+(bwdUV-w),
									dstpV+dpitchUV*(owcenter)+(bwdUV-w),
									owin,w2pitchUV,dpitchUV,iwin/owin);
							ResizeVLineYV12(w2pV+w2pitchUV*(bhtUV-1-wcenter)+(w),
									dstpV+dpitchUV*(bhtUV-1-owcenter)+(w),
									owin,w2pitchUV,dpitchUV,iwin/owin);
							ResizeVLineYV12(w2pV+w2pitchUV*(bhtUV-1-wcenter)+(bwdUV-w),
									dstpV+dpitchUV*(bhtUV-1-owcenter)+(bwdUV-w),
									owin,w2pitchUV,dpitchUV,iwin/owin);
						}
					}

				}
			}
			
			
			if(*type=='f' || *type=='F')
			{
				
				if(vi.IsRGB32())
				{
					for(int w=0; w<bwd/2;w++)	
					{
	
										// lower quarter image
						ResizeVLineRGB32(srcp+pitch*((w*bht/bwd)/2)+kb*w,
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+kb*w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineRGB32(srcp+pitch*((w*bht/bwd)/2)+kb*(bwd-w),
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+kb*(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
										// top quarter image
						ResizeVLineRGB32(srcp+pitch*(bht-(w*bht/bwd)/2)+kb*w,
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+kb*w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineRGB32(srcp+pitch*(bht-(w*bht/bwd)/2)+kb*(bwd-w),
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+kb*(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
					}
					for(int h=0;h<bht/2;h++)
					{
										// Left quadrant image
						ResizeHLineRGB32(srcp+pitch*h+kb*((h*bwd/bht)/2),
											dstp+dpitch*h+kb*(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineRGB32(srcp+pitch*(bht-h)+kb*((h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+kb*(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
										// Right quadrant image
						ResizeHLineRGB32(srcp+pitch*h+kb*(bwd-(h*bwd/bht)/2),
											dstp+dpitch*h+kb*(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineRGB32(srcp+pitch*(bht-h)+kb*(bwd-(h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+kb*(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
					}

				}

				if(vi.IsRGB24())
				{
					for(int w=0; w<bwd/2;w++)	
					{
	
										// lower quarter image
						ResizeVLineRGB24(srcp+pitch*((w*bht/bwd)/2)+kb*w,
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+kb*w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineRGB24(srcp+pitch*((w*bht/bwd)/2)+kb*(bwd-w),
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+kb*(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
										// top quarter image
						ResizeVLineRGB24(srcp+pitch*(bht-(w*bht/bwd)/2)+kb*w,
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+kb*w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineRGB24(srcp+pitch*(bht-(w*bht/bwd)/2)+kb*(bwd-w),
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+kb*(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
					}
					for(int h=0;h<bht/2;h++)
					{
										// Left quadrant image
						ResizeHLineRGB24(srcp+pitch*h+kb*((h*bwd/bht)/2),
											dstp+dpitch*h+kb*(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineRGB24(srcp+pitch*(bht-h)+kb*((h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+kb*(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
										// Right quadrant image
						ResizeHLineRGB24(srcp+pitch*h+kb*(bwd-(h*bwd/bht)/2),
											dstp+dpitch*h+kb*(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineRGB24(srcp+pitch*(bht-h)+kb*(bwd-(h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+kb*(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
					}

				}
				if(vi.IsYUY2())
				{
					for(int w=0; w<bwd/2;w+=2)	
					{
	
										// lower quarter image
						ResizeVLineYUY2(srcp+pitch*((w*bht/bwd)/2)+kb*w,
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+kb*w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineYUY2(srcp+pitch*((w*bht/bwd)/2)+kb*(bwd-w),
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+kb*(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
										// top quarter image
						ResizeVLineYUY2(srcp+pitch*(bht-(w*bht/bwd)/2)+kb*w,
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+kb*w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineYUY2(srcp+pitch*(bht-(w*bht/bwd)/2)+kb*(bwd-w),
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+kb*(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
					}
					for(int h=0;h<bht/2;h++)
					{
										// Left quadrant image
						ResizeHLineYUY2(srcp+pitch*h+kb*((h*bwd/bht)/2),
											dstp+dpitch*h+kb*(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineYUY2(srcp+pitch*(bht-h)+kb*((h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+kb*(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
										// Right quadrant image
						ResizeHLineYUY2(srcp+pitch*h+kb*(bwd-(h*bwd/bht)/2),
											dstp+dpitch*h+kb*(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineYUY2(srcp+pitch*(bht-h)+kb*(bwd-(h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+kb*(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
					}

				}
				if(vi.IsYV12())
				{
					const	int bwdUV = LeftFrame->GetRowSize(PLANAR_U);
					const	int bhtUV = LeftFrame->GetHeight(PLANAR_U);
					for(int w=0; w<bwd/2;w++)	
					{
	
										// lower quarter image
						ResizeVLineYV12(srcp+pitch*((w*bht/bwd)/2)+w,
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineYV12(srcp+pitch*((w*bht/bwd)/2)+(bwd-w),
											dstp+dpitch*(w*(bht/2-2*dy)/(bwd))+(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
										// top quarter image
						ResizeVLineYV12(srcp+pitch*(bht-(w*bht/bwd)/2)+w,
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+w,
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
						ResizeVLineYV12(srcp+pitch*(bht-(w*bht/bwd)/2)+(bwd-w),
											dstp+dpitch*(bht-w*(bht/2-2*dy)/(bwd))+(bwd-w),
												w*(bht/2-2*dy)/(bwd/2),pitch,dpitch,
												(double)(bht)/(bht-4*dy));
					}
					for(w=0; w<bwdUV/2;w++)	
					{
										// U plane
										// lower quarter image
						ResizeVLineYV12(srcpU+pitchUV*((w*bhtUV/bwdUV)/2)+w,
											dstpU+dpitchUV*(w*(bhtUV/2-2*dy/2)/(bwdUV))+w,
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
						ResizeVLineYV12(srcpU+pitchUV*((w*bhtUV/bwdUV)/2)+(bwdUV-w),
											dstpU+dpitchUV*(w*(bhtUV/2-2*dy/2)/(bwdUV))+(bwdUV-w),
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
										// top quarter image
						ResizeVLineYV12(srcpU+pitchUV*(bhtUV-(w*bhtUV/bwdUV)/2)+w,
											dstpU+dpitchUV*(bhtUV-w*(bhtUV/2-2*dy/2)/(bwdUV))+w,
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
						ResizeVLineYV12(srcpU+pitchUV*(bhtUV-(w*bhtUV/bwdUV)/2)+(bwdUV-w),
											dstpU+dpitchUV*(bhtUV-w*(bhtUV/2-2*dy/2)/(bwdUV))+(bwdUV-w),
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
												// V plane
										// lower quarter image
						ResizeVLineYV12(srcpV+pitchUV*((w*bhtUV/bwdUV)/2)+w,
											dstpV+dpitchUV*(w*(bhtUV/2-2*dy/2)/(bwdUV))+w,
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
						ResizeVLineYV12(srcpV+pitchUV*((w*bhtUV/bwdUV)/2)+(bwdUV-w),
											dstpV+dpitchUV*(w*(bhtUV/2-2*dy/2)/(bwdUV))+(bwdUV-w),
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
										// top quarter image
						ResizeVLineYV12(srcpV+pitchUV*(bhtUV-(w*bhtUV/bwdUV)/2)+w,
											dstpV+dpitchUV*(bhtUV-w*(bhtUV/2-2*dy/2)/(bwdUV))+w,
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
						ResizeVLineYV12(srcpV+pitchUV*(bhtUV-(w*bhtUV/bwdUV)/2)+(bwdUV-w),
											dstpV+dpitchUV*(bhtUV-w*(bhtUV/2-2*dy/2)/(bwdUV))+(bwdUV-w),
												w*(bhtUV/2-2*dy/2)/(bwdUV/2),pitchUV,dpitchUV,
												(double)(bhtUV)/(bhtUV-4*dy/2));
					}
					for(int h=0;h<bht/2;h++)
					{
										// Left quadrant image
						ResizeHLineYV12(srcp+pitch*h+((h*bwd/bht)/2),
											dstp+dpitch*h+(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineYV12(srcp+pitch*(bht-h)+((h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+(h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
										// Right quadrant image
						ResizeHLineYV12(srcp+pitch*h+(bwd-(h*bwd/bht)/2),
											dstp+dpitch*h+(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
						ResizeHLineYV12(srcp+pitch*(bht-h)+(bwd-(h*bwd/bht)/2),
											dstp+dpitch*(bht-h)+(bwd-h*(bwd/2-2*dx)/(bht)),
											h*(bwd/2-2*dx)/(bht/2),
											(double)bwd/(bwd-4*dx));
					}
					
								
					for( h=0;h<bhtUV/2;h++)
					{
										// U plane
										// Left quadrant image
						ResizeHLineYV12(srcpU+pitchUV*h+((h*bwdUV/bhtUV)/2),
											dstpU+dpitchUV*h+(h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
						ResizeHLineYV12(srcpU+pitchUV*(bhtUV-h)+((h*bwdUV/bhtUV)/2),
											dstpU+dpitchUV*(bhtUV-h)+(h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
										// Right quadrant image
						ResizeHLineYV12(srcpU+pitchUV*h+(bwdUV-(h*bwdUV/bhtUV)/2),
											dstpU+dpitchUV*h+(bwdUV-h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
						ResizeHLineYV12(srcpU+pitchUV*(bhtUV-h)+(bwdUV-(h*bwdUV/bhtUV)/2),
											dstpU+dpitchUV*(bhtUV-h)+(bwdUV-h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
										// V plane
										// Left quadrant image
						ResizeHLineYV12(srcpV+pitchUV*h+((h*bwdUV/bhtUV)/2),
											dstpV+dpitchUV*h+(h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
						ResizeHLineYV12(srcpV+pitchUV*(bhtUV-h)+((h*bwdUV/bhtUV)/2),
											dstpV+dpitchUV*(bhtUV-h)+(h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
										// Right quadrant image
						ResizeHLineYV12(srcpV+pitchUV*h+(bwdUV-(h*bwdUV/bhtUV)/2),
											dstpV+dpitchUV*h+(bwdUV-h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
						ResizeHLineYV12(srcpV+pitchUV*(bhtUV-h)+(bwdUV-(h*bwdUV/bhtUV)/2),
											dstpV+dpitchUV*(bhtUV-h)+(bwdUV-h*(bwdUV/2-2*dx/2)/(bhtUV)),
											h*(bwdUV/2-2*dx/2)/(bhtUV/2),
											(double)bwdUV/(bwdUV-4*dx/2));
					}

				}
			}
		if (crumple)
				return RightFrame;
		return LeftFrame;
	}



/***************************************************************************************
bool VFastResizerRGB24(const unsigned char* srcptr,unsigned char* dstptr,int srcht, int srcwd,int dstht, int dstwd,
					 int spitch, int dpitch,  bool vflip=false);

bool HFastResizerYUY2(const unsigned char* sptr,unsigned char* dptr,int sht, int swd,int dht, int dwd,
					 int spitch, int dpitch,  bool hflip=false);
 // ResizeHLineRGB32 requires the srcp and dstp to point
	to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw, double factor);
// ResizeVLineRGB32 requires the srcp and dstp to point to middle of window winh/2.
 The factor is frame width/winh
inline void ResizeVLineRGB32( const unsigned char* srcp, unsigned char* dstp,
							 int winh,int spitch, int dpitch,double factor);

*******************************************************************************************/
			

/***************************************************************/
// This is the function that created the filter, when the filter has been called.

AVSValue __cdecl Create_TransCrumple(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	
			
	return new TransCrumple(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip
						args[2].AsInt(),	// overlap of clips
						args[3].AsString("crumple"),	// "crumple" "fan"

 						args[4].AsBool(true),	// true for fold, false for unfold
						
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransCrumple", "cci[type]s[fold]b", Create_TransCrumple, 0);
				
    return "TransCrumple Transition Mask Plugin";
}

*/