/******************************************************************************
The TransDisco Filter copies  circular disks from source frame to destination frame. The diameter of
copied round area increases from zero to specified value./ or decrease from radius to zero
if emerge is false.
The rounds can be rotated by nturns during transition.
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sampleModified 22-9-04
*************************************************************************************************/

/*  
#include "windows.h"
#include "avisynth.h"
#include "FastDisco.cpp"
#include "FastDiscoRotate.cpp"
*/
class TransDisco : public GenericVideoFilter 
{
		PClip RightClip;
		int  rad;
		int nturn;
		bool emerge;
		
		int overlap;

		void * abuf;
		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransDisco(PClip _child, PClip _RightClip,int _overlap,
		int _rad,int _nturn,bool _emerge,  
			IScriptEnvironment* env) ;	
			
 	~TransDisco();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransDisco::TransDisco(PClip _child, PClip _RightClip,int _overlap,
					   int _rad, int _nturn,bool _emerge,
					   IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap(_overlap),
		rad(_rad),nturn(_nturn), emerge(_emerge) 
			
{
char * Tname="TransDisco";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransDisco:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransDisco: This color format is not supported here");
if (vi.width != rvi.width || vi.height != rvi.height)
	env->ThrowError("%s:Clips have differing frame dimensions", Tname);
if(rad<8 || rad >sqrt(vi.width*vi.width+vi.height*vi.height)/2)
	env->ThrowError("TransDisco:radius should be greater than 8 and less than %d",sqrt(vi.width*vi.width+vi.height*vi.height)/2); 	
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;

}
// This is where any actual destructor code used goes
TransDisco::~TransDisco() {
	
// This is where you can deallocate any memory you might have used.
}
//_______________________________________________________________
void TransDisco::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}

bool TransDisco::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


	
/***************************************************************/	
	PVideoFrame __stdcall TransDisco::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		PVideoFrame dstFrame, srcFrame;
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		
		rad=rad & 0xfffffffe;
		const	int bwd = vi.width; 
        const	int bht = LeftFrame->GetHeight();	// LeftClip->GetHeight()?
		const	int pitch = LeftFrame->GetPitch();	
		if(!(pitch == RightFrame->GetPitch()))
			env->ThrowError("TransDisco: The clips have different pitch values");
		int radius=(rad*(n+1))/(overlap+1);
		if(radius<=2)
			return LeftFrame;
		unsigned char* dstp;
		const unsigned char * srcp;
		int nx=bwd/(2*rad);
		if(nx<1)
			nx=1;
		int ny = bht/(2*rad);
		if(ny<1)
			ny=1;
		int starty=(bht/(2*ny)) & 0xfffffffe;
		int startx = (bwd/(2*nx)) & 0xfffffffe;
		int radinc=((int) (rad * 1.6)) & 0xfffffffe;
		int degree= ((double)(n+1)*nturn*360)/(overlap+1);
		degree= degree%360;


		if(emerge)
		{
			env->MakeWritable(&LeftFrame);
			dstp= LeftFrame->GetWritePtr();
			srcp= RightFrame->GetReadPtr();
			dstFrame=LeftFrame;
			srcFrame=RightFrame;
		}
		if(!emerge)
		{
			env->MakeWritable(&RightFrame);
			dstp= RightFrame->GetWritePtr();
			srcp= LeftFrame->GetReadPtr();
			radius=(rad-radius);
			srcFrame=LeftFrame;
			dstFrame=RightFrame;
		}
		


		if(degree==0)
		{
			if(vi.IsRGB32())			
				for(int r=startx; r<bwd;r+=radinc)
					for(int c =starty; c<bht;c+=(radinc))
						FastDiscoRGB32( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, false, false);
				
		
			if(vi.IsRGB24())		
				for(int r=startx; r<bwd;r+=radinc)
					for(int c =starty; c<bht;c+=(radinc))
						FastDiscoRGB24( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, false, false);
			
		
			if(vi.IsYUY2())		
				for(int r=startx; r<bwd;r+=radinc)
					for(int c =starty; c<bht;c+=(radinc))
						FastDiscoYUY2( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, false, false);
			if(vi.IsYV12())
			{
				for(int r=startx; r<bwd;r+=radinc)
					for(int c =starty; c<bht;c+=(radinc))
						FastDiscoYV12( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, false, false);

				const unsigned char *srcpU = srcFrame->GetReadPtr(PLANAR_U);
				unsigned char *dstpU = dstFrame->GetWritePtr(PLANAR_U);
				const unsigned char *srcpV = srcFrame->GetReadPtr(PLANAR_V);
				unsigned char *dstpV = dstFrame->GetWritePtr(PLANAR_V);
				const int bwdUV=srcFrame->GetRowSize(PLANAR_U);
				const int bhtUV=srcFrame->GetHeight(PLANAR_U);
				const int spitchUV=srcFrame->GetPitch(PLANAR_U);
				const int dpitchUV=dstFrame->GetPitch(PLANAR_U);
				for( r=startx/2; r<bwdUV;r+=radinc/2)
					for(int c =starty/2; c<bhtUV;c+=(radinc)/2)
					{
						FastDiscoYV12( srcpU, dstpU, radius/2,
								r, c, bhtUV, bwdUV,
								r, c, bhtUV, bwdUV,
								spitchUV,dpitchUV, false, false);

						FastDiscoYV12( srcpV, dstpV, radius/2,
								r, c, bhtUV, bwdUV,
								r, c, bhtUV, bwdUV,
								spitchUV,dpitchUV, false, false);
					}
			}

		}
/*********************************************************************************/
		if(!(degree==0))
		{

			if(vi.IsRGB32())		
				for( int r=startx; r<bwd;r+=radinc )			
					for(int c =starty; c<bht;c+=radinc)
						FastDiscoRotateRGB32( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, degree);			
			
		
			if(vi.IsRGB24())		
				for( int r=startx; r<bwd;r+=radinc )
					for(int c =starty; c<bht;c+=radinc)
						FastDiscoRotateRGB24( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, degree);		

			if(vi.IsYUY2())
				for( int r=startx; r<bwd;r+=radinc )			
					for(int c =starty; c<bht;c+=radinc)
						FastDiscoRotateYUY2( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, degree);
			if(vi.IsYV12())
			{
				for( int r=startx; r<bwd;r+=radinc )			
					for(int c =starty; c<bht;c+=radinc)
						FastDiscoRotateYV12( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								pitch,pitch, degree);

				const unsigned char *srcpU = srcFrame->GetReadPtr(PLANAR_U);
				unsigned char *dstpU = dstFrame->GetWritePtr(PLANAR_U);
				const unsigned char *srcpV = srcFrame->GetReadPtr(PLANAR_V);
				unsigned char *dstpV = dstFrame->GetWritePtr(PLANAR_V);
				const int bwdUV=srcFrame->GetRowSize(PLANAR_U);
				const int bhtUV=srcFrame->GetHeight(PLANAR_U);
				const int spitchUV=srcFrame->GetPitch(PLANAR_U);
				const int dpitchUV=dstFrame->GetPitch(PLANAR_U);

				for(r=startx/2; r<bwdUV;r+=radinc/2 )			
					for(int c =starty/2; c<bhtUV;c+=radinc/2)
					{
						FastDiscoRotateYV12( srcpU, dstpU, radius/2,
								r, c, bhtUV, bwdUV,
								r, c, bhtUV, bwdUV,
								spitchUV,dpitchUV, degree);
				
						FastDiscoRotateYV12( srcpV, dstpV, radius/2,
								r, c, bhtUV, bwdUV,
								r, c, bhtUV, bwdUV,
								spitchUV,dpitchUV, degree);
					}
			}

		}
		return dstFrame;
	}

/**********************************************************************************/

// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransDisco(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
					
	return new TransDisco(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip
						args[2].AsInt(),	// overlap of clips -ve seconds, +ve frames
 						args[3].AsInt(40),	// radius of disk
						args[4].AsInt(0),	// 0 : no rotation, -ve clockwise, +ve anticlockwise turns
						args[5].AsBool(true),
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.

/*extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransDisco", "cci[radius]i[nturns]i[emerge]b", Create_TransDisco, 0);
				
    return "TransDisco Transition TransDisco Plugin";
}
*/
