/******************************************************************************
TransDoor filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip opening  vertical or horizontal doors or
The Right Clip masks  the Left clip by closing vertical or horizontal doors.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "FastResizer.cpp"
*/
class TransDoor : public GenericVideoFilter
 {
		PClip RightClip;
		const bool vert;
		const bool open;
		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransDoor(PClip _child, PClip _RightClip,int _overlap,
				const bool _vert, const bool _open,  
			IScriptEnvironment* env) ;	
				
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
	~TransDoor();
};	

//Here is the acutal constructor code used

TransDoor::TransDoor(PClip _child, PClip _RightClip,int _overlap, 
						 const bool _vert, const bool _open,
						 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap(_overlap),
						vert(_vert), open(_open) 
			
{
	char *	Tname="TransDoor";
	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransDoor:Clips have differing pixel_types");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransDoor: This color format is not yet supported ");
	if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
		env->ThrowError("TransDoor: The heights/widths of clips are unequal");
	if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
		env->ThrowError("TransDoor: Height/width of frame are not multiple of 4");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}

	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;

}
TransDoor::~TransDoor()
{ 

// This is where you can deallocate any memory you might have used.
}
	

/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
void TransDoor::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}

bool TransDoor::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


	
/***************************************************************/	
		PVideoFrame __stdcall TransDoor::GetFrame(int en, IScriptEnvironment* env)
		{
			if (en < video_fade_start)
				return child->GetFrame(en, env);
			if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
			int n=en-video_fade_start;
		
			PVideoFrame LeftFrame = child->GetFrame(en, env);

			PVideoFrame RightFrame = RightClip->GetFrame(n, env);
			PVideoFrame door, base;

			const VideoInfo& lvi = child->GetVideoInfo();
			const VideoInfo& rvi= RightClip->GetVideoInfo();
			
			const	int bwd = lvi.width;
			const	int bht = LeftFrame->GetHeight();
			int deltay =((n+1)*bht)/(2*(overlap+1));
			deltay = deltay & 0xFFFFFFFC;
			int deltax = ((n+1)*bwd)/(2*(overlap+1));
			deltax=deltax & 0xFFFFFFFC;
			int Doory, bp,dp,dht,dwd,Doorx,basex,basey;
		
		

			if(open)
			{
				base=RightFrame;
				door=LeftFrame;
				
			}
	
			else
			{
				door=RightFrame;
				base=LeftFrame;
								
			}
			env->MakeWritable(&base);
			unsigned char*basep=base->GetWritePtr();
			const unsigned char*doorp=door->GetReadPtr();
			const int dpitch=door->GetPitch();
			const int bpitch=base->GetPitch();

			if( vert)	// vertical doors
			
			{
				// Vertical Doors open revealing Right frame
			
				dht=bht;			
				Doorx= bwd/2;
				Doory =0;
				basex=bwd/2+deltax;
				basey=0;
				dwd=bwd/2-deltax;
				if(!open)
				{
					basex=bwd-2-deltax;
					dwd=deltax & 0xFFFFFFFE;
					
				}

			
			}
			if(!vert )
			{
					// Horizontal doors open
				dht=bht/2-deltay;
				dwd= bwd;
				Doorx=0;
				Doory=bht/2;
				basex=0;
				basey=bht/2+deltay;
				if(!open)			
				{
					//  doors close
					dht=deltay;
					basey=bht-1-deltay;
				}
			}

		
			if (dwd<2 || dht<2)
				return LeftFrame; // no work to be done
	
			if(vi.IsRGB24())
			{
				bp= 3*basex+basey*bpitch;
				dp= 3*Doorx+Doory*dpitch;
				if(vert)
				{
					HFastResizerRGB24(doorp,basep,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					HFastResizerRGB24(doorp+dp,basep+bp,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);

				}
				if(!vert)
				{
				
							// no vert resizing Left frame opening
							// resize left half of frame
					VFastResizerRGB24(doorp,basep,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					VFastResizerRGB24(doorp+dp,basep+bp,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);

					
				}
				
			}
		



/***************************************************************************************
bool VFastResizerRGB24(const unsigned char* srcptr,unsigned char* dstptr,int srcht, int srcwd,int dstht, int dstwd,
					 int spitch, int dpitch,  bool vflip=false);

bool HFastResizerYUY2(const unsigned char* sptr,unsigned char* dptr,int sht, int swd,int dht, int dwd,
					 int spitch, int dpitch,  bool hflip=false);

*******************************************************************************************/
			
			if(lvi.IsRGB32())
			{

				bp= 4*basex+basey*bpitch;
				dp= 4*Doorx+Doory*dpitch;
				if(vert)
				{
					HFastResizerRGB32(doorp,basep,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					HFastResizerRGB32(doorp+dp,basep+bp,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);
				}
				if(!vert)
				{
				
							// no vert resizing Left frame opening
							// resize left half of frame
					VFastResizerRGB32(doorp,basep,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					VFastResizerRGB32(doorp+dp,basep+bp,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);

					
				}
			}

			if(lvi.IsYUY2())
			{

				bp= 2*basex+basey*bpitch;
				dp= 2*Doorx+Doory*dpitch;
				if(vert)
				{
					HFastResizerYUY2(doorp,basep,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					HFastResizerYUY2(doorp+dp,basep+bp,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);
				}
				if(!vert)
				{
				
							// no vert resizing Left frame opening
							// resize left half of frame
					VFastResizerYUY2(doorp,basep,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					VFastResizerYUY2(doorp+dp,basep+bp,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);

					
				}
			}

			if(vi.IsYV12())
			{
				const unsigned char *doorpU = door->GetReadPtr(PLANAR_U);
				unsigned char *basepU = base->GetWritePtr(PLANAR_U);
				const unsigned char *doorpV = door->GetReadPtr(PLANAR_V);
				unsigned char *basepV = base->GetWritePtr(PLANAR_V);
				const int bwdUV=base->GetRowSize(PLANAR_U);
				const int bhtUV=base->GetHeight(PLANAR_U);
				const int bpitchUV=base->GetPitch(PLANAR_U);
				const int dpitchUV=door->GetPitch(PLANAR_U);
				

				bp= basex+basey*bpitch;
				dp= Doorx+Doory*dpitch;
				int bpUV=basex/2+basey/2*bpitchUV;
				int dpUV= Doorx/2+Doory/2*dpitchUV;

				if(vert)
				{
					HFastResizerYV12(doorp,basep,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					HFastResizerYV12(doorp+dp,basep+bp,bht,bwd/2,dht,dwd,
									dpitch,bpitch,false);

						// U plane
					HFastResizerYV12(doorpU,basepU,bhtUV,bwd/4,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
							// resize Right half of left Frame
					HFastResizerYV12(doorpU+dpUV,basepU+bpUV,bhtUV,bwd/4,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
							// V plane
					HFastResizerYV12(doorpV,basepV,bhtUV,bwd/4,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
							// resize Right half of left Frame
					HFastResizerYV12(doorpV+dpUV,basepV+bpUV,bhtUV,bwd/4,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
				}
				if(!vert)
				{
				
							// no vert resizing Left frame opening
							// resize left half of frame
					VFastResizerYV12(doorp,basep,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);
							// resize Right half of left Frame
					VFastResizerYV12(doorp+dp,basep+bp,bht/2,bwd,dht,dwd,
									dpitch,bpitch,false);
							// U plane
					VFastResizerYV12(doorpU,basepU,bht/4,bwd/2,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
							// resize Right half of left Frame
					VFastResizerYV12(doorpU+dpUV,basepU+bpUV,bht/4,bwd/2,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
							// U plane
					VFastResizerYV12(doorpV,basepV,bht/4,bwd/2,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
							// resize Right half of left Frame
					VFastResizerYV12(doorpV+dpUV,basepV+bpUV,bht/4,bwd/2,dht/2,dwd/2,
									dpitchUV,bpitchUV,false);
				}
			}

			
			return base;
			
		}



/***************************************************************/
// This is the function that created the filter, when the filter has been called.

AVSValue __cdecl Create_TransDoor(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
			
	return new TransDoor(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	// overlap of clips. -ve seconds, +ve frames
						args[3].AsBool(true),	// true: vert doors, false Hor Doors
						args[4].AsBool(true),	// true: doors open. false: doors close
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransDoor", "cci[vert]b[open]b", Create_TransDoor, 0);
				
    return "TransDoor Transition Mask Plugin";
}

*/