/******************************************************************************
TransFlipTurn filter plugin for Avisynth by V.C.Mohan
The RightClip  masks a stationary LeftClip by emerging flipping and rotating while growing to full size
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
Modified 22-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "FastResizer.cpp"
#include "FastDiscoRotate.h"
//#include "LineZ.cpp"
*/
class TransFlipTurn : public GenericVideoFilter {
		PClip RightClip;
		const int nvflip1;
		const int nhflip1;
		const int nturns;
		bool vf, hf;

		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransFlipTurn(PClip _child, PClip _RightClip,int _overlap,
		const int _nvflip, const int _nhflip, const int _nturns,  
			IScriptEnvironment* env) ;
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
			
 	~TransFlipTurn();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransFlipTurn::TransFlipTurn(PClip _child, PClip _RightClip,int _overlap, 
						 const int _nvflip, const int _nhflip,const int _nturns,
						 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap( _overlap),
						nvflip1(_nvflip), nhflip1(_nhflip),nturns(_nturns) 
			
{
	char * Tname="TransFlipTurn";	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransFlipTurn:Clips have differing pixel_types");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransFlipTurn: This color format is not yet supported ");
	if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
		env->ThrowError("TransFlipTurn: The heights/widths of clips are unequal");
	if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
		env->ThrowError("TransFlipTurn: Height/width of frame are not multiple of 4");
	vf=false;
	hf=false;
	
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname);
	if(nhflip1 >overlap/4 || nvflip1 > overlap/4)
		env->ThrowError("TransFlipTurn:flips more than 1/4 of number of overlapping frames ");
	if(nhflip1 <-overlap/4 || nvflip1 <- overlap/4)
		env->ThrowError("TransFlipTurn:flips more than 1/4 of number of overlapping frames ");


  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
	
// This is where any actual destructor code used goes
TransFlipTurn::~TransFlipTurn() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/
void TransFlipTurn::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransFlipTurn::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}

//________________________________________________________________________________________
	
		PVideoFrame __stdcall TransFlipTurn::GetFrame(int en, IScriptEnvironment* env)
		{
			if (en < video_fade_start)
				return child->GetFrame(en, env);
			if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
			int n=en-video_fade_start;
		
			PVideoFrame LeftFrame = child->GetFrame(en, env);

			PVideoFrame RightFrame = RightClip->GetFrame(n, env);
			const VideoInfo& rvi= RightClip->GetVideoInfo();
			
			int tap=20;	// this value decides how much taper is. bht/tap and bwd/tap are max tapers
			double winw, winh,taperw,taperh,ltaper,rtaper,toptaper,bottaper;
			const	int bwd = vi.width;
			const	int bht = LeftFrame->GetHeight();
			int degree=((n+1)*nturns*360)/(overlap+1);	// degrees of rotation 
			int hdn, vdn;	// number of frames per flip
			int nvflip,nhflip;	// these will be positive numbers
			if(nvflip1<0)
				nvflip=-nvflip1;
			else
				nvflip=nvflip1;

			if(nhflip1<0)
				nhflip=-nhflip1;
			else
				nhflip=nhflip1;
			if(nvflip>0)
				vdn=overlap/(nvflip+1);
			if(nvflip==0)
			{
				vdn=2*overlap;			// ensure that no flip takes place
				toptaper=0;
				bottaper=0;
			}
			
			if(nhflip>0)
				hdn=overlap/(nhflip+1);	// frames per horizontal flip
			if(nhflip==0)
			{
				hdn=2*overlap;			// ensure no flip takes place
				ltaper=0;
				rtaper=0;
			}
			
			if(n>hdn && (n%hdn) ==hdn/2 )					// flips 
				hf=!hf;			
			if(n>vdn && (n%vdn) == vdn/2)
				vf=!vf;
		

			if(n>vdn)
			{
				if(n%vdn < vdn/2)
					winh=(n*bht/overlap)*(double)(vdn/2-n%vdn)/(vdn/2);	// window decreases for flip
				if(n%vdn >= vdn/2)
					winh=(n*bht/overlap)*(double)(n%vdn-vdn/2)/(vdn/2);	// window increases from flip

				if(nvflip<0)
				{
					toptaper=(1.0-((double)((n+vdn/2)%vdn)/vdn))*(n*bwd)/(overlap*tap);
					bottaper=((double)((n+vdn/2)%vdn)/vdn)*(n*bwd)/(overlap*tap);
				}
				else
				{
					bottaper=(1.0-((double)((n+vdn/2)%vdn)/vdn))*(n*bwd)/(overlap*tap);
					toptaper=((double)((n+vdn/2)%vdn)/vdn)*(n*bwd)/(overlap*tap);
				}

			
			}
			if(n<=vdn || n>(1+nvflip)*vdn)
			{
				winh=(double)n*bht/overlap;
				vf=false;				// no flip occurs in these zones
				toptaper=0;	// new
				bottaper=0;	//new
				
			}
			

			if(n>hdn)
			{

				if(n%hdn < hdn/2)
					winw=(n*bwd/overlap)*(double)(hdn/2-n%hdn)/(hdn/2);

				if(n%hdn >= hdn/2)
					winw=(n*bwd/overlap)*(double)(n%hdn-hdn/2)/(hdn/2);
				if(nhflip<0)
				{
					ltaper=(1.0-((double)((n+hdn/2)%hdn)/hdn))*(n*bht)/(overlap*tap);
					rtaper=((double)((n+hdn/2)%hdn)/hdn)*(n*bht)/(overlap*tap);
				}
				else
				{
					rtaper=(1.0-((double)((n+hdn/2)%hdn)/hdn))*(n*bht)/(overlap*tap);
					ltaper=((double)((n+hdn/2)%hdn)/hdn)*(n*bht)/(overlap*tap);
				}

			}
			if(n<=hdn || n>(1+nhflip)*hdn)
			{
				winw=(double)n*bwd/overlap;
				hf=false;
				ltaper=0;
				rtaper=0;
				
			}
			int wwin=((int)winw)& 0xfffffffe;	// Fast resizer accepts only even numbers of pixels
			int vwin=((int)winh) & 0xfffffffe;
			if(vwin<2 )
				vwin=2;
			if(wwin<2)
				wwin=2;

			const	int dpitch = LeftFrame->GetPitch();	
			const	int spitch = RightFrame->GetPitch();
			const unsigned char *srcp;
			unsigned char * dstp;
			env->MakeWritable(&LeftFrame);
			dstp=LeftFrame->GetWritePtr();
			srcp=RightFrame->GetReadPtr();
			PVideoFrame work= env->NewVideoFrame(vi);
			unsigned char * wp= work->GetWritePtr();
			const int wpitch=work->GetPitch();
			const int kb=vi.BitsPerPixel()/8;
			PVideoFrame work2= env->NewVideoFrame(vi);
			unsigned char * w2p= work2->GetWritePtr();
			const int w2pitch=work2->GetPitch();
			int radius= sqrt(vwin*vwin+wwin*wwin)/2;
					// Resize the right frame and flip to the correct position on to work2

			if(vi.IsRGB32())
			{
				VFastResizerRGB32(srcp,wp,bht,bwd,vwin,bwd,spitch,wpitch,vf);
				HFastResizerRGB32(wp,w2p,vwin,bwd,vwin,wwin,wpitch,w2pitch,hf);
				if((nturns) !=0)					
					FastDiscoRotateRGB32(w2p,dstp,
					  radius,wwin/2,vwin/2,vwin,wwin,
					  bwd/2,bht/2,bht,bwd,w2pitch,dpitch,degree);
				
					
			}
			if(vi.IsRGB24())
			{
				VFastResizerRGB24(srcp,wp,bht,bwd,vwin,bwd,spitch,wpitch,vf);
				HFastResizerRGB24(wp,w2p,vwin,bwd,vwin,wwin,wpitch,w2pitch,hf);
				if((nturns) !=0)					
					FastDiscoRotateRGB24(w2p,dstp,
					  radius,wwin/2,vwin/2,vwin,wwin,
					  bwd/2,bht/2,bht,bwd,w2pitch,dpitch,degree);
				
			}
			

			if(vi.IsYUY2())
			{
				vwin=vwin & 0xfffffffc;
				wwin=wwin & 0xfffffffc;
				vwin=vwin<4?4:vwin;
				wwin=wwin<4?4:wwin;
				VFastResizerYUY2(srcp,wp,bht,bwd,vwin,bwd,spitch,wpitch,vf);
				HFastResizerYUY2(wp,w2p,vwin,bwd,vwin,wwin,wpitch,w2pitch,hf);
				if((nturns) !=0)				
					FastDiscoRotateYUY2(w2p,dstp,
					  radius,wwin/2,vwin/2,vwin,wwin,
					  bwd/2,bht/2,bht,bwd,w2pitch,dpitch,degree);
				
			}
			if(vi.IsRGB24() || vi.IsRGB32()|| vi.IsYUY2() )

				if((nturns) ==0)
							// Taper both horizontal and vertical to get depth effect if no rotation
					for(int h=0;h<vwin;h++)
					{
						taperw=toptaper+h*(bottaper-toptaper)/vwin;					
						for(int w=taperw;w<wwin-taperw;w++)	// new					
						{
							taperh=ltaper+w*(rtaper-ltaper)/wwin;		
							if(h>taperh && h<vwin-taperh)	// new			
								for(int k=0;k<kb;k++)									
									*(dstp+dpitch*(h+(bht-vwin)/2)+kb*(w+(bwd-wwin)/2)+k)	
										= *(w2p+h*w2pitch+kb*w+k);
						}
					}

			
					

			if(vi.IsYV12())
			{
				const unsigned char *srcpU=RightFrame->GetReadPtr(PLANAR_U);
				const unsigned char *srcpV=RightFrame->GetReadPtr(PLANAR_V);
				const int spitchUV=RightFrame->GetPitch(PLANAR_U);
				unsigned char *dstpU=LeftFrame->GetWritePtr(PLANAR_U);
				unsigned char *dstpV=LeftFrame->GetWritePtr(PLANAR_V);
				const int dpitchUV=LeftFrame->GetPitch(PLANAR_U);
				unsigned char *wpU= work->GetWritePtr(PLANAR_U);
				unsigned char *wpV= work->GetWritePtr(PLANAR_V);
				const int wpitchUV=work->GetPitch(PLANAR_U);
				const	int bwdUV = LeftFrame->GetRowSize(PLANAR_U);
				const	int bhtUV = LeftFrame->GetHeight(PLANAR_U);
					
				unsigned char *w2pU= work2->GetWritePtr(PLANAR_U);
				unsigned char *w2pV= work2->GetWritePtr(PLANAR_V);
				const int w2pitchUV=work2->GetPitch(PLANAR_U);
				vwin=vwin & 0xfffffffc;
				wwin=wwin & 0xfffffffc;
				taperw=(int)taperw & 0xfffffffe;
				taperh=(int)taperh & 0xfffffffe;
				VFastResizerYV12(srcp,wp,bht,bwd,vwin,bwd,spitch,wpitch,vf);
				HFastResizerYV12(wp,w2p,vwin,bwd,vwin,wwin,wpitch,w2pitch,hf);
				if((nturns) !=0)				
					FastDiscoRotateYV12(w2p,dstp,
					  radius,wwin/2,vwin/2,vwin,wwin,
					  bwd/2,bht/2,bht,bwd,w2pitch,dpitch,degree);
						// U plane
				VFastResizerYV12(srcpU,wpU,bhtUV,bwdUV,vwin/2,bwdUV,spitchUV,wpitchUV,vf);
				HFastResizerYV12(wpU,w2pU,vwin/2,bwdUV,vwin/2,wwin/2,wpitchUV,w2pitchUV,hf);
				if((nturns) !=0)				
					FastDiscoRotateYV12(w2pU,dstpU,
					  radius/2,wwin/4,vwin/4,vwin/2,wwin/2,
					  bwdUV/2,bhtUV/2,bhtUV,bwdUV,w2pitchUV,dpitchUV,degree);
						// V plane
				VFastResizerYV12(srcpV,wpV,bhtUV,bwdUV,vwin/2,bwdUV,spitchUV,wpitchUV,vf);
				HFastResizerYV12(wpV,w2pV,vwin/2,bwdUV,vwin/2,wwin/2,wpitchUV,w2pitchUV,hf);
				if((nturns) !=0)				
					FastDiscoRotateYV12(w2pV,dstpV,
					  radius/2,wwin/4,vwin/4,vwin/2,wwin/2,
					  bwdUV/2,bhtUV/2,bhtUV,bwdUV,w2pitchUV,dpitchUV,degree);
			
				if((nturns) ==0)
							// Taper both horizontal and vertical to get depth effect if no rotation
					for(int h=0;h<vwin;h++)
					{
						taperw=toptaper+h*(bottaper-toptaper)/vwin;					
						for(int w=taperw;w<wwin-taperw;w++)	// new					
						{
							taperh=ltaper+w*(rtaper-ltaper)/wwin;		
							if(h>taperh && h<vwin-taperh)	// new
							{
								*(dstp+dpitch*(h+(bht-vwin)/2)+(w+(bwd-wwin)/2))	
										= *(w2p+h*w2pitch+w);
								if((h & 1)==0 && (w & 1)==0)
								{
									*(dstpU+dpitchUV*(h+(bht-vwin)/2)/2+(w+(bwd-wwin)/2)/2)	
										= *(w2pU+h/2*w2pitchUV+w/2);
									*(dstpV+dpitchUV*(h+(bht-vwin)/2)/2+(w+(bwd-wwin)/2)/2)	
										= *(w2pV+h/2*w2pitchUV+w/2);
								}
							}
						}
					}
			}

			return LeftFrame;
		}



			

			


/***************************************************************************************
bool VFastResizerRGB24(const unsigned char* srcptr,unsigned char* dstptr,int srcht, int srcwd,int dstht, int dstwd,
					 int spitch, int dpitch,  bool vflip=false);

bool HFastResizerYUY2(const unsigned char* sptr,unsigned char* dptr,int sht, int swd,int dht, int dwd,
					 int spitch, int dpitch,  bool hflip=false);
bool FastDiscoRotateRGB32(const unsigned char *srcp,unsigned char*dstp,
					  int radius,int srcx,int srcy,int srch,int srcw,
					  int dstx,int dsty,int dsth,int dstw,int spitch, int dpitch,int degree);
*******************************************************************************************/
			

/***************************************************************/
// This is the function that created the filter, when the filter has been called.

AVSValue __cdecl Create_TransFlipTurn(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	if((args[3].AsInt(0) & 1)>0 || (args[4].AsInt(0) & 1)> 0)
		env->ThrowError("TransFlipTurn:The number of flips must be  even numbers");
	

			
	return new TransFlipTurn(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip
						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
 						args[3].AsInt(0),	// number of  vertical flips
						args[4].AsInt(0),	// number of horizontal flips
						args[5].AsInt(4),	// number of turns
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransFlipTurn", "cci[vflip]i[hflip]i[nturns]i", Create_TransFlipTurn, 0);
				
    return "TransFlipTurn Transition Mask Plugin";
}

*/