/******************************************************************************
TransFunnel filter plugin for Avisynth by V.C.Mohan
The LeftClip  funnels out  in the direction specified,unmasking a stationary RightClip clip.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also 
used partly adapted function coding 
from Avisynth and simple sample. 
 shading along roll length are other params
 Modified 23-9-04
********************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "LineZ.cpp"
*/
class TransFunnel : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;

		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		
		
		 
  public:
							//Definition of function
    TransFunnel(PClip _child, PClip _RightClip,int _overlap,
				const char* _dir,
				   
				IScriptEnvironment* env) ;	
			
 	~TransFunnel();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
	
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/

//Here is the acutal constructor code used

TransFunnel::TransFunnel(PClip _child, PClip _RightClip,
						 int _overlap,const char* _dir,
					  
					 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap(_overlap), dir(_dir)
						 
			
{
	char * Tname="TransFunnel";
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransFunnel:Clips have differing pixel_types");

	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2()&& !vi.IsYV12())
		env->ThrowError("TransFunnel: This color format is not supported here");
	if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
		env->ThrowError("TransFunnel: The heights/widths of clips are unequal");
	if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
		env->ThrowError("TransFunnel: Height/width of frame are not multiple of 4");
	if(!(*dir=='u') && !(*dir=='U') && !(*dir=='d') && !(*dir=='D') &&
		!(*(dir)=='l') && !(*(dir)=='L') && !(*(dir)=='r') && !(*(dir)=='R'))
		env->ThrowError("TransFunnel:up down, left and right only  are valid directions");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);

	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransFunnel::~TransFunnel() {
	
// This is where you can deallocate any memory you might have used.
}
//______________________________________________________________________
void TransFunnel::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransFunnel::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


	
/***************************************************************/	
	PVideoFrame __stdcall TransFunnel::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& vi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
	
		const	int bwd = vi.width;
        const	int bht = LeftFrame->GetHeight();
		double dx =(2.0*(n+1)*bwd)/(overlap+1);
		double dy =(2.0*(n+1)*bht)/(overlap+1);

		if(2*n>=overlap)
		{
			dx =(2.0*(overlap-n)*bwd)/(overlap+1);
			dy =(2.0*(overlap-n)*bht)/(overlap+1);

		}
		int dropy=(bht*(2*n-overlap))/overlap;
		int dropx=(bwd*(2*n-overlap))/overlap;
		const	int pitch = LeftFrame->GetPitch();

		const	int rpitch = RightFrame->GetPitch();

		if(pitch!=rpitch)
			env->ThrowError("TransFunnel:unequal pitch values");
		int  wht, wwd;
		env->MakeWritable(&RightFrame);
		unsigned char* Rp= RightFrame->GetWritePtr();
		const unsigned char * Lp= LeftFrame->GetReadPtr();
		int kb=vi.BitsPerPixel()/8;
	
		const int pitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
		const int bhtUV = LeftFrame->GetHeight(PLANAR_U);
		const unsigned char *LpU = LeftFrame->GetReadPtr(PLANAR_U);
		const unsigned char *LpV = LeftFrame->GetReadPtr(PLANAR_V);
		unsigned char *RpU = RightFrame->GetWritePtr(PLANAR_U);
		unsigned char *RpV = RightFrame->GetWritePtr(PLANAR_V);

/*
// FlipResizeHLineRGB32 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// ResizeVLineRGB32 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineRGB32( const unsigned char* srcp, unsigned char* dstp,
 int winh,int spitch, int dpitch,double factor);
 inline void CopyVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
	*/	
	
		if((*dir=='d' || *dir=='D'))
			
		{			//LeftFrame funnels out s wards unmasking RightFrame

			if(vi.IsRGB32())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineRGB32( Lp+h*pitch+kb*bwd/2, 
										Rp+h*pitch+kb*bwd/2, h*bwd/dy,dy/h);
					for(h=dy;h<bht;h++)
						CopyHLineRGB32(Lp+h*pitch,Rp+h*pitch,bwd);
				}
				if(2*n>=overlap)
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineRGB32( Lp+(dropy+h)*pitch+kb*bwd/2, 
										Rp+h*pitch+kb*bwd/2, h*dx/bht,bwd*bht/(h*dx));
			}
			if(vi.IsRGB24())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineRGB24( Lp+h*pitch+kb*bwd/2, 
										Rp+h*pitch+kb*bwd/2, h*bwd/dy,dy/h);
					for(h=dy;h<bht;h++)
						CopyHLineRGB24(Lp+h*pitch,Rp+h*pitch,bwd);
				}
				if(2*n>=overlap)
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineRGB24( Lp+(dropy+h)*pitch+kb*bwd/2, 
										Rp+h*pitch+kb*bwd/2, h*dx/bht,bwd*bht/(h*dx));
			}
			if(vi.IsYUY2())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineYUY2( Lp+(bht-1-h)*pitch+kb*bwd/2, 
										Rp+(bht-1-h)*pitch+kb*bwd/2, h*bwd/dy,dy/h);
					for(h=dy;h<bht;h++)
						CopyHLineYUY2(Lp+(bht-1-h)*pitch,Rp+(bht-1-h)*pitch,bwd);
				}
				if(2*n>=overlap)
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineYUY2( Lp+(bht-1-(dropy+h))*pitch+kb*bwd/2, 
										Rp+(bht-1-h)*pitch+kb*bwd/2,
										h*dx/bht,bwd*bht/(h*dx));
			}
			if(vi.IsYV12())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineYV12( Lp+(bht-1-h)*pitch+bwd/2, 
										Rp+(bht-1-h)*rpitch+bwd/2, h*bwd/dy,dy/h);
					for(h=dy;h<bht;h++)
						CopyHLineYV12(Lp+(bht-1-h)*pitch,Rp+(bht-1-h)*rpitch,bwd);
							// U  and V planes
					for(h=1;h<dy/2;h++)
					{
						ResizeHLineYV12( LpU+(bhtUV-1-h)*pitchUV+bwdUV/2, 
										RpU+(bhtUV-1-h)*pitchUV+bwdUV/2, h*bwdUV/(dy/2),(dy/2)/h);
						ResizeHLineYV12( LpV+(bhtUV-1-h)*pitchUV+bwdUV/2, 
										RpV+(bhtUV-1-h)*pitchUV+bwdUV/2, h*bwdUV/(dy/2),(dy/2)/h);
					}
					for(h=dy/2;h<bhtUV;h++)
					{
						CopyHLineYV12(LpU+(bhtUV-1-h)*pitchUV,RpU+(bhtUV-1-h)*rpitchUV,bwdUV);
					
						CopyHLineYV12(LpV+(bhtUV-1-h)*pitchUV,RpV+(bhtUV-1-h)*rpitchUV,bwdUV);
					}
				}
				if(2*n>=overlap)
				{
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineYV12( Lp+(bht-1-(dropy+h))*pitch+bwd/2, 
										Rp+(bht-1-h)*rpitch+bwd/2,
										h*dx/bht,bwd*bht/(h*dx));
							// U plane
					for(h=1;h<bhtUV-dropy/2;h++)
					{
						ResizeHLineYV12( LpU+(bhtUV-1-(dropy/2+h))*pitchUV+bwdUV/2, 
										RpU+(bhtUV-1-h)*rpitchUV+bwdUV/2,
										h*(dx/2)/bhtUV,bwdUV*bhtUV/(h*dx/2));
							// V plane
					
						ResizeHLineYV12( LpV+(bhtUV-1-(dropy/2+h))*pitchUV+bwdUV/2, 
										RpV+(bhtUV-1-h)*rpitchUV+bwdUV/2,
										h*(dx/2)/bhtUV,bwdUV*bhtUV/(h*dx/2));
					}
				}
			}
		}


		if((*dir=='u' || *dir=='U'))
			
		{			//LeftFrame funnels out s wards unmasking RightFrame

			if(vi.IsRGB32())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineRGB32( Lp+(bht-1-h)*pitch+kb*bwd/2, 
										Rp+(bht-1-h)*pitch+kb*bwd/2, h*bwd/dy,dy/h);
					for(h=dy;h<bht;h++)
						CopyHLineRGB32(Lp+(bht-1-h)*pitch,Rp+(bht-1-h)*pitch,bwd);
				}
				if(2*n>=overlap)
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineRGB32( Lp+(bht-1-(dropy+h))*pitch+kb*bwd/2, 
										Rp+(bht-1-h)*pitch+kb*bwd/2,
										h*dx/bht,bwd*bht/(h*dx));
			}
			if(vi.IsRGB24())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineRGB24( Lp+(bht-1-h)*pitch+kb*bwd/2, 
											Rp+(bht-1-h)*pitch+kb*bwd/2,
												h*bwd/dy,dy/h);
					for(h=dy;h<bht;h++)
						CopyHLineRGB24(Lp+(bht-1-h)*pitch,Rp+(bht-1-h)*pitch,bwd);
				}
				if(2*n>=overlap)
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineRGB24( Lp+(bht-1-(dropy+h))*pitch+kb*bwd/2,
										Rp+(bht-1-h)*pitch+kb*bwd/2,
										h*dx/bht,bwd*bht/(h*dx));
				
				
			}
			if(vi.IsYUY2())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineYUY2( Lp+h*pitch+kb*bwd/2, 
										Rp+h*pitch+kb*bwd/2, h*bwd/dy,dy/h);
					for(h=dy;h<bht;h++)
						CopyHLineYUY2(Lp+h*pitch,Rp+h*pitch,bwd);
				}
				if(2*n>=overlap)
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineYUY2( Lp+(dropy+h)*pitch+kb*bwd/2, 
											Rp+h*pitch+kb*bwd/2, 
											h*dx/bht,bwd*bht/(h*dx));
			}
			if(vi.IsYV12())
			{
				if(2*n<overlap)
				{
					for(int h=1;h<dy;h++)						
						ResizeHLineYV12( Lp+h*pitch+bwd/2, 
										Rp+h*rpitch+bwd/2, h*bwd/dy,dy/h);
					for(h=1;h<dy/2;h++)
					{
						ResizeHLineYV12( LpU+h*pitchUV+bwdUV/2, 
										RpU+h*rpitchUV+bwdUV/2, h*bwdUV/(dy/2),(dy/2)/h);
						ResizeHLineYV12( LpV+h*pitchUV+bwdUV/2, 
										RpV+h*rpitchUV+bwdUV/2, h*bwdUV/(dy/2),(dy/2)/h);
					}
					for(h=dy;h<bht;h++)
						CopyHLineYV12(Lp+h*pitch,Rp+h*rpitch,bwd);
					for(h=dy/2;h<bhtUV;h++)
					{
						CopyHLineYV12(LpU+h*pitchUV,RpU+h*rpitchUV,bwdUV);
						CopyHLineYV12(LpV+h*pitchUV,RpV+h*rpitchUV,bwdUV);
					}
				}
				if(2*n>=overlap)
				{
					for(int h=1;h<bht-dropy;h++)
						ResizeHLineYV12( Lp+(dropy+h)*pitch+bwd/2, 
											Rp+h*rpitch+bwd/2, 
											h*dx/bht,bwd*bht/(h*dx));
					for(h=1;h<bhtUV-dropy/2;h++)
					{
						ResizeHLineYV12( LpU+(dropy/2+h)*pitchUV+bwdUV/2, 
											RpU+h*rpitchUV+bwdUV/2, 
											h*(dx/2)/bhtUV,bwd*bht/(h*dx));
						ResizeHLineYV12( LpV+(dropy/2+h)*pitchUV+bwdUV/2, 
											RpV+h*rpitchUV+bwdUV/2, 
											h*(dx/2)/bhtUV,bwd*bht/(h*dx));
					}
				}
			}

		}

		if(*dir=='l' || *dir=='L')
		{
			if(vi.IsRGB32())
			{
				if(2*n<overlap)
				{
					for(int w=1;w<dx;w++)
						ResizeVLineRGB32(Lp+pitch*bht/2+kb*w, Rp+pitch*bht/2+kb*w, 
											w*bht/dx,pitch,pitch,dx/w);
					for(w=dx;w<bwd;w++)
						CopyVLineRGB32(Lp+kb*w,Rp+kb*w,bht,pitch,pitch);
				}
				if(2*n>=overlap)
					for(int w=1;w<bwd-dropx;w++)
						ResizeVLineRGB32(Lp+pitch*bht/2+(dropx+w)*kb, 
										Rp+pitch*bht/2+(w)*kb,
										w*dy/bwd,pitch,pitch,bwd*bht/(w*dy));
			}
			if(vi.IsRGB24())
			{
				if(2*n<overlap)
				{
					for(int w=1;w<dx;w++)
						ResizeVLineRGB24(Lp+pitch*bht/2+kb*w, Rp+pitch*bht/2+kb*w, 
											w*bht/dx,pitch,pitch,dx/w);
					for(w=dx;w<bwd;w++)
						CopyVLineRGB24(Lp+kb*w,Rp+kb*w,bht,pitch,pitch);
				}
				if(2*n>=overlap)
					for(int w=1;w<bwd-dropx;w++)
						ResizeVLineRGB24(Lp+pitch*bht/2+(dropx+w)*kb, 
										Rp+pitch*bht/2+(w)*kb,
										w*dy/bwd,pitch,pitch,bwd*bht/(w*dy));
			}
			if(vi.IsYUY2())
			{
				if(2*n<overlap)
				{
					for(int w=2;w<dx;w+=2)
						ResizeVLineYUY2(Lp+pitch*bht/2+kb*w, Rp+pitch*bht/2+kb*w,
										w*bht/dx,pitch,pitch,dx/w);
					for(w=((int)dx)& 0xFFFFFFFE;w<bwd;w+=2)
						CopyVLineYUY2(Lp+kb*w,Rp+kb*w,bht,pitch,pitch);
				}
				dropx= dropx & 0xFFFFFFFE;
				if(2*n>=overlap)
					for(int w=2;w<bwd-dropx;w+=2)
						ResizeVLineYUY2(Lp+pitch*bht/2+(dropx+w)*kb, 
										Rp+pitch*bht/2+(w)*kb,
										w*dy/bwd,pitch,pitch,bwd*bht/(w*dy));
			}

			if(vi.IsYV12())
			{
				if(2*n<overlap)
				{
					for(int w=1;w<dx;w++)
						ResizeVLineYV12(Lp+pitch*bht/2+w, Rp+rpitch*bht/2+w,
										w*bht/dx,pitch,rpitch,dx/w);
					for(w=dx;w<bwd;w++)
						CopyVLineYV12(Lp+w,Rp+w,bht,pitch,rpitch);
					for(w=1;w<dx/2;w++)
					{
						ResizeVLineYV12(LpU+pitchUV*bhtUV/2+w, RpU+rpitchUV*bhtUV/2+w,
										w*bhtUV/(dx/2),pitchUV,rpitchUV,(dx/2)/w);
						ResizeVLineYV12(LpV+pitchUV*bhtUV/2+w, RpV+rpitchUV*bhtUV/2+w,
										w*bhtUV/(dx/2),pitchUV,rpitchUV,(dx/2)/w);
					}
					for(w=dx;w<bwd;w++)
						CopyVLineYV12(Lp+w,Rp+w,bht,pitch,rpitch);
					for(w=dx/2;w<bwdUV;w++)
					{
						CopyVLineYV12(LpU+w,RpU+w,bhtUV,pitchUV,rpitchUV);
						CopyVLineYV12(LpV+w,RpV+w,bhtUV,pitchUV,rpitchUV);
					}
				}
				dropx= dropx & 0xFFFFFFFE;
				if(2*n>=overlap)
				{
					for(int w=1;w<bwd-dropx;w++)
						ResizeVLineYV12(Lp+pitch*bht/2+(dropx+w), 
										Rp+rpitch*bht/2+(w),
										w*dy/bwd,pitch,rpitch,bwd*bht/(w*dy));
					for(w=1;w<bwdUV-dropx/2;w++)
					{
						ResizeVLineYV12(LpU+pitchUV*bhtUV/2+(dropx/2+w), 
										RpU+rpitchUV*bhtUV/2+(w),
										w*(dy/2)/bwdUV,pitchUV,rpitchUV,bwdUV*bhtUV/(w*dy/2));
						ResizeVLineYV12(LpV+pitchUV*bhtUV/2+(dropx/2+w), 
										RpV+rpitchUV*bhtUV/2+(w),
										w*(dy/2)/bwdUV,pitchUV,rpitchUV,bwdUV*bhtUV/(w*dy/2));
					}
				}
			}

		}

		if(*dir=='r' || *dir=='R')
		{
			if(vi.IsRGB32())
			{
				if(2*n<overlap)
				{
					for(int w=1;w<dx;w++)
						ResizeVLineRGB32(Lp+pitch*bht/2+kb*(bwd-1-w),
										Rp+pitch*bht/2+kb*(bwd-1-w),
										w*bht/dx,pitch,pitch,dx/w);
					for(w=dx;w<bwd;w++)
						CopyVLineRGB32(Lp+kb*(bwd-1-w),Rp+kb*(bwd-1-w),bht,pitch,pitch);
				}
				if(2*n>=overlap)
					for(int w=1;w<bwd-dropx;w++)
						ResizeVLineRGB32(Lp+pitch*bht/2+(bwd-1-(dropx+w))*kb, 
										Rp+pitch*bht/2+(bwd-1-w)*kb,
										w*dy/bwd,pitch,pitch,bwd*bht/(w*dy));
			}
			if(vi.IsRGB24())
			{
				if(2*n<overlap)
				{
					for(int w=1;w<dx;w++)
						ResizeVLineRGB24(Lp+pitch*bht/2+kb*(bwd-1-w),
										Rp+pitch*bht/2+kb*(bwd-1-w),
										w*bht/dx,pitch,pitch,dx/w);
					for(w=dx;w<bwd;w++)
						CopyVLineRGB24(Lp+kb*(bwd-1-w),Rp+kb*(bwd-1-w),bht,pitch,pitch);
				}
				if(2*n>=overlap)
					for(int w=1;w<bwd-dropx;w++)
						ResizeVLineRGB24(Lp+pitch*bht/2+(bwd-1-(dropx+w))*kb, 
										Rp+pitch*bht/2+(bwd-1-w)*kb,
										w*dy/bwd,pitch,pitch,bwd*bht/(w*dy));
			}
			if(vi.IsYUY2())
			{
				if(2*n<overlap)
				{
					for(int w=2;w<dx;w+=2)
						ResizeVLineYUY2(Lp+pitch*bht/2+kb*(bwd-1-w),
										Rp+pitch*bht/2+kb*(bwd-1-w),
										w*bht/dx,pitch,pitch,dx/w);
					for(w=((int)dx)& 0xFFFFFFFE;w<bwd;w+=2)
						CopyVLineYUY2(Lp+kb*(bwd-1-w),Rp+kb*(bwd-1-w),bht,pitch,pitch);
				}
				dropx= dropx & 0xFFFFFFFE;
				if(2*n>=overlap)
					for(int w=2;w<bwd-dropx;w+=2)
						ResizeVLineYUY2(Lp+pitch*bht/2+(bwd-1-(dropx+w))*kb, 
										Rp+pitch*bht/2+(bwd-1-w)*kb,
										w*dy/bwd,pitch,pitch,bwd*bht/(w*dy));
			}

			if(vi.IsYV12())
			{
				if(2*n<overlap)
				{
					for(int w=1;w<dx;w++)
						ResizeVLineYV12(Lp+pitch*bht/2+(bwd-1-w),
										Rp+rpitch*bht/2+(bwd-1-w),
										w*bht/dx,pitch,rpitch,dx/w);
					for(w=dx;w<bwd;w++)
						CopyVLineYV12(Lp+(bwd-1-w),Rp+(bwd-1-w),bht,pitch,rpitch);
					for(w=1;w<dx/2;w++)
					{
						ResizeVLineYV12(LpU+pitchUV*bhtUV/2+(bwdUV-1-w),
										RpU+rpitchUV*bhtUV/2+(bwdUV-1-w),
										w*bhtUV/(dx/2),pitchUV,rpitchUV,(dx/2)/w);
						ResizeVLineYV12(LpV+pitchUV*bhtUV/2+(bwdUV-1-w),
										RpV+rpitchUV*bhtUV/2+(bwdUV-1-w),
										w*bhtUV/(dx/2),pitchUV,rpitchUV,(dx/2)/w);
					}
					for(w=dx/2;w<bwdUV;w++)
					{
						CopyVLineYV12(LpU+(bwdUV-1-w),RpU+(bwdUV-1-w),bhtUV,pitchUV,rpitchUV);
						CopyVLineYV12(LpV+(bwdUV-1-w),RpV+(bwdUV-1-w),bhtUV,pitchUV,rpitchUV);
					}
				}
				dropx= dropx & 0xFFFFFFFE;
				if(2*n>=overlap)
				{
					for(int w=1;w<bwd-dropx;w++)
						ResizeVLineYV12(Lp+pitch*bht/2+(bwd-1-(dropx+w)), 
										Rp+rpitch*bht/2+(bwd-1-w),
										w*dy/bwd,pitch,rpitch,bwd*bht/(w*dy));
					for(w=1;w<bwdUV-dropx/2;w++)
					{
						ResizeVLineYV12(LpU+pitchUV*bhtUV/2+(bwdUV-1-(dropx/2+w)), 
										RpU+rpitchUV*bhtUV/2+(bwdUV-1-w),
										w*(dy/2)/bwdUV,pitchUV,rpitchUV,bwdUV*bhtUV/(w*dy/2));
					
						ResizeVLineYV12(LpV+pitchUV*bhtUV/2+(bwdUV-1-(dropx/2+w)), 
										RpV+rpitchUV*bhtUV/2+(bwdUV-1-w),
										w*(dy/2)/bwdUV,pitchUV,rpitchUV,bwdUV*bhtUV/(w*dy/2));
					}
				}
			}

		}
										
		return RightFrame;
		

		
	
}

/********** 



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransFunnel(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransFunnel(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),		// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("right"),		// Transition direction up, down,left,right
					
																							
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransFunnel", "cci[dir]s", Create_TransFunnel, 0);
					
    return "TransFunnel  Plugin";
}
*/
