/******************************************************************************
TransPaint filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
modified on 17-1-06
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "FastORing.cpp"
*/
class TransPaint : public GenericVideoFilter {
		PClip RightClip;		// define the second clip
		const char *type;
		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		 
  public:
	  int *	xyarr;
	  bool array;
					//Definition of function
    TransPaint(PClip _child,PClip _RightClip,
		int _overlap,const char *_type, IScriptEnvironment* env) ;	
			
 	~TransPaint();		//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
	// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransPaint::TransPaint(PClip _child,PClip _RightClip,
					   int _overlap,const char * _type, IScriptEnvironment* env) :
	GenericVideoFilter(_child),RightClip(_RightClip),
		overlap(_overlap),type(_type) 
			
{
	char * Tname="TransPaint";
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	
		
	if(!(strcmp(type,"fence")==0) && !(strcmp(type,"paint")==0) &&
		!(strcmp(type,"rings")==0) && !(strcmp(type,"bricks")==0))
		env->ThrowError("TransPaint:valid Type options are fence, rings, bricks and paint only");
	
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransPaint:Clips have differing pixel_types");
	if(overlap>vi.width/2 && overlap>vi.height/2)
		env->ThrowError("TransPaint:overlaping frames not to exceed half of frame width and height");
	
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransPaint: This color format is not yet supported");
	if(vi.height != rvi.height || vi.width != rvi.width)
		env->ThrowError("TransPaint: The frame heights and widths of the clips are unequal");  
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

	if(strcmp(type,"fence")==0 && overlap> vi.width/2)
		env->ThrowError("%s: fence option overlap should not be greater than half of frame width in pixels  ", Tname);
	if(strcmp(type,"paint")==0 && overlap> vi.height/2)
		env->ThrowError("%s: paint option overlap should not be greater than half of frame height in pixels  ", Tname);
	array=false;

	
	

  
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransPaint::~TransPaint() 
{
	if(array)
		delete []xyarr;
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/
void TransPaint::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransPaint::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


	
/***************************************************************/	


PVideoFrame __stdcall TransPaint::GetFrame(int en, IScriptEnvironment* env)
{
				// This is the implementation of the constructor.
		
	if (en < video_fade_start)
				return child->GetFrame(en, env);
	if (en > video_fade_end)
	{
		if(array)
		{
			delete []xyarr;
			array = false;
		}
		return RightClip->GetFrame(en - video_fade_start, env);
	}
	int n=en-video_fade_start;
		
	PVideoFrame LeftFrame = child->GetFrame(en, env);

	PVideoFrame RightFrame = RightClip->GetFrame(n, env);
	const	int bwd = vi.width;
    const	int bht = LeftFrame->GetHeight();
	const	int lpitch = LeftFrame->GetPitch();
	const	int rpitch = RightFrame->GetPitch();
	const	int nframes = overlap;
	env->MakeWritable(&LeftFrame);
	unsigned char* lp= LeftFrame->GetWritePtr();
	const unsigned char * rp= RightFrame->GetReadPtr();
	
	const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
	const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
	const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
	const int bhtUV = LeftFrame->GetHeight(PLANAR_U);

	unsigned char *lpU, *lpV;
	const unsigned char  *rpU, *rpV;

	if(vi.IsYV12())
	{
		lpU = LeftFrame->GetWritePtr(PLANAR_U);
		lpV = LeftFrame->GetWritePtr(PLANAR_V);
		rpU = RightFrame->GetReadPtr(PLANAR_U);
		rpV = RightFrame->GetReadPtr(PLANAR_V);
	}
	int kb=vi.BitsPerPixel()/8;

	if(!array && (strcmp(type,"fence")==0 || strcmp(type,"paint")==0 || strcmp(type,"rings")==0) )
	{
	
		xyarr = new int [overlap+3];
		int i=0;
	
		// we need non repeating numbers
		while(i< overlap)
		{
			xyarr[i]= rand() % overlap;
			int j=0;
			for(j=0;j<i; j++)
				if(xyarr[i]== xyarr[j])
					break;
			if(j==i)
			
				i++;
		}
		// add 3 values to avoid mod in every access
		xyarr[overlap]=xyarr[0];
		xyarr[overlap+1]= xyarr[1];
		xyarr[overlap+2]= xyarr[2];
		array=true;
	}
		

	if(strcmp(type,"fence")==0)
	{

			
		double brw = (bwd)/overlap;	// board width
		
		int dy=8*bht/9;						// board height
		double dx=(bwd-brw)/overlap;		// board slant width
	
		
		// build fence post supporting structure
		for (int w=0;w<bwd;w++)
			for(int h=0;h<20+(w%5);h++)
			{
				for(int k=0; k<kb;k++)
				{
					*(lp+(bht/3+h+(w%5))*lpitch+w*kb+k)=*(rp+(bht/3+h)*lpitch+w*kb+k);
					*(lp+(3*bht/4+h+(w%7))*lpitch+w*kb+k)=*(rp+(3*bht/4+h)*lpitch+w*kb+k);
				}
				if((w &1 )==1 || (h & 1) ==1)
					continue;
				if(vi.IsYV12())
				{
					*(lpU+(bhtUV/3+h/2+(w%5)/2)*lpitchUV+w/2)=*(rpU+(bhtUV/3+h/2)*lpitchUV+w/2);
					*(lpU+(3*bhtUV/4+h/2+(w%7)/2)*lpitchUV+w/2)=*(rpU+(3*bhtUV/4+h/2)*lpitchUV+w/2);
					*(lpV+(bhtUV/3+h/2+(w%5)/2)*lpitchUV+w/2)=*(rpV+(bhtUV/3+h/2)*lpitchUV+w/2);
					*(lpV+(3*bhtUV/4+h/2+(w%7)/2)*lpitchUV+w/2)=*(rpV+(3*bhtUV/4+h/2)*lpitchUV+w/2);
				}
			}

		
		// build fence posts
		for(int stroke =0; stroke<= n; stroke++)
		{
			int sx=xyarr[stroke]*brw;					
			int sy = (xyarr[stroke+1]*bht)/(10*overlap);
			for( int h=sy; h<sy+dy;h++)
			{
				int w=(xyarr[stroke]*brw)+(h*dx)/dy;
				
			
				for(int b=0;b<brw;b++)
				{
					for(int k=0; k<kb;k++)
						if((h+b%10)*lpitch+kb*(b+w)+k>0 && (h+b%10)*lpitch+kb*(b+w)+k <bht*lpitch)
							*(lp+(h+b%10)*lpitch+kb*(b+w)+k)=*(rp+(h+b%10)*rpitch+kb*(b+w)+k);
				
					
					if((h & 1) ==1|| ((b+w)&1)==1 )
						continue;
					
					if(!vi.IsYV12())
						continue;
					if((h+b%10)*lpitch+(b+w)>0 && (h+b%10)*lpitch+(b+w)<bht*lpitch)
					{
						*(lpU+(h/2+(b%10)/2)*lpitchUV+(b+w)/2)
							=*(rpU+(h/2+(b%10)/2)*rpitchUV+(b+w)/2);
						*(lpV+(h/2+(b%10)/2)*lpitchUV+(b+w)/2)
							=*(rpV+(h/2+(b%10)/2)*rpitchUV+(b+w)/2);
					}
				}
						
			}
		}
	}
	if(strcmp(type,"paint")==0)
	{

		double brw = (bht/(overlap));	// brush width
		
							// paint brush starting x position
		
		int dx=(9*bwd/29)& 0xFFFFFFFE;

		int kb=vi.BitsPerPixel()/8;
		int dryup=bwd/10;	// paint brush dries up at end of stroke
					// left to right brush strokes drying up end
			for(int stroke =0; stroke<n+1; stroke++)
			{
				int sx = (xyarr[stroke]*bwd/(10*overlap)) & 0xFFFFFFFE;
				int sy = xyarr[stroke+1]*brw;

				for(int b=0;b<brw;b++)

					
					for(int w=sx;w<sx+dx+dryup*(b%5) && w<bwd;w++)
					{
						int h=sy+(w*2*brw)/dx;							

						if((h+b)>0 && (h+b)<bht)
						{
							for(int k=0; k<kb;k++)
							{
								*(lp+(h+b)*lpitch+kb*(w)+k)=*(rp+(h+b)*lpitch+kb*(w)+k);
								*(lp+(h+b)*lpitch+kb*(bwd-w)+k)=*(rp+(h+b)*lpitch+kb*(bwd-w)+k);
								
							}
							if(vi.IsYV12() && ((h+b)&1)==0 && (w&1)==0)
							{
								*(lpU+(h+b)/2*lpitchUV+w/2)=*(rpU+(h+b)/2*lpitchUV+w/2);
								*(lpV+(h+b)/2*lpitchUV+w/2)=*(rpV+(h+b)/2*lpitchUV+w/2);
								*(lpU+(h+b)/2*lpitchUV+bwdUV-w/2)=*(rpU+(h+b)/2*lpitchUV+bwdUV-w/2);
								*(lpV+(h+b)/2*lpitchUV+bwdUV-w/2)=*(rpV+(h+b)/2*lpitchUV+bwdUV-w/2);
							}
						}

					}
			}
	}

			// The rings function really does not belong as no such building element
	if(strcmp(type,"rings")==0)
	{
		double rt=(2.0*bwd)/overlap;				// ring thickness
		
		if(vi.IsYUY2())
			for(int stroke=0;stroke<n+1;stroke++)
			{
				int srad=(xyarr[stroke]*bht)/(4*overlap) +2*rt;
				int sx= (xyarr[stroke+1] *bwd)/overlap;
				int sy = (xyarr[stroke +2]*bht)/overlap;
				FastORingYUY2(rp,lp,srad,rt,
						sx,sy,bht,bwd,
						sx,sy,bht,bwd,
						rpitch,lpitch);
			}

		if(vi.IsRGB32())
			for(int stroke=0;stroke<n+1;stroke++)
			{
				int srad=(xyarr[stroke]*bht)/(4*overlap) +2*rt;
				int sx= (xyarr[stroke+1] *bwd)/overlap;
				int sy = (xyarr[stroke +2]*bht)/overlap;
				FastORingRGB32(rp,lp,srad,rt,
						sx,sy,bht,bwd,
						sx,sy,bht,bwd,
						rpitch,lpitch);
			}

		if(vi.IsRGB24())
			for(int stroke=0;stroke<n+1;stroke++)
			{
				int srad=(xyarr[stroke]*bht)/(4*overlap) +2*rt;
				int sx= (xyarr[stroke+1] *bwd)/overlap;
				int sy = (xyarr[stroke +2]*bht)/overlap;
				FastORingRGB24(rp,lp,srad,rt,
						sx,sy,bht,bwd,
						sx,sy,bht,bwd,
						rpitch,lpitch);
			}
		if(vi.IsYV12())
			for(int stroke=0;stroke<n+1;stroke++)
			{
				int srad=(xyarr[stroke]*bht)/(4*overlap) +2*rt;
				int sx= (xyarr[stroke+1] *bwd)/overlap;
				int sy = (xyarr[stroke +2]*bht)/overlap;
				FastORingYV12(rp,lp,srad,rt,
						sx,sy,bht,bwd,
						sx,sy,bht,bwd,
						rpitch,lpitch);
			
				FastORingYV12(rpU,lpU,srad/2,rt/2,
						sx/2,sy/2,bhtUV,bwdUV,
						sx/2,sy/2,bhtUV,bwdUV,
						rpitchUV,lpitchUV);
				FastORingYV12(rpV,lpV,srad/2,rt/2,
						sx/2,sy/2,bhtUV,bwdUV,
						sx/2,sy/2,bhtUV,bwdUV,
						rpitchUV,lpitchUV);
			}



	}
	if(strcmp(type,"bricks")==0)
	{
		int brw=1+bwd/overlap;// brick width
		int brh=bht/overlap;	// brick height
		if(vi.IsYUY2())		
			for(int nbw=0; nbw<n+1;nbw++)
				for(int nbh=0;nbh<n; nbh++)								
					for(int bw=(nbh%2)*brw/2;bw<(nbw +nbh +1)*brw -1+(nbh%2)*brw/2;bw+=brw)
						for(int h=nbh*brh;h<(nbh+1)*brh-1;h++)
							if(h<(n+1)*brh )							
								for(int w=bw;w<bw+brw-1;w++)
									if(w<bwd)
										for(int k=0; k<kb;k++)
											*(lp+(bht-(1+n)*brh+h)*lpitch+2*w+k)
												=*(rp+(bht-(1+n)*brh+h)*lpitch+2*w+k);

		if(vi.IsYV12())
		{
			brh=brh & 0xfffffffe;
			brw= brw & 0xfffffffe;
			for(int nbw=0; nbw<n+1;nbw++)
				for(int nbh=0;nbh<n; nbh++)								
					for(int bw=(nbh%2)*brw/2;bw<(nbw +nbh +1)*brw -1+(nbh%2)*brw/2;bw+=brw)
						for(int h=nbh*brh;h<(nbh+1)*brh-1;h++)
							if(h<(n+1)*brh )							
								for(int w=bw;w<bw+brw-1;w++)
									if(w<bwd)
									{										
										*(lp+(bht-(1+n)*brh+h)*lpitch+w)
												=*(rp+(bht-(1+n)*brh+h)*lpitch+w);
										if((w & 1)==1 || ((n*brh+h) & 1)==1)
											continue;
										*(lpU+(bht-(1+n)*brh+h)/2*lpitchUV+w/2)
												=*(rpU+(bht-(1+n)*brh+h)/2*lpitchUV+w/2);
										*(lpV+(bht-(1+n)*brh+h)/2*lpitchUV+w/2)
												=*(rpV+(bht-(1+n)*brh+h)/2*lpitchUV+w/2);
									}
		}

		if( vi.IsRGB32() || vi.IsRGB24())
			for(int nbw=0; nbw<n+1;nbw++)
				for(int nbh=0;nbh<n+1; nbh++)								
					for(int bw=0;bw<(nbw+nbh+1)*brw;bw+=brw)
						for(int h=nbh*brh;h<nbh*brh+brh-1;h++)
							if(((n+1)*brh-h)<bht-1 )						
								for(int w=bw+(nbh%2)*brw/2;w<bw+brw-1+(nbh%2)*brw/2;w++)
									if( w<bwd)
										for(int k=0; k<kb;k++)
											*(lp+((n+1)*brh-h)*lpitch+kb*w+k)
												=*(rp+((n+1)*brh-h)*lpitch+kb*w+k);
	

	}
	return LeftFrame;
}

		
		

/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransPaint(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransPaint(args[0].AsClip(),	// clip as LeftClip
								args[1].AsClip(), // as right clip
								args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
								args[3].AsString("paint"),	// paint, fence, bricks, rings
								env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransPaint", "cci[type]s", Create_TransPaint, 0);
				
    return "TransPaint Transition Plugin";
}

*/