/******************************************************************************
TransPeel filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also 
used partly adapted function coding 
from Avisynth and simple sample. Roll Diameter and
 shading along roll length are other params
 Modified on 23 sep 2004
********************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "LineZ.cpp"
*/
class TransPeel : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;
		const int Maxroll;
		const int shade;
		const bool pull;
		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransPeel(PClip _child, PClip _RightClip,
				int _overlap,const char* _dir,
				const int _maxroll, const int _shade,const bool _pull,  
				IScriptEnvironment* env) ;	
			
 	~TransPeel();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);  

};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
  


	

//Here is the acutal constructor code used

TransPeel::TransPeel(PClip _child, PClip _RightClip,
					 int _overlap,const char* _dir,
					 const int _maxroll, const int _shade,const bool _pull,
					 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
						overlap( _overlap),dir(_dir),
						Maxroll(_maxroll), shade(_shade), pull(_pull)
			
{
	
const VideoInfo& rvi= RightClip->GetVideoInfo();
char * Tname="TransPeel";


if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransPeel:Clips have differing pixel_types");

if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2()&& !vi.IsYV12())
	env->ThrowError("TransPeel: This color format is not supported here");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("TransPeel: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransPeel: Height/width of frame are not multiple of 4");
if(!(*dir=='l') && !(*dir=='L') && !(*dir=='r') && !(*dir=='R') &&
	!(*dir=='u') && !(*dir=='U') && !(*dir=='d') && !(*dir=='D'))
	env->ThrowError("TransPeel:up, down,left or right are valid directions");
if(Maxroll<8)
   env->ThrowError("TransPeel: roll diameter is too small. Least is 8");
if(vi.width<vi.height)
	if(Maxroll>vi.width/4)
		env->ThrowError("TransPeel: roll diameter is large. Limit to 1/4 of width=%d",vi.width);
if(vi.width>=vi.height)
	if(Maxroll>vi.height/4)
		env->ThrowError("TransPeel: roll diameter is large. Limit to 1/4 of height = %d", vi.height);
if(*dir=='l' || *dir=='L' || *dir=='r' || *dir=='R')
	if(Maxroll>vi.width/4)
		env->ThrowError("TransPeel: roll diameter is large. Max 1/4 of width.Opt is 120");
if(*dir=='u' || *dir=='U' || *dir=='d' || *dir=='D')
	if(Maxroll>vi.width/4)
		env->ThrowError("TransPeel: roll diameter is large. Limit to 1/4 of height. Opt is 120");
if(shade<-255 || shade >255)
	env->ThrowError("TransPeel:shade can be between -255 and 255 only");
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}// This is where any actual destructor code used goes
TransPeel::~TransPeel() {
	
// This is where you can deallocate any memory you might have used.
}

//_____________________________________________________________________

void TransPeel::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransPeel::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


	
/***************************************************************/	
	PVideoFrame __stdcall TransPeel::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		int maxroll=Maxroll & 0xfffffffc;		 
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		int deltay =((n+1)*bht)/(overlap+1);
		deltay = deltay & 0xFFFFFFFC;
		int deltax = ((n+1)*bwd)/(overlap+1);
		deltax=deltax & 0xFFFFFFFC;
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		int  wht, wwd;
		env->MakeWritable(&RightFrame);
		unsigned char* RightFramep= RightFrame->GetWritePtr();
		const unsigned char * LeftFramep= LeftFrame->GetReadPtr();

		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
		const int bhtUV = LeftFrame->GetHeight(PLANAR_U);

		const unsigned char *LeftFramepU, *LeftFramepV;
		 unsigned char  *RightFramepU, *RightFramepV;

		if(vi.IsYV12())
		{
			LeftFramepU = LeftFrame->GetReadPtr(PLANAR_U);
			LeftFramepV = LeftFrame->GetReadPtr(PLANAR_V);
			RightFramepU = RightFrame->GetWritePtr(PLANAR_U);
			RightFramepV = RightFrame->GetWritePtr(PLANAR_V);
		}

		if(*dir=='d' || *dir=='D')
			
		{			//LeftFrame peels downward unmasking RightFrame
			if(deltay<8)
				return LeftFrame;
			wht=deltay/2;
			if(deltay>maxroll)
				wht=maxroll/2;
			if(bht-deltay<maxroll)
				wht=(bht-deltay)/2;
			if(wht<4)
				return RightFrame;

			if(vi.IsRGB32())
			{
				if(!pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB32(LeftFramep+(bht-1-deltay+wht)*lpitch+4*w,
									RightFramep+(bht-1-deltay-wht/2)*rpitch+4*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB32(LeftFramep+4*w,
									RightFramep+4*w,
									(bht-deltay-wht),lpitch,rpitch);
						
					}
				if(pull)
						// LeftFrame rolls and pulls downward unmasking RightFrame
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB32(LeftFramep+(deltay-wht)*lpitch+4*w,
									RightFramep+(wht/2)*rpitch+4*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB32(LeftFramep+(deltay+wht)*lpitch+4*w,
									RightFramep+(wht)*rpitch+4*w,
									bht-1-deltay-wht,lpitch,rpitch);
					}

				return RightFrame;
			}
			if(vi.IsRGB24())
			{
				if(!pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB24(LeftFramep+(bht-1-deltay+wht)*lpitch+3*w,
									RightFramep+(bht-1-deltay-wht/2)*rpitch+3*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB24(LeftFramep+3*w,
									RightFramep+3*w,
									(bht-deltay-wht),lpitch,rpitch);
						
					}
				if(pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB24(LeftFramep+(deltay-wht)*lpitch+3*w,
									RightFramep+(wht/2)*rpitch+3*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB24(LeftFramep+(deltay+wht)*lpitch+3*w,
									RightFramep+(wht)*rpitch+3*w,
									bht-1-deltay-wht,lpitch,rpitch);
					}

				return RightFrame;
			}
			if(vi.IsYUY2())
			{
				if(!pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYUY2(LeftFramep+(deltay-wht)*lpitch+2*w,
									RightFramep+(deltay+wht/2)*rpitch+2*w,
									wht,lpitch,rpitch,shade);
						CopyVLineYUY2(LeftFramep+(deltay+wht)*lpitch+2*w,
									RightFramep+(deltay+wht)*rpitch+2*w,
									bht-1-deltay-wht,lpitch,rpitch);
						
					}
				if(pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYUY2(LeftFramep+(bht-1-deltay+wht)*lpitch+2*w,
									RightFramep+(bht-1-wht/2)*rpitch+2*w,
									wht,lpitch,rpitch,shade);
						CopyVLineYUY2(LeftFramep+2*w,
									RightFramep+(deltay)*rpitch+2*w,
									(bht-deltay-wht),lpitch,rpitch);
					}
				return RightFrame;
			}

			if(vi.IsYV12())
			{
				if(!pull)
				{
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYV12(LeftFramep+(deltay-wht)*lpitch+w,
									RightFramep+(deltay+wht/2)*rpitch+w,
									wht,lpitch,rpitch,shade);
						CopyVLineYV12(LeftFramep+(deltay+wht)*lpitch+w,
									RightFramep+(deltay+wht)*rpitch+w,
									bht-1-deltay-wht,lpitch,rpitch);
						
					}
					for(w=0;w<bwdUV;w++)
					{
						PeelzVLineYV12(LeftFramepU+(deltay-wht)/2*lpitchUV+w,
									RightFramepU+(deltay+wht/2)/2*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepU+(deltay+wht)/2*lpitchUV+w,
									RightFramepU+(deltay+wht)/2*rpitchUV+w,
									(bht-1-deltay-wht)/2,lpitchUV,rpitchUV);
						PeelzVLineYV12(LeftFramepV+(deltay-wht)/2*lpitchUV+w,
									RightFramepV+(deltay+wht/2)/2*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepV+(deltay+wht)/2*lpitchUV+w,
									RightFramepV+(deltay+wht)/2*rpitchUV+w,
									(bht-1-deltay-wht)/2,lpitchUV,rpitchUV);
						
					}
				}
				if(pull)
				{
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYV12(LeftFramep+(bht-1-deltay+wht)*lpitch+w,
									RightFramep+(bht-1-wht/2)*rpitch+w,
									wht,lpitch,rpitch,shade);
						CopyVLineYV12(LeftFramep+w,
									RightFramep+(deltay)*rpitch+w,
									(bht-deltay-wht),lpitch,rpitch);
					}
					for(w=0;w<bwdUV;w++)
					{
						PeelzVLineYV12(LeftFramepU+(bht-1-deltay+wht)/2*lpitchUV+w,
									RightFramepU+(bht-1-wht/2)/2*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepU+w,
									RightFramepU+(deltay/2)*rpitchUV+w,
									(bht-deltay-wht)/2,lpitchUV,rpitchUV);
						PeelzVLineYV12(LeftFramepV+(bht-1-deltay+wht)/2*lpitchUV+w,
									RightFramepV+(bht-1-wht/2)/2*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepV+w,
									RightFramepV+(deltay/2)*rpitchUV+w,
									(bht-deltay-wht)/2,lpitchUV,rpitchUV);
					}
				}
				return RightFrame;
			}

		}
			

		if(*dir=='u' || *dir=='U')
			
		{			//LeftFrame peels or rolls and pulls up unmasking RightFrame
				
			if(deltay<8)
				return LeftFrame;
			wht=deltay/2;
			if(deltay>maxroll)
				wht=maxroll/2;
			if(bht-deltay<maxroll)
				wht=(bht-deltay)/2;
			if(wht<4)
				return RightFrame;

			if(vi.IsRGB32())
			{
				if(!pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB32(LeftFramep+(deltay-wht)*lpitch+4*w,
									RightFramep+(deltay+wht/2)*rpitch+4*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB32(LeftFramep+(deltay+wht)*lpitch+4*w,
									RightFramep+(deltay+wht)*rpitch+4*w,
									bht-1-deltay-wht,lpitch,rpitch);
					
					}
				if(pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB32(LeftFramep+(bht-1-deltay+wht)*lpitch+4*w,
									RightFramep+(bht-1-wht/2)*rpitch+4*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB32(LeftFramep+4*w,
									RightFramep+(deltay)*rpitch+4*w,
									(bht-deltay-wht),lpitch,rpitch);
					}
				return RightFrame;
			}
			if(vi.IsRGB24())
			{
				if(!pull)			
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB24(LeftFramep+(deltay-wht)*lpitch+3*w,
									RightFramep+(deltay+wht/2)*rpitch+3*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB24(LeftFramep+(deltay+wht)*lpitch+3*w,
									RightFramep+(deltay+wht)*rpitch+3*w,
									bht-1-deltay-wht,lpitch,rpitch);
						
					}
				if(pull)			
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineRGB24(LeftFramep+(bht-1-deltay+wht)*lpitch+3*w,
									RightFramep+(bht-1-wht/2)*rpitch+3*w,
									wht,lpitch,rpitch,shade);
						CopyVLineRGB24(LeftFramep+3*w,
									RightFramep+(deltay)*rpitch+3*w,
									(bht-deltay-wht),lpitch,rpitch);
					}

				return RightFrame;
			}
			if(vi.IsYUY2())
			{
				if(!pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYUY2(LeftFramep+(bht-1-deltay+wht)*lpitch+2*w,
									RightFramep+(bht-1-deltay-wht/2)*rpitch+2*w,
									wht,lpitch,rpitch,shade);
						CopyVLineYUY2(LeftFramep+2*w,
									RightFramep+2*w,
									(bht-deltay-wht),lpitch,rpitch);
						
					}
				if(pull)
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYUY2(LeftFramep+(deltay-wht)*lpitch+2*w,
									RightFramep+(wht/2)*rpitch+2*w,
									wht,lpitch,rpitch,shade);
						CopyVLineYUY2(LeftFramep+(deltay+wht)*lpitch+2*w,
									RightFramep+(wht)*rpitch+2*w,
									bht-1-deltay-wht,lpitch,rpitch);
					}

				return RightFrame;
			}
			if(vi.IsYV12())
			{
				if(!pull)
				{
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYV12(LeftFramep+(bht-1-deltay+wht)*lpitch+w,
									RightFramep+(bht-1-deltay-wht/2)*rpitch+w,
									wht,lpitch,rpitch,shade);
						CopyVLineYV12(LeftFramep+w,
									RightFramep+w,
									(bht-deltay-wht),lpitch,rpitch);
						
					}
					for(w=0;w<bwdUV;w++)
					{
						PeelzVLineYV12(LeftFramepU+(bht-1-deltay+wht)/2*lpitchUV+w,
									RightFramepU+(bht-1-deltay-wht/2)/2*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepU+w,
									RightFramepU+w,
									(bht-deltay-wht)/2,lpitchUV,rpitchUV);
						PeelzVLineYV12(LeftFramepV+(bht-1-deltay+wht)/2*lpitchUV+w,
									RightFramepV+(bht-1-deltay-wht/2)/2*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepV+w,
									RightFramepV+w,
									(bht-deltay-wht)/2,lpitchUV,rpitchUV);
						
					}
				}
				if(pull)
				{
					for(int w=0;w<bwd;w++)
					{
						PeelzVLineYV12(LeftFramep+(deltay-wht)*lpitch+w,
									RightFramep+(wht/2)*rpitch+w,
									wht,lpitch,rpitch,shade);
						CopyVLineYV12(LeftFramep+(deltay+wht)*lpitch+w,
									RightFramep+(wht)*rpitch+w,
									bht-1-deltay-wht,lpitch,rpitch);
					}
					for(w=0;w<bwdUV;w++)
					{
						PeelzVLineYV12(LeftFramepU+(deltay-wht)/2*lpitchUV+w,
									RightFramepU+(wht/4)*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepU+(deltay+wht)/2*lpitchUV+w,
									RightFramepU+(wht/2)*rpitch+w,
									(bht-1-deltay-wht)/2,lpitchUV,rpitchUV);
						PeelzVLineYV12(LeftFramepV+(deltay-wht)/2*lpitchUV+w,
									RightFramepV+(wht/4)*rpitchUV+w,
									wht/2,lpitchUV,rpitchUV,0);
						CopyVLineYV12(LeftFramepV+(deltay+wht)/2*lpitchUV+w,
									RightFramepV+(wht/2)*rpitch+w,
									(bht-1-deltay-wht)/2,lpitchUV,rpitchUV);
					}
				}

				return RightFrame;
			}

		}

			
		if(*dir=='r' || *dir=='R')
			
		{		//LeftFrame peels out Right/rolls and pulls Right unmasking RightFrame
			if(deltax<8)
				return LeftFrame;
			wwd=deltax/2;
			if(deltax>maxroll)
				wwd=maxroll/2;
			if(bwd-deltax<maxroll)
				wwd=(bwd-deltax)/2;
			if(wwd<4)
				return RightFrame;

			if(vi.IsRGB32())
			{
				if(!pull)
					for(int h=0;h<bht-1;h++)				
					{
						PeelzHLineRGB32(LeftFramep+4*(deltax-wwd)+h*rpitch,
									RightFramep+4*(deltax+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+h*lpitch+4*(deltax+wwd),
									RightFramep+h*rpitch+4*(deltax+wwd),
										bwd-deltax-wwd);
					}
				if(pull)
					for(int h=0;h<bht-1;h++)				
					{
						PeelzHLineRGB32(LeftFramep+4*(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+4*(bwd-1-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+h*lpitch,
									RightFramep+h*rpitch+4*deltax,
										bwd-deltax-wwd);
						
					}

				return RightFrame;
			}
			if(vi.IsRGB24())
			{
				if(!pull)
				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineRGB24(LeftFramep+3*(deltax-wwd)+h*rpitch,
									RightFramep+3*(deltax+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+h*lpitch+3*(wwd+deltax),
									RightFramep+h*rpitch+3*(wwd+deltax),
										bwd-deltax-wwd);
					}
				if(pull)
				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineRGB24(LeftFramep+3*(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+3*(bwd-1-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+h*lpitch,
									RightFramep+h*rpitch+3*deltax,
										bwd-deltax-wwd);
						
					}

				return RightFrame;
			}
			if(vi.IsYUY2())
			{
				if(!pull)
					for(int h=0;h<bht-1;h++)				
					{
						PeelzHLineYUY2(LeftFramep+2*(deltax-wwd)+h*rpitch,
									RightFramep+2*(deltax+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+h*lpitch+2*(deltax+wwd),
									RightFramep+h*rpitch+2*(deltax+wwd),
										bwd-deltax-wwd);
					}
				if(pull)
					for(int h=0;h<bht-1;h++)				
					{
						PeelzHLineYUY2(LeftFramep+2*(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+2*(bwd-1-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+h*lpitch,
									RightFramep+h*rpitch+2*deltax,
										bwd-deltax-wwd);
						
					}

				return RightFrame;
			}

			if(vi.IsYV12())
			{
				if(!pull)
				{
					for(int h=0;h<bht-1;h++)				
					{
						PeelzHLineYV12(LeftFramep+(deltax-wwd)+h*rpitch,
									RightFramep+(deltax+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+h*lpitch+(deltax+wwd),
									RightFramep+h*rpitch+(deltax+wwd),
										bwd-deltax-wwd);
					}
					for(h=0;h<bhtUV;h++)				
					{
						PeelzHLineYV12(LeftFramepU+(deltax-wwd)/2+h*rpitchUV,
									RightFramepU+(deltax+wwd/2)/2+h*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+h*lpitchUV+(deltax+wwd)/2,
									RightFramepU+h*rpitchUV+(deltax+wwd)/2,
										(bwd-deltax-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(deltax-wwd)/2+h*rpitchUV,
									RightFramepV+(deltax+wwd/2)/2+h*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+h*lpitchUV+(deltax+wwd)/2,
									RightFramepV+h*rpitchUV+(deltax+wwd)/2,
										(bwd-deltax-wwd)/2);
					}
				}
				if(pull)
				{
					for(int h=0;h<bht-1;h++)				
					{
						PeelzHLineYV12(LeftFramep+(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+(bwd-1-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+h*lpitch,
									RightFramep+h*rpitch+deltax,
										bwd-deltax-wwd);
						
					}

					for(h=0;h<bht-1;h++)				
					{
						PeelzHLineYV12(LeftFramepU+(bwd-1-deltax+wwd)/2+h*lpitchUV,
									RightFramepU+(bwd-1-wwd/2)/2+h*rpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+h*lpitchUV,
									RightFramepU+h*rpitchUV+deltax/2,
										(bwd-deltax-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(bwd-1-deltax+wwd)/2+h*lpitchUV,
									RightFramepV+(bwd-1-wwd/2)/2+h*rpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+h*lpitchUV,
									RightFramepV+h*rpitchUV+deltax/2,
										(bwd-deltax-wwd)/2);
						
					}
				}

				return RightFrame;
			}

		}
			

		if(*dir=='l' || *dir=='L')
			
		{			//LeftFrame moves westward unmasking RightFrame
			if(deltax<8)
				return LeftFrame;
			wwd=deltax/2;
			if(deltax>maxroll)
				wwd=maxroll/2;
			if(bwd-deltax<maxroll)
				wwd=(bwd-deltax)/2;
			if(wwd<4)
				return RightFrame;

			if(vi.IsRGB32())
			{
				if(!pull)
				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineRGB32(LeftFramep+4*(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+4*(bwd-1-deltax-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax-wwd);
					}
				if(pull)
				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineRGB32(LeftFramep+4*(deltax-wwd)+h*rpitch,
									RightFramep+4*(wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+h*lpitch+4*(deltax+wwd),
									RightFramep+h*rpitch+4*(wwd),
										bwd-deltax-wwd);
						
					}

				return RightFrame;
			}
			if(vi.IsRGB24())
			{
				if(!pull)
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineRGB24(LeftFramep+3*(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+3*(bwd-1-deltax-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax-wwd);
					}
				if(pull)
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineRGB24(LeftFramep+3*(deltax-wwd)+h*rpitch,
									RightFramep+3*(wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+h*lpitch+3*(wwd+deltax),
									RightFramep+h*rpitch+3*(wwd),
										bwd-deltax-wwd);
						
					}

				return RightFrame;
			}
			if(vi.IsYUY2())
			{
				if(!pull)
				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineYUY2(LeftFramep+2*(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+2*(bwd-1-deltax-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax-wwd);
					}
				if(pull)
				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineYUY2(LeftFramep+2*(deltax-wwd)+h*rpitch,
									RightFramep+2*(wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+h*lpitch+2*(deltax+wwd),
									RightFramep+h*rpitch+2*(wwd),
										bwd-deltax-wwd);
						
					}

				return RightFrame;
			}
			if(vi.IsYV12())
			{
				if(!pull)
				{
				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineYV12(LeftFramep+(bwd-1-deltax+wwd)+h*lpitch,
									RightFramep+(bwd-1-deltax-wwd/2)+h*rpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax-wwd);
					}
					for(h=0;h<bhtUV;h++)
					{
						PeelzHLineYV12(LeftFramepU+(bwd-1-deltax+wwd)/2+h*lpitchUV,
									RightFramepU+(bwd-1-deltax-wwd/2)/2+h*rpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+h*lpitchUV,
									RightFramepU+h*rpitchUV,
										(bwd-deltax-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(bwd-1-deltax+wwd)/2+h*lpitchUV,
									RightFramepV+(bwd-1-deltax-wwd/2)/2+h*rpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+h*lpitchUV,
									RightFramepV+h*rpitchUV,
										(bwd-deltax-wwd)/2);
					}
				}
				if(pull)
				{				
					for(int h=0;h<bht-1;h++)
					{
						PeelzHLineYV12(LeftFramep+(deltax-wwd)+h*rpitch,
									RightFramep+(wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+h*lpitch+(deltax+wwd),
									RightFramep+h*rpitch+(wwd),
										bwd-deltax-wwd);
						
					}
					for(h=0;h<bhtUV;h++)
					{
						PeelzHLineYV12(LeftFramepU+(deltax-wwd)/2+h*rpitchUV,
									RightFramepU+(wwd/4)+h*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+h*lpitchUV+(deltax+wwd)/2,
									RightFramepU+h*rpitchUV+(wwd/2),
										(bwd-deltax-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(deltax-wwd)/2+h*rpitchUV,
									RightFramepV+(wwd/4)+h*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+h*lpitchUV+(deltax+wwd)/2,
									RightFramepV+h*rpitchUV+(wwd/2),
										(bwd-deltax-wwd)/2);
						
					}
				}

				return RightFrame;
			}
		}
		env->ThrowError("TransPeel: Invalid direction.Must be either  UP, Down, Left, Right");
		
	}
	

/********** 



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransPeel(AVSValue args, void* user_data, IScriptEnvironment* env) 
{


				
	return new TransPeel(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("up"),	// Transition direction
						args[4].AsInt(80),			// diameter of roll default 80
						args[5].AsInt(127),				// shade.-255 to 0 noshading to Max 255 default -127
						args[6].AsBool(false),				// true : roll and pull, (default)False : roll only
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransPeel", "cci[dir]s[rollDia]i[shade]i[pull]b", Create_TransPeel, 0);
				
    return "TransPeel  Plugin";
}

*/