/******************************************************************************
TransPush filter plugin for Avisynth by V.C.Mohan
The  Right Clip pushes out the Left Clip in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 

********************************************************************************/  
/*#include "windows.h"
#include "avisynth.h"
#include "LineZ.cpp"
*/
class TransPush : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;

		int overlap;
		void * abuf;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		
		 
  public:
							//Definition of function
    TransPush(PClip _child, PClip _RightClip,
				int _overlap, const char* _dir,
				  
				IScriptEnvironment* env) ;	
			
 	~TransPush();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
bool __stdcall GetParity(int n);

};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
void TransPush::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
  
 
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransPush::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//Here is the acutal constructor code used

TransPush::TransPush(PClip _child, PClip _RightClip,
					 int _overlap,const char* _dir,
					 
					 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
					overlap(_overlap),dir(_dir)
						
			
{
char * Tname="TransPush";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransPush:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransPush: This color format is not supported here");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("TransPush: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransPush: Height/width of frame are not multiple of 4");
if(!(*dir=='l') && !(*dir=='L') && !(*dir=='r') && !(*dir=='R') &&
	!(*dir=='u') && !(*dir=='U') && !(*dir=='d') && !(*dir=='D'))
	env->ThrowError("TransPush:up, down,left or right are valid directions");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);

	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransPush::~TransPush() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
	PVideoFrame __stdcall TransPush::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
			return child->GetFrame(en, env);
		if (en > video_fade_end)
			return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
				 
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		int deltay = ((n+1)*bht)/(overlap+1);
		deltay = deltay & 0xFFFFFFFC;
		int deltax = ((n+1)*bwd)/(overlap+1);
		deltax=deltax & 0xFFFFFFFC;
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		int  wht, wwd;
		
		const unsigned char* RightFramep= RightFrame->GetReadPtr();
		const unsigned char * LeftFramep= LeftFrame->GetReadPtr();
		const unsigned char * lpU, *lpV, *rpU, *rpV;
		unsigned char *wpU, *wpV;
		const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
		const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		PVideoFrame work = env->NewVideoFrame(vi);
		unsigned char* wp=work->GetWritePtr();
		const	int wpitch = work->GetPitch();
		const int wpitchUV=work->GetPitch(PLANAR_U);
		if(vi.IsYV12())
		{
				rpU = RightFrame->GetReadPtr(PLANAR_U);
				lpU = LeftFrame->GetReadPtr(PLANAR_U);
				rpV = RightFrame->GetReadPtr(PLANAR_V);
				lpV = LeftFrame->GetReadPtr(PLANAR_V);
				wpU = work->GetWritePtr(PLANAR_U);
				wpV=  work->GetWritePtr(PLANAR_V);
		}
				

		if(*dir=='u' || *dir=='U')
			
		{			//RightFrame pushes left frame upward  
			if(deltay<2)
				return LeftFrame;
		//	if(deltay>bht-2)
		//		return RightFrame;
			
		
			if(vi.IsRGB32())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineRGB32(LeftFramep+deltay*lpitch+4*w,
									wp+deltay*wpitch+4*w,
									bht-deltay,lpitch,wpitch);
					CopyVLineRGB32(RightFramep+(bht-deltay)*rpitch+4*w,
									wp+4*w,
									deltay,rpitch,wpitch);
				}
				
			}
			if(vi.IsRGB24())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineRGB24(LeftFramep+deltay*lpitch+3*w,
									wp+deltay*wpitch+3*w,
									bht-deltay,lpitch,wpitch);
					CopyVLineRGB24(RightFramep+(bht-deltay)*rpitch+3*w,
									wp+3*w,
									deltay,rpitch,wpitch);
					
				}
			
			}
			if(vi.IsYUY2())
			{
				for(int w=0;w<bwd;w+=2)
				{
					CopyVLineYUY2(LeftFramep+(deltay)*lpitch+2*w,
									wp+2*w,
									bht-deltay,lpitch,wpitch);
					CopyVLineYUY2(RightFramep+2*w,
									wp+(bht-deltay)*wpitch+2*w,
									deltay,rpitch,wpitch);
				}
		
			}
			if(vi.IsYV12())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineYV12(LeftFramep+(deltay)*lpitch+w,
									wp+w,
									bht-deltay,lpitch,wpitch);
					CopyVLineYV12(RightFramep+w,
									wp+(bht-deltay)*wpitch+w,
									deltay,rpitch,wpitch);
					if((w & 1)==1)
						continue;
					CopyVLineYV12(lpU+(deltay)/2*lpitchUV+w/2,
									wpU+w/2,
									(bht-deltay)/2,lpitchUV,wpitchUV);
					CopyVLineYV12(lpV+(deltay)/2*lpitchUV+w/2,
									wpV+w/2,
									(bht-deltay)/2,lpitchUV,wpitchUV);
					CopyVLineYUY2(rpU+w/2,
									wpU+(bht-deltay)/2*wpitchUV+w/2,
									deltay/2,rpitchUV,wpitchUV);
					CopyVLineYUY2(rpV+w/2,
									wpV+(bht-deltay)/2*wpitchUV+w/2,
									deltay/2,rpitchUV,wpitchUV);
				}
	
			}

		}
			

		if(*dir=='d' || *dir=='D')
			
		{			//LeftFrame peels down unmasking RightFrame
				
			if(deltay<2)
				return LeftFrame;
			
			if(vi.IsRGB32())
			{
				for(int w=0;w<bwd;w++)
				{
				
					CopyVLineRGB32(LeftFramep+(deltay)*lpitch+4*w,
									wp+4*w,
									bht-deltay,rpitch,wpitch);
					CopyVLineRGB32(RightFramep+4*w,
									wp+(bht-deltay)*wpitch+4*w,
									deltay,rpitch,wpitch);
				}
			
			}
			if(vi.IsRGB24())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineRGB24(LeftFramep+(deltay)*lpitch+3*w,
									wp+3*w,
									bht-deltay,rpitch,wpitch);
					CopyVLineRGB24(RightFramep+3*w,
									wp+(bht-deltay)*wpitch+3*w,
									deltay,rpitch,wpitch);
				}
			
			}
		
			if(vi.IsYUY2())
			{
				for(int w=0;w<bwd;w+=2)
				{
					
					CopyVLineYUY2(LeftFramep+2*w,
									wp+deltay*wpitch+2*w,
									bht-deltay,lpitch,wpitch);
					CopyVLineYUY2(RightFramep+(bht-deltay)*rpitch+2*w,
									wp+2*w,
									deltay,rpitch,wpitch);
				}
			
			}
			if(vi.IsYV12())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineYV12(LeftFramep+w,
									wp+deltay*wpitch+w,
									bht-deltay,lpitch,wpitch);
					CopyVLineYV12(RightFramep+(bht-deltay)*rpitch+w,
									wp+w,
									deltay,rpitch,wpitch);
					if((w & 1)==1)
						continue;
					CopyVLineYV12(lpU+w/2,
									wpU+deltay/2*wpitchUV+w/2,
									(bht-deltay)/2,lpitchUV,wpitchUV);
					CopyVLineYV12(rpU+(bht-deltay)/2*rpitchUV+w/2,
									wpU+w/2,
									deltay/2,rpitchUV,wpitchUV);

					CopyVLineYV12(lpV+w/2,
									wpV+deltay/2*wpitchUV+w/2,
									(bht-deltay)/2,lpitchUV,wpitchUV);
					CopyVLineYV12(rpV+(bht-deltay)/2*rpitchUV+w/2,
									wpV+w/2,
									deltay/2,rpitchUV,wpitchUV);
				}
		
			}

		}
			
		if(*dir=='r' || *dir=='R')
			
		{		//LeftFrame is pushed out by RightFrame
			if(deltax<2)
				return LeftFrame;
		
			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)				
				{
					CopyHLineRGB32(LeftFramep+h*lpitch,
									wp+h*wpitch+4*deltax,
										bwd-deltax);
					CopyHLineRGB32(RightFramep+h*rpitch+(bwd-deltax)*4,
									wp+h*wpitch,
										deltax);
				}
	
			}
			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineRGB24(LeftFramep+h*lpitch,
									wp+h*wpitch+3*deltax,
										bwd-deltax);
					CopyHLineRGB24(RightFramep+h*rpitch+(bwd-deltax)*3,
									wp+h*wpitch,
										deltax);
				}
	
			}
			if(vi.IsYUY2())
			{
				for(int h=0;h<bht;h++)				
				{
					CopyHLineYUY2(LeftFramep+h*lpitch,
									wp+h*wpitch+2*deltax,
										bwd-deltax);
					CopyHLineYUY2(RightFramep+h*rpitch+(bwd-deltax)*2,
									wp+h*wpitch,
										deltax);
				}

			}
			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)				
				{
					CopyHLineYV12(LeftFramep+h*lpitch,
									wp+h*wpitch+deltax,
										bwd-deltax);
					CopyHLineYV12(RightFramep+h*rpitch+(bwd-deltax),
									wp+h*wpitch,
										deltax);
					if((h & 1)==1)
						continue;
					CopyHLineYV12(lpU+h/2*lpitchUV,
									wpU+h/2*wpitchUV+deltax/2,
										(bwd-deltax)/2);
					CopyHLineYV12(rpU+h/2*rpitchUV+(bwd-deltax)/2,
									wpU+h/2*wpitchUV,
										deltax/2);

					CopyHLineYV12(lpV+h/2*lpitchUV,
									wpV+h/2*wpitchUV+deltax/2,
										(bwd-deltax)/2);
					CopyHLineYV12(rpV+h/2*rpitchUV+(bwd-deltax)/2,
									wpV+h/2*wpitchUV,
										deltax/2);
				}

			}

		}
			

		if(*dir=='l' || *dir=='L')
			
		{			//LeftFrame moves westward unmasking RightFrame
			if(deltax<2)
				return LeftFrame;
			

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineRGB32(LeftFramep+h*lpitch+4*deltax,
									wp+h*wpitch,
										bwd-deltax);
					CopyHLineRGB32(RightFramep+h*rpitch,
									wp+h*wpitch+4*(bwd-deltax),
										deltax);
				}
			
			}
			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineRGB24(LeftFramep+h*lpitch+3*deltax,
									wp+h*wpitch,
										bwd-deltax);
					CopyHLineRGB24(RightFramep+h*rpitch,
									wp+h*wpitch+3*(bwd-deltax),
										deltax);
				}
		
			}
			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineYUY2(LeftFramep+h*lpitch+2*deltax,
									wp+h*wpitch,
										bwd-deltax);
					CopyHLineYUY2(RightFramep+h*rpitch,
									wp+h*wpitch+2*(bwd-deltax),
										deltax);
				}
	
			}
			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineYV12(LeftFramep+h*lpitch+deltax,
									wp+h*wpitch,
										bwd-deltax);
					CopyHLineYV12(RightFramep+h*rpitch,
									wp+h*wpitch+(bwd-deltax),
										deltax);
					if((h & 1)==1)
						continue;
					CopyHLineYV12(lpU+h/2*lpitchUV+deltax/2,
									wpU+h/2*wpitchUV,
										(bwd-deltax)/2);
					CopyHLineYV12(rpU+h/2*rpitchUV,
									wpU+h/2*wpitchUV+(bwd-deltax)/2,
										deltax/2);

					CopyHLineYV12(lpV+h/2*lpitchUV+deltax/2,
									wpV+h/2*wpitchUV,
										(bwd-deltax)/2);
					CopyHLineYV12(rpV+h/2*rpitchUV,
									wpV+h/2*wpitchUV+(bwd-deltax)/2,
										deltax/2);
				}
	
			}
		}
		return work;
		
	}

/********** 



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransPush(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransPush(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("left"),	// Transition direction
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransPush", "cci[dir]s", Create_TransPush, 0);
				
    return "TransPush  Plugin";
}
*/
