/******************************************************************************
The TransRipple Filter creates a ripple 
Created on Nov 12, 2003
Modified 23-9-04 
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "FastDisco.cpp"
#include "FastORing.cpp"
*/
class TransRipple : public GenericVideoFilter 
{
		PClip RightClip;
		int radmax;		// Max radius to grow by end of clip
		int lambda;		//wavelength
		int centerx;	// center x coord	
		int centery;	// center y coord
		int amp;		// wave height
		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
				
		 
  public:
							//Definition of function
    TransRipple(PClip _Left,PClip _Right,
				int _overlap,int _lambda,int _amp,  
			IScriptEnvironment* env) ;
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
  
			
 	~TransRipple();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
	bool __stdcall GetParity(int n);
	// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransRipple::TransRipple(PClip _child,PClip _Right,
						 int _overlap,int _lambda,int _amp,
						   IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_Right),
					overlap(_overlap),lambda(_lambda), amp(_amp)
			
{
	
const VideoInfo& rvi= RightClip->GetVideoInfo();

char * Tname="TransRipple";

	
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransRipple: This color format is not yet supported ");	
  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
if(!(vi.height== rvi.height) || !(vi.width == rvi.width) 
   || !(vi.pixel_type==rvi.pixel_type) )
   env->ThrowError("TransRipple: Height, width, colorspace of clips are unequal");
radmax=sqrt(vi.height*vi.height+vi.width*vi.width)/2;
//lambda=10*radmax/vi.num_frames;
 // amp=10;
if(lambda>radmax/4 )
	env->ThrowError("TransRipple:Too large a wavelength. Limit is %d", radmax/4);
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;

}
// This is where any actual destructor code used goes
TransRipple::~TransRipple() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/

void TransRipple::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransRipple::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}

	

//----------------------------------------------------------------------------------	
	PVideoFrame __stdcall TransRipple::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n,env);
			
		const	int bwd = vi.width; 
        const	int bht = LeftFrame->GetHeight();	// LeftClip->GetHeight()?
		const	int lpitch = LeftFrame->GetPitch();
		const	int rpitch = RightFrame->GetPitch();
		int nframes= overlap;
		env->MakeWritable(&LeftFrame);
		unsigned char* dstp= LeftFrame->GetWritePtr();
		// ENSURE that rad grows to radmax by 1/3 clip length and remains at that value
		int rad=3*n<(nframes)?(3*n*radmax)/nframes:radmax;
		rad=rad & 0xfffffffc;
		
			// radius for 1/3 length is zero and then grows to radmax
		int radius=3*n<=nframes? 0:(3*n-nframes)*radmax/(nframes);
		radius= radius & 0xfffffffc;
		PVideoFrame work = env->NewVideoFrame(vi);
		unsigned char * workp= work->GetWritePtr();
		const int wpitch=work->GetPitch();
		const unsigned char * srcp = RightFrame->GetReadPtr();
		int rr=0;
		if(3*n>2*nframes)
		{
			
			rr=radmax*(3*n-2*nframes)/(nframes);
			// this rr controls how much the frame expands from center outward without getting magnified

			lambda=lambda & 0xfffffffe;
			amp=amp-((amp-1)*(3*n-2*nframes))/(nframes);	// decrease to 1 by end
		}

		centerx=bwd/2;
		centery=bht/2;
		int startx= (centerx-rad);
		int endx = centerx+rad;
		int starty = centery-rad;
		int endy = centery+rad;
		if(startx<0)
			startx=0;
		if(endx>=bwd-1)
			endx=bwd-2;
		if(starty<0)
			starty=0;
		if(endy>bht-1)
			endy=bht-1;

		



		

		if(vi.IsRGB32())
		{
			if(3*n<nframes)
					// copy image inside the outer square on to work area
			for(int h=starty; h<endy; h++)
				for(int w=startx; w<endx; w++)
					*((int*)(workp+(bht-1-h)*lpitch+4*w))
						=*((int*)(dstp+(bht-1-h)*lpitch+4*w));
				
			else
				for(int h=starty; h<endy; h++)
				for(int w=startx; w<endx; w++)
					*((int*)(workp+(bht-1-h)*wpitch+4*w))
						=*((int*)(srcp+(bht-1-h)*rpitch+4*w));

					// repeatedly generate a water drop effect disks with reducing radius

			for( int r=rad; r>radius;r-=lambda )
			{
					// acts as long as rad>radmax(radius) i.e. 1/3 length
					// copies leftframe ripples on to work
				FastDiscoRGB32( dstp, workp, r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								lpitch,wpitch, amp, true,lambda);
			}
								// from Rightframe start the ripple		
			for(r=radius;r>rr;r-=lambda)
					// starts after 1/3 length
					// copies rightframe ripples on to work
				FastDiscoRGB32(srcp,workp,r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								rpitch,wpitch, amp, true,lambda);
								// copy image inside the work on to  output/input frame area
			for(int h=starty; h<endy; h++)
			{
				for(int w=startx; w<endx; w++)
				{
					*((int*)(dstp+(bht-1-h)*lpitch+4*w))
						=*((int*)(workp+(bht-1-h)*wpitch+4*w));
				}
			}

			return LeftFrame;
		}
		if(vi.IsRGB24())
		{
			if(3*n<nframes)
			

					// copy image inside the outer square on to work area
			for(int h=starty; h<endy; h++)
				for(int w=3*startx; w<3*endx; w++)
					*(workp+(bht-1-h)*wpitch+w)
						=*(dstp+(bht-1-h)*lpitch+w);
			
			else
				for(int h=starty; h<endy; h++)
					for(int w=3*startx; w<3*endx; w++)
						*(workp+(bht-1-h)*wpitch+w)
							=*(srcp+(bht-1-h)*lpitch+w);

			for( int r=rad; r>radius;r-=lambda )
				
				FastDiscoRGB24( dstp, workp, r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								lpitch,wpitch, amp,true,lambda);
											// from Rightframe start the ripple		
			for(r=radius;r>rr;r-=lambda)
				FastDiscoRGB24(srcp,workp,r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								rpitch,wpitch, amp, true,lambda);
					// copy image inside the work on to  output/input frame area
			for(int h=starty; h<endy; h++)
				for(int w=3*startx; w<3*endx; w++)
					*(dstp+(bht-1-h)*lpitch+w)
						=*(workp+(bht-1-h)*wpitch+w);

			return LeftFrame;
		}
		if(vi.IsYUY2())
		{
			if(3*n<nframes)
					// copy image inside the outer square on to work area
			for(int h=starty; h<endy; h++)
				for(int w=startx; w<endx; w+=2)
					*((int*)(workp+h*wpitch+2*w))
						=*((int*)(dstp+h*lpitch+2*w));
				
			else
				for(int h=starty; h<endy; h++)
					for(int w=startx; w<endx; w+=2)
						*((int*)(workp+h*wpitch+2*w))
							=*((int*)(srcp+h*rpitch+2*w));

					// repeatedly generate a water drop effect disks with reducing radius

			for( int r=rad; r>radius;r-=lambda )
			{
				
				FastDiscoYUY2( dstp, workp, r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								lpitch,wpitch, amp, true,lambda);
			}
											// from Rightframe start the ripple		
			for(r=radius;r>rr;r-=lambda)
				FastDiscoYUY2(srcp,workp,r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								rpitch,wpitch, amp, true,lambda);
					// copy image inside the work on to  output/input frame area
			for(int h=starty; h<endy; h++)
			{
				for(int w=startx; w<endx; w+=2)
				{
					*((int*)(dstp+h*lpitch+2*w))
						=*((int*)(workp+h*lpitch+2*w));
				}
			}

			return LeftFrame;
		}

		if(vi.IsYV12())
		{
			unsigned char *LeftFramep = LeftFrame->GetWritePtr();
			const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
			const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
			const int wpitchUV=work->GetPitch(PLANAR_U);
			const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
			const int bhtUV = LeftFrame->GetHeight(PLANAR_U);
			unsigned char *LeftFramepU = LeftFrame->GetWritePtr(PLANAR_U);
			unsigned char *LeftFramepV = LeftFrame->GetWritePtr(PLANAR_V);
			unsigned char *RightFramepU = RightFrame->GetWritePtr(PLANAR_U);
			unsigned char *RightFramepV = RightFrame->GetWritePtr(PLANAR_V);
			unsigned char *workpU = work->GetWritePtr(PLANAR_U);
			unsigned char *workpV = work->GetWritePtr(PLANAR_V);
		
					// copy image inside the outer square on to work area
		if(3*n<nframes)
		{
			for(int h=starty; h<endy; h++)
			{
				for(int w=startx; w<endx; w++)
				{
					*(workp+h*lpitch+w)
						=*(LeftFramep+h*lpitch+w);
				}
			}
			for(h=starty/2; h<endy/2; h++)
			{
				for(int w=startx/2; w<endx/2; w++)
				{
					*(workpU+h*lpitchUV+w)
						=*(LeftFramepU+h*lpitchUV+w);
					*(workpV+h*lpitchUV+w)
						=*(LeftFramepV+h*lpitchUV+w);
				}
			}
		}
		else
		{
			for(int h=starty; h<endy; h++)
			{
				for(int w=startx; w<endx; w++)
				{
					*(workp+h*wpitch+w)
						=*(srcp+h*rpitch+w);
				}
			}
			for(h=starty/2; h<endy/2; h++)
			{
				for(int w=startx/2; w<endx/2; w++)
				{
					*(workpU+h*wpitchUV+w)
						=*(RightFramepU+h*rpitchUV+w);
					*(workpV+h*wpitchUV+w)
						=*(RightFramepV+h*rpitchUV+w);
				}
			}
		}
					// repeatedly generate a water drop effect disks with reducing radius

			for( int r=rad; r>radius;r-=lambda )
			{
				
				FastDiscoYV12( dstp, workp, r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								lpitch,lpitch, amp, true,lambda);
			}
			for( r=rad/2; r>radius/2;r-=lambda/2 )
			{
				
				FastDiscoYV12( LeftFramepU, workpU, r,
								centerx/2, centery/2, bhtUV, bwdUV,
								centerx/2, centery/2, bhtUV, bwdUV,
								lpitchUV,wpitchUV, amp, true,lambda/2);
				FastDiscoYV12( LeftFramepV, workpV, r,
								centerx/2, centery/2, bhtUV, bwdUV,
								centerx/2, centery/2, bhtUV, bwdUV,
								lpitchUV,wpitchUV, amp, true,lambda/2);
			}
											// from Rightframe start the ripple		
			for(r=radius;r>rr;r-=lambda)
				FastDiscoYV12(srcp,workp,r,
								centerx, centery, bht, bwd,
								centerx, centery, bht, bwd,
								rpitch,wpitch, amp, true,lambda);
			for(r=radius/2;r>rr/2;r-=lambda/2)
			{
				FastDiscoYV12(RightFramepU,workpU,r,
								centerx/2, centery/2, bhtUV, bwdUV,
								centerx/2, centery/2, bhtUV, bwdUV,
								rpitchUV,wpitchUV, amp, true,lambda/2);
				FastDiscoYV12(RightFramepV,workpV,r,
								centerx/2, centery/2, bhtUV, bwdUV,
								centerx/2, centery/2, bhtUV, bwdUV,
								rpitchUV,wpitchUV, amp, true,lambda/2);
			}
					// copy image inside the work on to  output/input frame area
			for(int h=starty; h<endy; h++)
			{
				for(int w=startx; w<endx; w++)
				{
					*(dstp+h*lpitch+w)
						=*(workp+h*wpitch+w);
				}
			}
			for( h=starty/2; h<endy/2; h++)
			{
				for(int w=startx/2; w<endx/2; w++)
				{
					*(LeftFramepU+h*lpitchUV+w)
						=*(workpU+h*wpitchUV+w);
					*(LeftFramepV+h*lpitchUV+w)
						=*(workpV+h*wpitchUV+w);
				}
			}

			return LeftFrame;
		}


		
	}



	
AVSValue __cdecl Create_TransRipple(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	
	if(args[3].AsInt()<16 )
		env->ThrowError("TransRipple: wavelength must be atleast 16");
	if(args[4].AsInt()<2 || args[4].AsInt()>12)
		env->ThrowError("TransRipple: wave height between 2 and 12 only valid");
		
	return new TransRipple(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),   
						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsInt(40),		// wavelength/interval in pixels
						args[4].AsInt(8),		// wave height value 2 to 12
						env);
// Calls the constructor with the arguments provied.
}

/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransRipple", "cci[lambda]i[amp]i", Create_TransRipple, 0);
				
    return "Transition Ripple Plugin";
}

*/



