/******************************************************************************
TransScratch filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
Modified 23-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "FastORing.cpp"
*/
class TransScratch : public GenericVideoFilter {
		PClip RightClip;		// define the second clip
		const char * type;
		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
			
  public:
							//Definition of function
    TransScratch(PClip _child,PClip _RightClip,
				int _overlap,const char * _type,
				IScriptEnvironment* env) ;	
			
 	~TransScratch();		//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
	// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransScratch::TransScratch(PClip _child,PClip _RightClip,
						   int _overlap,const char * _type,IScriptEnvironment* env) :
	GenericVideoFilter(_child),RightClip(_RightClip),
					overlap(_overlap),type(_type) 
			
{
	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
		char * Tname="TransScratch";

	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransScratch:Clips have differing pixel_types");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransScratch: This color format is not yet supported");
	if (vi.width != rvi.width || vi.height != rvi.height)
	env->ThrowError("%s:Clips have differing frame dimensions", Tname);
	if(!(*type=='d') && !(*type=='D') && !(*type=='r') && !(*type=='R') && 
		!(*type=='t') && !(*type=='T') && !(*type=='l') && !(*type=='L')
		&& !(*type=='f') && !(*type=='F')&&!(*type=='o') && !(*type=='O')
		&&!(*type=='m') && !(*type=='M')&&!(*type=='v') && !(*type=='V')
		&&!(*type=='h') && !(*type=='H'))
		env->ThrowError("TransScratch: Options are rain, dust,laser, fount, o-rings,merge, vline, hline and TV only");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;

}
// This is where any actual destructor code used goes
TransScratch::~TransScratch() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/
void TransScratch::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransScratch::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}

//__________________________________________________________________

PVideoFrame __stdcall TransScratch::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const	int bwd = vi.width;
        const	int bht = LeftFrame->GetHeight();
		const	int lpitch = LeftFrame->GetPitch();
		const	int rpitch = RightFrame->GetPitch();
		const	int nframes = overlap;
		env->MakeWritable(&LeftFrame);
		unsigned char* lp= LeftFrame->GetWritePtr();
		const unsigned char * rp= RightFrame->GetReadPtr();
			// The Rand function has a max value of RAND_MAX (7FFF)
			// So need to work around
		 double mult=(double)(bht*bwd/RAND_MAX);
		 int span = mult;
		 int numbers=n*RAND_MAX/nframes;
		
			if(vi.IsRGB24())
			{
				if(*type=='t' || *type=='T')
						// at random points in the frame copy a length of span from leftframe on to right frame
					
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-3*span);
						for(int s=0; s<span;s++)
							*((int*)(lp+offset+3*s))=*((int*)(rp+offset+3*s));
					}
				if(*type=='d' || *type=='D')
							// copy random color values from LeftFrame to corresponding point on Left Frame
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-3*span);
						for(int s=offset;s<bht*lpitch;s+=RAND_MAX)
							for(int k=0;k<3;k++)
								*(lp+s+k)=*(rp+s+k);
					}
				if(*type=='r' || *type=='R')
					for(int i=0,int offset=0; i<numbers; i++)
								// copy a slanting line of height span from LeftFrame to RightFrame
					{
						offset=(offset+rand())%((bht-span)*lpitch-2*span);
							for(int s=0; s<span;s++)								
								*((int*)(lp+s*lpitch+2*s+offset))
								=*((int*)(rp+s*rpitch+2*s+offset));
					}
					if(*type=='l' || *type=='L')
							// copy a random light beam from bottom center of left to right frame.
						for(int i=0; i<n*500/nframes; i++)
						{
							int length=rand()%(2*(bht-1));// doubled h to cover frame fully
							int wmax=rand()%(bwd/2);	// halfed w to remain in frame
							int k = rand()%4;
							for(int h=0; h<length;h++)
								{
									int hh=(h<bht?h:2*bht-2-h);
									*(lp+3*bwd/2+3*h*wmax/length+hh*lpitch+k)
										=*(rp+3*bwd/2+3*h*wmax/length+hh*rpitch+k);
									*(lp+3*bwd/2-3*h*wmax/length+hh*lpitch+k)
										=*(rp+3*bwd/2-3*h*wmax/length+hh*rpitch+k);
								}
								
						}
					if(*type=='f' || *type=='F')
								// copy values from a random parabola on Left to Right Frame
								// focus of parobola coord are px and py. 4pY=X^2
						for(int i=0; i<n*800/nframes;i++)
						{

							int py=2+rand()%(bht-2);
							int px=rand()%(bwd/2);

							double pp=double(px*px)/(4*py);
							for(int w=0;w<bwd/2;w++)
							{
								
								int h=py-(w-px)*(w-px)/(4*pp);
								if(h<=0 | h>=bht)
									continue;
								for(int k=0;k<3;k++)
								{
									*(lp+h*lpitch+3*(bwd/2-w)+k)
										=*(rp+h*rpitch+3*(bwd/2-w)+k);
									*(lp+h*lpitch+3*(bwd/2+w)+k)
										=*(rp+h*rpitch+3*(bwd/2+w)+k);
								}
							}
							
	
						}
					if(*type=='o' || *type=='O')
						for(int i=0; i<n*50/nframes;i++)
						{
							int radius=12+rand()%(bht-12);
							
							int ox=rand()% (bwd/2);
							int oy=rand()%(bht-12);
							FastORingRGB24(rp,lp,radius,8,ox,oy,bht,bwd,ox,oy,bht,bwd,rpitch,lpitch);
							FastORingRGB24(rp,lp,radius,8,bwd-ox,oy,bht,bwd,bwd-ox,oy,bht,bwd,rpitch,rpitch);
						}

					if(*type=='m' || *type=='M')
					{
						if(4*n<=3*nframes)
							for(int h=0; h<bht; h+=2+((3*nframes-4*n)/3)/4)
								for(int w=0; w<bwd;w++)
									for(int k=0; k<3;k++)
										*(lp+h*lpitch+3*w+k)
											=*(rp+h*rpitch+3*w+k);

						if(4*n>3*nframes)						
							for(int h=0; h<bht-1; h+=2+(4*n-3*nframes)/4)
								for(int i=h+1; i<h+2+(4*n-3*nframes)/4 && i<bht-1; i++)
									for(int w=0; w<bwd;w++)
										for(int k=0; k<3;k++)
											*(lp+i*lpitch+3*w+k)
												=*(rp+i*rpitch+3*w+k);

						
					}

					if(*type=='v' || *type =='V')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int w= (rand()%bwd);
							for(int h=0;h<bht;h++)
								for(int k=0; k<3;k++)
										*(lp+h*lpitch+3*w+k)
										=*(rp+h*rpitch+3*w+k);
						}
					}

					if(*type=='h' || *type =='H')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int h= (rand()%bht);
							for(int w=0;w<bwd;w++)
								for(int k=0; k<3;k++)
										*(lp+h*lpitch+3*w+k)
										=*(rp+h*rpitch+3*w+k);
						}
					}
				
				return LeftFrame;
			}
				
			// TV Noise effect. Random noise like no signal TV appearance
			if(vi.IsRGB32())
			{

				if(*type=='t' || *type=='T')
					
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-4*span);
						for(int s=0; s<span;s++)
							*((int*)(lp+offset+4*s))
							=*((int*)(rp+offset+4*s));
					}
						// Dust effect. Dust from Right Frame gradually envelops LeftFrame
				if(*type=='d' || *type=='D')
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-4*span);
						for(int s=offset;s<bht*lpitch;s+=(RAND_MAX & 0xfffc))
							*((int*)(lp+s))=*((int*)(rp+s));
					}

						// Rain. A drizzle of RightFrame increases to downpour
				if(*type=='r' || *type=='R')
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%((bht-span)*lpitch-2*span);
							for(int s=0; s<span;s++)								
								*((int*)(lp+s*lpitch+2*s+offset))
								=*((int*)(rp+s*rpitch+2*s+offset));
					}

					// Laser Beam. Light Beams of Right Frame 
				if(*type=='l' || *type=='L')
					for(int i=0; i<n*500/nframes;i++)
					{
						int length=rand()%(2*(bht-1));
						int wmax=rand()%(bwd/2);
						int k = rand()%4;
						for(int h=0; h<length;h++)
						{
							int hh=(h<bht?h:2*bht-2-h);
								*((int *)(lp+2*bwd+4*h*wmax/length+k+hh*lpitch))
									=*((int *)(rp+2*bwd+4*h*wmax/length+k+hh*rpitch));
								*((int *)(lp+2*bwd-4*h*wmax/length+k+hh*lpitch))
									=*((int *)(rp+bwd*2-4*h*wmax/length+k+hh*rpitch));
						}
								
					}
					//A water fountain effect. Water drop trajectories are parabolas
					if(*type=='f' || *type=='F')
						for(int i=0; i<n*800/nframes;i++)
						{

							int py=2+rand()%(bht-2);
							int px=rand()%(bwd/2);

							double pp=double(px*px)/(4*py);
							for(int w=0;w<bwd/2;w++)
							{
								
								int h=py-((w-px)*(w-px)/(4*pp));
								if(h<=0 || h>=bht)
									continue;
					
								*((int*)(lp+h*lpitch+4*(bwd/2-w)))
									=*((int*)(rp+h*rpitch+4*(bwd/2-w)));
								*((int*)(lp+h*lpitch+4*(bwd/2+w)))
									=*((int*)(rp+h*rpitch+4*(bwd/2+w)));
							
							}
	
						}
						// Merge. Looks like dissolve but different

					if(*type=='m' || *type=='M')
					{
						if(4*n<=3*nframes)
							for(int h=0; h<bht; h+=2+((3*nframes-4*n)/3)/4)
								for(int w=0; w<bwd;w++)
									*((int*)(lp+h*lpitch+4*w))
									=*((int*)(rp+h*rpitch+4*w));
/*						if(2*n==nframes)
							for(int h=0; h<bht; h+=2)
								for(int w=0; w<bwd;w++)
									*((int*)(lp+h*lpitch+4*w))
									=*((int*)(rp+h*rpitch+4*w));
*/
						if(4*n>3*nframes)						
							for(int h=0; h<bht-1; h+=2+(4*n-3*nframes)/4)
								for(int i=h+1; i<h+2+(4*n-3*nframes)/4 && i<bht-1; i++)
									for(int w=0; w<bwd;w++)
										*((int*)(lp+i*lpitch+4*w))
										=*((int*)(rp+i*rpitch+4*w));

						
					}

					if(*type=='v' || *type =='V')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int w= (rand()%bwd);
							for(int h=0;h<bht;h++)
								*((int*)(lp+h*lpitch+4*w))
										=*((int*)(rp+h*rpitch+4*w));
						}
					}

					if(*type=='h' || *type =='H')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int h= (rand()%bht);
							for(int w=0;w<bwd;w++)
								*((int*)(lp+h*lpitch+4*w))
										=*((int*)(rp+h*rpitch+4*w));
						}
					}


					if(*type=='o' || *type=='O')
						for(int i=0; i<n*50/nframes;i++)
						{
							int radius=12+rand()%(bht-12);
							
							int ox=rand()% (bwd/2);
							int oy=rand()%(bht-12);
							FastORingRGB32(rp,lp,radius,8,ox,oy,bht,bwd,ox,oy,bht,bwd,rpitch,lpitch);
							FastORingRGB32(rp,lp,radius,8,bwd-ox,oy,bht,bwd,bwd-ox,oy,bht,bwd,rpitch,lpitch);
						}

				
				return LeftFrame;
			}


		if(vi.IsYUY2())
			{

				if(*type=='t' || *type=='T')
					
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-4*span);
						for(int s=0; s<span;s++)
							*((int*)(lp+offset+4*s))=*((int*)(rp+offset+4*s));
					}
				if(*type=='d' || *type=='D')
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-4*span);
						for(int s=offset;s<bht*lpitch;s+=(RAND_MAX & 0xfffc))
							*((int*)(lp+s))=*((int*)(rp+s));
					}
				if(*type=='r' || *type=='R')
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%((bht-span)*lpitch-2*span);
							for(int s=0; s<span;s++)								
								*((int*)(lp+s*lpitch+s+offset))
								=*((int*)(rp+s*rpitch+s+offset));
					}
				if(*type=='l' || *type=='L')
					for(int i=0; i<n*500/nframes;i++)
					{
						int length=rand()%(2*bht);
						int wmax=rand()%(bwd/2);
						
						for(int h=0; h<length;h++)
						{
							int hh=bht-(h<bht?h:2*bht-2-h);
								*((int *)(lp+bwd+2*h*wmax/length+hh*lpitch))
									=*((int *)(rp+bwd+2*h*wmax/length+hh*rpitch));
								*((int *)(lp+bwd-2*h*wmax/length+hh*lpitch))
									=*((int *)(rp+bwd-2*h*wmax/length+hh*rpitch));
						}
								
					}
					if(*type=='f' || *type=='F')
						for(int i=0; i<n*800/nframes;i++)
						{

							int py=2+rand()%(bht-2);
							int px=rand()%(bwd/2);

							double pp=double(px*px)/(4*py);
							for(int w=0;w<bwd/2-2;w+=2)
							{
								
								int h=bht+((w-px)*(w-px)/(4*pp))-py;
								if(h>=bht || h<0)
									continue;
					
								*((int*)(lp+h*lpitch+2*(bwd/2-w)))
									=*((int*)(rp+h*rpitch+2*(bwd/2-w)));
								*((int*)(lp+h*lpitch+2*(bwd/2+w)))
									=*((int*)(rp+h*rpitch+2*(bwd/2+w)));
							}
	
						}

					if(*type=='o' || *type=='O')
						for(int i=0; i<n*50/nframes;i++)
						{
							int radius=12+rand()%(bht-12);
							
							int ox=rand()% (bwd/2);
							int oy=rand()%(bht-12);
							FastORingYUY2(rp,lp,radius,8,ox,oy,bht,bwd,ox,oy,bht,bwd,rpitch,lpitch);
							FastORingYUY2(rp,lp,radius,8,bwd-ox,oy,bht,bwd,bwd-ox,oy,bht,bwd,rpitch,lpitch);
						}

						if(*type=='m' || *type=='M')
						{
						if(4*n<=3*nframes)
							for(int h=0; h<bht; h+=2+((3*nframes-4*n)/3)/4)
								for(int w=0; w<bwd;w+=2)
									*((int*)(lp+h*lpitch+2*w))
									=*((int*)(rp+h*rpitch+2*w));

						if(4*n>3*nframes)						
							for(int h=0; h<bht-1; h+=2+(4*n-3*nframes)/4)
								for(int i=h+1; i<h+2+(4*n-3*nframes)/4 && i<bht-1; i++)
									for(int w=0; w<bwd;w+=2)
										*((int*)(lp+i*lpitch+2*w))
										=*((int*)(rp+i*rpitch+2*w));

						
					}

					if(*type=='v' || *type =='V')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int w= (rand()%bwd) & 0xFFFFFFFE;
							for(int h=0;h<bht;h++)
								*((int*)(lp+h*lpitch+2*w))
										=*((int*)(rp+h*rpitch+2*w));
						}
					}

					if(*type=='h' || *type =='H')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int h= (rand()%bht);
							for(int w=0;w<bwd;w+=2)
								*((int*)(lp+h*lpitch+2*w))
										=*((int*)(rp+h*rpitch+2*w));
						}
					}

				
				return LeftFrame;
			}

			if(vi.IsYV12())
			{



				const int lpitchUV = LeftFrame->GetPitch(PLANAR_U);	// The pitch,height and width information
				const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
				const int bhtUV = LeftFrame->GetHeight(PLANAR_U);	// the U and V planes so we only the U
				const int rpitchUV = RightFrame->GetPitch(PLANAR_U);	// plane values and use them for V as
		
		
				const unsigned char * rpU = RightFrame->GetReadPtr(PLANAR_U);
				unsigned char *lpU = LeftFrame->GetWritePtr(PLANAR_U);
				const unsigned char *rpV = RightFrame->GetReadPtr(PLANAR_V);
				unsigned char *lpV = LeftFrame->GetWritePtr(PLANAR_V);

				if(*type=='t' || *type=='T')					
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-span);
						for(int s=0; s<span;s++)
							*(lp+offset+s)=*(rp+offset+s);
					
							// U V planes
							
						for(s=0; s<span/2;s++)
						{
							*(lpU+offset/4+s)=*(rpU+offset/4+s);
							*(lpV+offset/4+s)=*(rpV+offset/4+s);
						}
							
					}
				if(*type=='d' || *type=='D')
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%(bht*lpitch-span);
						for(int s=offset;s<bht*lpitch;s+=(RAND_MAX & 0xfffc))
							*(lp+s)=*(rp+s);
						for(s=offset/4;s<bhtUV*lpitchUV;s+=(RAND_MAX & 0xfffc)/4)
						{
							*(lpU+s)=*(rpU+s);
							*(lpV+s)=*(rpV+s);
						}
					}
				if(*type=='r' || *type=='R')
					for(int i=0,int offset=0; i<numbers; i++)
					{
						offset=(offset+rand())%((bht-span)*lpitch-2*span);
						for(int s=0; s<span;s++)								
								*(lp+s*lpitch+s+offset)
								=*(rp+s*rpitch+s+offset);
						for(s=0; s<span/2;s++)
						{
							*(lpU+s*lpitchUV+s+offset/4)
								=*(rpU+s*rpitchUV+s+offset/4);
							*(lpV+s*lpitchUV+s+offset/4)
								=*(rpV+s*rpitchUV+s+offset/4);
						}
					}
				if(*type=='l' || *type=='L')
					for(int i=0; i<n*500/nframes;i++)
					{
						int length=rand()%(2*bht);
						int wmax=rand()%(bwd/2);
						
						for(int h=0; h<length;h++)
						{
							int hh=bht-(h<bht?h:2*bht-2-h);
								*(lp+bwd/2+(h*wmax)/length+hh*lpitch)
									=*(rp+bwd/2+(h*wmax)/length+hh*rpitch);
								*(lp+bwd/2-(h*wmax)/length+hh*lpitch)
									=*(rp+bwd/2-(h*wmax)/length+hh*rpitch);
						}
						for(h=0; h<length/2;h++)
						{
							int hh=bhtUV-(h<bht?h:2*bht-2-h)/2;
								*(lpU+bwdUV/2+(h*wmax)/length+hh*lpitchUV)
									=*(rpU+bwdUV/2+(h*wmax/length)+hh*rpitchUV);
								*(lpU+bwdUV/2-(h*wmax)/length+hh*lpitchUV)
									=*(rpU+bwdUV/2-(h*wmax)/length+hh*rpitchUV);
								*(lpV+bwdUV/2+(h*wmax)/length+hh*lpitchUV)
									=*(rpV+bwdUV/2+(h*wmax/length)+hh*rpitchUV);
								*(lpV+bwdUV/2-(h*wmax)/length+hh*lpitchUV)
									=*(rpV+bwdUV/2-(h*wmax)/length+hh*rpitchUV);
						}
								
					}
					if(*type=='f' || *type=='F')
						for(int i=0; i<n*800/nframes;i++)
						{

							int py=2+rand()%(bht-2);
							int px=rand()%(bwd/2);

							double pp=double(px*px)/(4*py);
							for(int w=0;w<bwd/2;w++)
							{
								
								int h=bht-py+((w-px)*(w-px)/(4*pp));
								if(h>=bht || h<0)
									continue;
								*(lp+h*lpitch+(bwd/2-w))
									=*(rp+h*rpitch+(bwd/2-w));
								*(lp+h*lpitch+(bwd/2+w))
									=*(rp+h*rpitch+(bwd/2+w));
								if((w & 1)==1 || (h & 1)==1)
									continue;
								*(lpU+h/2*lpitchUV+(bwdUV/2-w/2))
									=*(rpU+h/2*rpitchUV+(bwdUV/2-w/2));
								*(lpU+h/2*lpitchUV+(bwdUV/2+w/2))
									=*(rpU+h/2*rpitchUV+(bwdUV/2+w/2));

								*(lpV+h/2*lpitchUV+(bwdUV/2-w/2))
									=*(rpV+h/2*rpitchUV+(bwdUV/2-w/2));									
								*(lpV+h/2*lpitchUV+(bwdUV/2+w/2))
									=*(rpV+h/2*rpitchUV+(bwdUV/2+w/2));
							}
							
	
						}

					if(*type=='o' || *type=='O')
						for(int i=0; i<n*50/nframes;i++)
						{
							int radius=12+rand()%(bht-12);
							
							int ox=rand()% (bwd/2);
							int oy=rand()%(bht-12);
							FastORingYV12(rp,lp,radius,8,ox,oy,bht,bwd,ox,oy,bht,bwd,rpitch,lpitch);
							FastORingYV12(rp,lp,radius,8,bwd-ox,oy,bht,bwd,bwd-ox,oy,bht,bwd,rpitch,lpitch);
							FastORingYV12(rpU,lpU,radius/2,4,ox/2,oy/2,bhtUV,bwdUV,
											ox/2,oy/2,bhtUV,bwdUV,rpitchUV,lpitchUV);
							FastORingYV12(rpU,lpU,radius/2,4,bwdUV-ox/2,oy/2,bhtUV,bwdUV,
											bwdUV-ox/2,oy/2,bhtUV,bwdUV,rpitchUV,lpitchUV);
							FastORingYV12(rpV,lpV,radius/2,4,ox/2,oy/2,bhtUV,bwdUV,
											ox/2,oy/2,bhtUV,bwdUV,rpitchUV,lpitchUV);
							FastORingYV12(rpV,lpV,radius/2,4,bwdUV-ox/2,oy/2,bhtUV,bwdUV,
											bwdUV-ox/2,oy/2,bhtUV,bwdUV,rpitchUV,lpitchUV);
						}

						if(*type=='m' || *type=='M')
						{
						if(4*n<=3*nframes)
							for(int h=0; h<bht; h+=2+(3*nframes-4*n)/4)
								for(int w=0; w<bwd;w++)
								{
									*(lp+h*lpitch+w)
										=*(rp+h*rpitch+w);
									if((h & 1)==1 || (w & 1)==1)
										continue;
									*(lpU+h/2*lpitchUV+w/2)
										=*(rpU+h/2*rpitchUV+w/2);
									*(lpV+h/2*lpitchUV+w/2)
										=*(rpV+h/2*rpitchUV+w/2);
								}

						if(4*n>3*nframes)						
							for(int h=0; h<bht-1; h+=2+(4*n-3*nframes)/4)
								for(int i=h+1; i<(h+2+(4*n-3*nframes)/4) && i<bht-1; i++)
									for(int w=0; w<bwd;w++)
									{
										*(lp+i*lpitch+w)
											=*(rp+i*rpitch+w);
										if((i & 1)==1 || (w & 1)==1)
											continue;
										*(lpU+i/2*lpitchUV+w/2)
											=*(rpU+i/2*rpitchUV+w/2);
										*(lpV+i/2*lpitchUV+w/2)
											=*(rpV+i/2*rpitchUV+w/2);
									}

						
					}

					if(*type=='v' || *type =='V')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int w= (rand()%bwd) ;
							for(int h=0;h<bht;h++)
							{
								*(lp+h*lpitch+w)
										=*(rp+h*rpitch+w);
								if((h & 1)==1 && (w & 1)==1)
									continue;
								*(lpU+h/2*lpitchUV+w/2)
										=*(rpU+h/2*rpitchUV+w/2);
								*(lpV+h/2*lpitchUV+w/2)
										=*(rpV+h/2*rpitchUV+w/2);
							}

						}
					}

					if(*type=='h' || *type =='H')
					{
						for(int i=0;i<n*bwd/nframes; i++)
						{
							int h= (rand()%bht);
							for(int w=0;w<bwd;w++)
							{								
								*(lp+h*lpitch+w)
									=*(rp+h*rpitch+w);
								if((h & 1)==1 && (w & 1)==1)
									continue;
								*(lpU+h/2*lpitchUV+w/2)
									=*(rpU+h/2*rpitchUV+w/2);
								*(lpV+h/2*lpitchUV+w/2)
									=*(rpV+h/2*rpitchUV+w/2);
							}
								
						}
					}
  
					return LeftFrame;
				}
				

				
	}



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransScratch(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransScratch(args[0].AsClip(),	// clip as LeftClip
								args[1].AsClip(), // as right clip
								args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames

								args[3].AsString("rain"), // TV noise, dust, rain, laser,
													//fount.O_ring,Vline, Hline, merge
								env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransScratch", "cci[style]s", Create_TransScratch, 0);
				
    return "TransScratch Transition Plugin";
}
*/
