/******************************************************************************
TransShuffle filter plugin for Avisynth by V.C.Mohan
This RightClip gradually Masks  a stationary LeftClip  in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
Modified on 23-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
*/
class TransShuffle : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;

		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
			

		 
  public:
							//Definition of function
    TransShuffle(PClip _child, PClip _RightClip,
				int _overlap,const char* _dir,  
				IScriptEnvironment* env) ;	
			
 	~TransShuffle();				//destructor
//	copywindow(const unsigned char* _R,unsigned char* _L, int _w, int _h, int _rp, int _lp);
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);	
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);		
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransShuffle::TransShuffle(PClip _child, PClip _RightClip,
						   int _overlap,const char* _dir, IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
					overlap(_overlap),dir(_dir) 
			
{
	
const VideoInfo& rvi= RightClip->GetVideoInfo();
char * Tname="TransShuffle";

if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransShuffle:Clips have differing pixel_types");
if (vi.width != rvi.width || vi.height != rvi.height)
	env->ThrowError("%s:Clips have differing frame dimensions", Tname);

if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2()&& !vi.IsYV12())
	env->ThrowError("TransShuffle: This color format is not supported here");
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);	
if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;

}
// This is where any actual destructor code used goes
TransShuffle::~TransShuffle() {

// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/
void TransShuffle::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransShuffle::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}

//--------------------------------------------------------------------------------	
	PVideoFrame __stdcall TransShuffle::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		PVideoFrame dst;
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		
		bool flipped=false;
		const	int bwd = lvi.width; 
        const	int bht = LeftFrame->GetHeight();	
		int deltay =((n+1)*bht)/(overlap+1);
		int deltax = ((n+1)*bwd)/(overlap+1);
		if(n>overlap/2)
		{
			flipped=true;
			deltay=deltay-bht/2;
			deltax=deltax-bwd/2;
		}
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
	

		deltax= deltax & 0xFFFFFFFE;
		deltay= deltay & 0xFFFFFFFE;
		if(deltax==0 || deltay ==0)
			return LeftFrame;
		int LeftFramex,LeftFramey,RightFramex,RightFramey,dstlx,dstrx,dstly,dstry ;
		int lwd,lht,rwd,rht;
		if( strcmp(dir, "up")==0)
			
		{
					// LeftFrame moves north, while RightFrame from underneath moves down
					//gradually covering LeftFrame till half time. Then Right frame moves up
					// while left hrame slides in below right frame
			if(!flipped)
			{
				LeftFramex =0;
				LeftFramey = deltay;
				lwd=bwd;				
				lht= bht-deltay;
				dstlx=0;
				dstly=0;
				RightFramex=0;
				RightFramey=bht-2*deltay;
				rht = deltay;
				rwd = bwd;
				dstrx=0;
				dstry=bht-deltay;
			}
			if(flipped)
			{
				LeftFramex =0;
				LeftFramey = bht/2-deltay;
				lwd=bwd;				
				lht= bht/2-deltay;
				dstlx=0;
				dstly=0;
				RightFramex=0;
				RightFramey=0;
				rht = bht/2+deltay;
				rwd = bwd;
				dstrx=0;
				dstry=bht/2-deltay;
			}
		}




		if(strcmp(dir, "down")==0)
		
		{
					// LeftFrame moves down halfway, while RightFrame from underneath moves up
					//gradually till half frames are seen. Then Right frame moves down
					// while left frame slides up below right frame
			if(!flipped)
			{
				LeftFramex =0;
				LeftFramey = 0;
				lwd=bwd;				
				lht=bht-deltay;
				dstlx=0;
				dstly=deltay;
				RightFramex=0;
				RightFramey=deltay;
				rht = deltay;
				rwd = bwd;
				dstrx=0;
				dstry=0;
			}
			if(flipped)
			{
				LeftFramex =0;
				LeftFramey = 2*deltay;
				lwd=bwd;				
				lht= bht/2-deltay;
				dstlx=0;
				dstly=bht/2+deltay;
				RightFramex=0;
				RightFramey=bht/2-deltay;
				rht = bht/2+deltay;
				rwd = bwd;
				dstrx=0;
				dstry=0;
			}
				
				
		}

		if(strcmp(dir, "left")==0)

		{
					// LeftFrame moves left halfway while Right Frame 
					// from underneath moves Right gradually till two halves
					// are seen. Then Right frame moves left while Left frame slides
					// right underneath
			if(!flipped)
			{
				LeftFramex =deltax;
				LeftFramey = 0;
				lwd=bwd-deltax;				
				lht=bht;
				dstlx=0;
				dstly=0;
				RightFramex=bwd-2*deltax;
				RightFramey=0;
				rht = bht;
				rwd = deltax;
				dstrx=bwd-deltax;
				dstry=0;
			}
			if(flipped)
			{
				LeftFramex =bwd/2-deltax;
				LeftFramey =0;
				lwd=bwd/2-deltax;				
				lht= bht;
				dstlx=0;
				dstly=0;
				RightFramex=0;
				RightFramey=0;
				rht = bht;
				rwd = bwd/2+deltax;
				dstrx=bwd/2-deltax;
				dstry=0;
			}
				
				
		}

		if(strcmp(dir, "right")==0)

		{
					// LeftFrame moves right halfway while Right Frame 
					// from underneath moves Leftt gradually till two halves
					// are seen. Then Right frame moves Right while Left frame slides
					// leftt underneath
			if(!flipped)
			{
				LeftFramex =0;
				LeftFramey = 0;
				lwd=bwd-deltax;				
				lht=bht;
				dstlx=deltax;
				dstly=0;
				RightFramex=deltax;
				RightFramey=0;
				rht = bht;
				rwd = deltax;
				dstrx=0;
				dstry=0;
			}
			if(flipped)
			{
				LeftFramex =2*deltax;
				LeftFramey =0;
				lwd=bwd/2-deltax;				
				lht= bht;
				dstlx=bwd/2+deltax;
				dstly=0;
				RightFramex=bwd/2-deltax;
				RightFramey=0;
				rht = bht;
				rwd = bwd/2+deltax;
				dstrx=0;
				dstry=0;
			}
			
			
				
		}

		
		lwd=lwd & 0xFFFFFFFE; //  make even number of pixels
		rwd=rwd & 0xFFFFFFFe;
		if(!flipped)
			if (rwd==0 || rht==0)
				return LeftFrame; // no work to be done

		dst=env->NewVideoFrame(vi);
		unsigned char* dstp= dst->GetWritePtr();
		const int dpitch = dst->GetPitch();
		const unsigned char* LeftFramep= LeftFrame->GetReadPtr();
		const unsigned char* RightFramep= RightFrame->GetReadPtr();

		if(lvi.IsRGB24())
		{		
				// RGB image is stored in a vertically flipped fashion, So from bottom up copy.
				// 3 bytes per pixel format.
			for(int h=0;h<lht;h++)
				for(int w=0;w<lwd;w++)
					for(int k=0;k<3;k++)
						*(dstp+(bht-1-(dstly+h))*lpitch+3*(dstlx+w)+k)
						=*(LeftFramep+(bht-1-(LeftFramey+h))*lpitch+3*(LeftFramex+w)+k);
			for(h=0;h<rht;h++)
				for(int w=0;w<rwd;w++)
					for(int k=0;k<3;k++)
						*(dstp+(bht-1-(dstry+h))*rpitch+3*(dstrx+w)+k)
						=*(RightFramep+(bht-1-(RightFramey+h))*rpitch+3*(RightFramex+w)+k);

		}
					
		if(lvi.IsRGB32())
		{
				// RGB frame is stored in memory vertically flipped
				// 4 bytes or one 32 bit word per pixel. so faster to treat in word size
				// copy bottom up of window
			for(int h=0;h<lht;h++)
				for(int w=0;w<lwd;w++)
					*((int *)(dstp+(bht-1-(dstly+h))*lpitch+4*(dstlx+w)))
						=*((int *)(LeftFramep+(bht-1-(LeftFramey+h))*lpitch+4*(LeftFramex+w)));
			for(h=0;h<rht;h++)
				for(int w=0;w<rwd;w++)
					*((int *)(dstp+(bht-1-(dstry+h))*rpitch+4*(dstrx+w)))
						=*((int *)(RightFramep+(bht-1-(RightFramey+h))*rpitch+4*(RightFramex+w)));

		}

		if(lvi.IsYUY2())
		{
				// This code deals with YUY2 colourspace where each 4 byte sequence represents
				// 2 pixels, (Y1, U, Y2 and then V). So always deal with even number of pixels.
				// copy LeftFrame Part
			for(int h=0;h<lht;h++)
				for(int w=0;w<lwd;w+=2)
					*((int *)(dstp+(dstly+h)*lpitch+2*(dstlx+w)))
						=*((int *)(LeftFramep+(LeftFramey+h)*lpitch+2*(LeftFramex+w)));
			for(h=0;h<rht;h++)
				for(int w=0;w<rwd;w+=2)
					*((int *)(dstp+(dstry+h)*rpitch+2*(dstrx+w)))
						=*((int *)(RightFramep+(RightFramey+h)*rpitch+2*(RightFramex+w)));


				

		}
		if(vi.IsYV12())
		{



			const int lpitchUV = LeftFrame->GetPitch(PLANAR_U);	// The pitch,height and width information
			const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
			const int bhtUV = LeftFrame->GetHeight(PLANAR_U);	// the U and V planes so we only the U
			const int rpitchUV = RightFrame->GetPitch(PLANAR_U);	// plane values and use them for V as
			const int dpitchUV = dst->GetPitch(PLANAR_U);
		
			const unsigned char * rpU = RightFrame->GetReadPtr(PLANAR_U);
			const unsigned char * rpV = RightFrame->GetReadPtr(PLANAR_V);
			const unsigned char *lpU = LeftFrame->GetReadPtr(PLANAR_U);
			const unsigned char *lpV = LeftFrame->GetReadPtr(PLANAR_V);
			unsigned char *dstpU = dst->GetWritePtr(PLANAR_U);
			unsigned char *dstpV = dst->GetWritePtr(PLANAR_V);

			for(int h=0;h<lht;h++)
				for(int w=0;w<lwd;w++)
				{
					*(dstp+(dstly+h)*dpitch+(dstlx+w))
						=*(LeftFramep+(LeftFramey+h)*lpitch+(LeftFramex+w));
					if((h & 1)==1 || (w &1)==1)
						continue;

					*(dstpU+(dstly+h)/2*lpitchUV+(dstlx+w)/2)
						=*(lpU+(LeftFramey+h)/2*lpitchUV+(LeftFramex+w)/2);
					*(dstpV+(dstly+h)/2*dpitchUV+(dstlx+w)/2)
						=*(lpV+(LeftFramey+h)/2*lpitchUV+(LeftFramex+w)/2);
				}


			for(h=0;h<rht;h++)
				for(int w=0;w<rwd;w++)
				{
					*(dstp+(dstry+h)*dpitch+(dstrx+w))
						=*(RightFramep+(RightFramey+h)*rpitch+(RightFramex+w));
					if((h & 1)==1 || (w &1)==1)
						continue;
					*(dstpU+(dstry+h)/2*dpitchUV+(dstrx+w)/2)
						=*(rpU+(RightFramey+h)/2*rpitchUV+(RightFramex+w)/2);
					*(dstpV+(dstry+h)/2*dpitchUV+(dstrx+w)/2)
						=*(rpV+(RightFramey+h)/2*rpitchUV+(RightFramex+w)/2);
				}
		}
		return dst;
			
	}

// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransShuffle(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransShuffle(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames

						args[3].AsString("up"),	// Transition direction 
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransShuffle", "cci[dir]s", Create_TransShuffle, 0);
				
    return "TransShuffle Transition  Plugin";
}
*/
