/******************************************************************************
TransSlantRollOut filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also 
used partly adapted function coding 
from Avisynth and simple sample. Roll Diameter and
 shading along roll length are other params
 Modified on 24-9-04
********************************************************************************/  
/*#include "windows.h"
#include "avisynth.h"
#include "LineZ.cpp"
*/
class TransSlantRollOut : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;
		const int Maxroll;
		const int shade;
		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
  public:
							//Definition of function
    TransSlantRollOut(PClip _child, PClip _RightClip,
				int _overlap,const char* _dir,
				const int _maxroll, const int _shade,  
				IScriptEnvironment* env) ;	
			
 	~TransSlantRollOut();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/

//Here is the acutal constructor code used

TransSlantRollOut::TransSlantRollOut(PClip _child, PClip _RightClip,
								   int _overlap,const char* _dir,
									const int _maxroll, const int _shade,
									IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip), 
						overlap(_overlap),dir(_dir),
						Maxroll(_maxroll), shade(_shade)
			
{
char * Tname="TransSlantRollOut";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransSlantRollOut:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransSlantRollOut: This color format is not supported here");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("TransSlantRollOut: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransSlantRollOut: Height/width of frame are not multiple of 4");
if(!(*dir=='n') && !(*dir=='N') && !(*dir=='s') && !(*dir=='S') &&
	!(*(dir+1)=='e') && !(*(dir+1)=='E') && !(*(dir+1)=='w') && !(*(dir+1)=='W'))
	env->ThrowError("TransSlantRollOut:ne, se, nw and sw only  are valid directions");
if(Maxroll<8)
   env->ThrowError("TransSlantRollOut: roll diameter is too small. Least is 8");
if(vi.width<vi.height)
	if(Maxroll>vi.width/4)
		env->ThrowError("TransSlantRollOut: roll diameter is large. Limit to 1/4 of width=%d",vi.width);
if(vi.width>=vi.height)
	if(Maxroll>vi.height/4)
		env->ThrowError("TransSlantRollOut: roll diameter is large. Limit to 1/4 of height = %d", vi.height);
if(shade<-255 || shade >255)
	env->ThrowError("TransSlantRollOut:shade can be between -255 and 255 only");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);

	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}

	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransSlantRollOut::~TransSlantRollOut()
 {
	
// This is where you can deallocate any memory you might have used.
}
/***************************************************************/	
void TransSlantRollOut::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransSlantRollOut::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}

//--------------------------------------------------------------------------------	
	
	
	PVideoFrame __stdcall TransSlantRollOut::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		int maxroll=Maxroll & 0xfffffffc;		 
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		int delta = ((n+1)*(bwd+bht))/(overlap+1);
		delta = delta & 0xFFFFFFFC;
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		int  wht, wwd;
		env->MakeWritable(&RightFrame);
		unsigned char* RightFramep= RightFrame->GetWritePtr();
		const unsigned char * LeftFramep= LeftFrame->GetReadPtr();
		unsigned char *RightFramepU, *RightFramepV;
		const unsigned char *LeftFramepU, *LeftFramepV;
		const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
		const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		if(vi.IsYV12())
		{
				LeftFramepU =LeftFrame->GetReadPtr(PLANAR_U);
				RightFramepU = RightFrame->GetWritePtr(PLANAR_U);
				LeftFramepV = LeftFrame->GetReadPtr(PLANAR_V);
				RightFramepV = RightFrame->GetWritePtr(PLANAR_V);
		}

		
		if(delta<8)
			return LeftFrame;
		wwd=delta/2;
		if(delta>maxroll)
			wwd=maxroll/2;
		if(bwd+bht-delta<maxroll)
			wwd=(bwd+bht-delta)/2;
		if(wwd<4)
			return RightFrame;
		int wwd2=wwd/2;
		if((*dir=='s' || *dir=='S')&& (*(dir+1)=='w' || *(dir+1)=='W'))
			
		{			//LeftFrame rolls out sw wards unmasking RightFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB32(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB32(LeftFramep+4*(bwd-delta+h+wwd)+(bht-h-1)*rpitch,
									RightFramep+4*(bwd-delta+h-wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB32(LeftFramep+4*(wwd)+(bht-h-1)*rpitch,
									RightFramep+4*(wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
					}


				}
			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB24(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB24(LeftFramep+3*(bwd-delta+h+wwd)+(bht-h-1)*rpitch,
									RightFramep+3*(bwd-delta+h-wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB24(LeftFramep+3*(wwd)+(bht-h-1)*rpitch,
									RightFramep+3*(wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
					}


				}

			}

			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYUY2(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYUY2(LeftFramep+2*(bwd-delta+h+wwd)+h*rpitch,
									RightFramep+2*(bwd-delta+h-wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYUY2(LeftFramep+2*(wwd)+h*rpitch,
									RightFramep+2*(wwd/2)+h*lpitch,
									wwd,shade);
					}


				}

			}

			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYV12(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramep+(bwd-delta+h+wwd)+h*rpitch,
									RightFramep+(bwd-delta+h-wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramep+(wwd)+h*rpitch,
									RightFramep+(wwd/2)+h*lpitch,
									wwd,shade);
					}
					if((h & 1)==1)
						continue;

					if(delta-h<8)
					{
						CopyHLineYV12(LeftFramepU+h/2*lpitchUV,
									RightFramepU+h/2*rpitchUV,
										bwd/2);
						CopyHLineYV12(LeftFramepV+h/2*lpitchUV,
									RightFramepV+h/2*rpitchUV,
										bwd/2);
					}

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramepU+(bwd-delta+h+wwd)/2+h/2*rpitchUV,
									RightFramepU+(bwd-delta+h-wwd/2)/2+h/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+h/2*lpitchUV,
									RightFramepU+h/2*rpitchUV,
										(bwd-delta+h-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(bwd-delta+h+wwd)/2+h/2*rpitchUV,
									RightFramepV+(bwd-delta+h-wwd/2)/2+h/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+h/2*lpitchUV,
									RightFramepV+h/2*rpitchUV,
										(bwd-delta+h-wwd)/2);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramepU+(wwd)/2+h/2*rpitchUV,
									RightFramepU+(wwd/2)/2+h/2*lpitchUV,
									wwd/2,0);
						PeelzHLineYV12(LeftFramepV+(wwd)/2+h/2*rpitchUV,
									RightFramepV+(wwd/2)/2+h/2*lpitchUV,
									wwd/2,0);
					}



				}

			}
		}

		if((*dir=='n' || *dir=='N')&& (*(dir+1)=='w' || *(dir+1)=='W'))
			
		{			//LeftFrame peels nw wards unmasking RightFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB32(LeftFramep+(h)*lpitch,
									RightFramep+(h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB32(LeftFramep+4*(bwd-delta+h+wwd)+(h)*rpitch,
									RightFramep+4*(bwd-delta+h-wwd/2)+(h)*lpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+(h)*lpitch,
									RightFramep+(h)*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB32(LeftFramep+4*(wwd)+(h)*rpitch,
									RightFramep+4*(wwd/2)+(h)*lpitch,
									wwd,shade);
					}


				}

			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB24(LeftFramep+(h)*lpitch,
									RightFramep+(h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB24(LeftFramep+3*(bwd-delta+h+wwd)+(h)*rpitch,
									RightFramep+3*(bwd-delta+h-wwd/2)+(h)*lpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+(h)*lpitch,
									RightFramep+(h)*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB24(LeftFramep+3*(wwd)+(h)*rpitch,
									RightFramep+3*(wwd/2)+(h)*lpitch,
									wwd,shade);
					}


				}

			}

			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYUY2(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYUY2(LeftFramep+2*(bwd-delta+h+wwd)+(bht-1-h)*rpitch,
									RightFramep+2*(bwd-delta+h-wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYUY2(LeftFramep+2*(wwd)+(bht-1-h)*rpitch,
									RightFramep+2*(wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
					}


				}

			}
			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYV12(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramep+(bwd-delta+h+wwd)+(bht-1-h)*rpitch,
									RightFramep+(bwd-delta+h-wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramep+(wwd)+(bht-1-h)*rpitch,
									RightFramep+(wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
					}


				
				if((h & 1)==1)
					continue;
				if(delta-h<8)
				{
					CopyHLineYV12(LeftFramepU+(bht-1-h)/2*lpitchUV,
									RightFramepU+(bht-1-h)/2*rpitchUV,
										bwd/2);
					CopyHLineYV12(LeftFramepV+(bht-1-h)/2*lpitchUV,
									RightFramepV+(bht-1-h)/2*rpitchUV,
										bwd/2);
				}

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramepU+(bwd-delta+h+wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepU+(bwd-delta+h-wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+(bht-1-h)/2*lpitchUV,
									RightFramepU+(bht-1-h)/2*rpitchUV,
										(bwd-delta+h-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(bwd-delta+h+wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepV+(bwd-delta+h-wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+(bht-1-h)/2*lpitchUV,
									RightFramepV+(bht-1-h)/2*rpitchUV,
										(bwd-delta+h-wwd)/2);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramepU+(wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepU+(wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
						PeelzHLineYV12(LeftFramepV+(wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepV+(wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
					}


				}
			}
			
				
		}
		if((*dir=='s' || *dir=='S')&& (*(dir+1)=='e' || *(dir+1)=='E'))
			
		{			//LeftFrame peels se wards unmasking RightFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB32(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB32(LeftFramep+4*(delta-h-wwd)+(bht-h-1)*rpitch,
									RightFramep+4*(delta-h+wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+(bht-1-h)*lpitch+4*(delta-h+wwd),
									RightFramep+(bht-1-h)*rpitch+4*(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB32(LeftFramep+4*(bwd-wwd)+(bht-h-1)*rpitch,
									RightFramep+4*(bwd-wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
					}


				}

			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB24(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB24(LeftFramep+3*(delta-h-wwd)+(bht-h-1)*rpitch,
									RightFramep+3*(delta-h+wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+(bht-1-h)*lpitch+3*(delta-h+wwd),
									RightFramep+(bht-1-h)*rpitch+3*(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB24(LeftFramep+3*(bwd-wwd)+(bht-h-1)*rpitch,
									RightFramep+3*(bwd-wwd/2)+(bht-h-1)*lpitch,
									wwd,shade);
					}


				}

			}

			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYUY2(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYUY2(LeftFramep+2*(delta-h-wwd)+h*rpitch,
									RightFramep+2*(delta-h+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+h*lpitch+2*(delta-h+wwd),
									RightFramep+h*rpitch+2*(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYUY2(LeftFramep+2*(bwd-wwd)+h*rpitch,
									RightFramep+2*(bwd-wwd/2)+h*lpitch,
									wwd,shade);
					}


				}
		
			}

			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYV12(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramep+(delta-h-wwd)+h*rpitch,
									RightFramep+(delta-h+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+h*lpitch+(delta-h+wwd),
									RightFramep+h*rpitch+(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramep+(bwd-wwd)+h*rpitch,
									RightFramep+(bwd-wwd/2)+h*lpitch,
									wwd,shade);
					}
					if((h & 1)==1)
						continue;
					if(delta-h<8)
					{
						CopyHLineYV12(LeftFramepU+h/2*lpitchUV,
									RightFramepU+h/2*rpitchUV,
										bwd/2);
						CopyHLineYV12(LeftFramepV+h/2*lpitchUV,
									RightFramepV+h/2*rpitchUV,
										bwd/2);
					}

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramepU+(delta-h-wwd)/2+h/2*rpitchUV,
									RightFramepU+(delta-h+wwd/2)/2+h/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+h/2*lpitchUV+(delta-h+wwd)/2,
									RightFramepU+h/2*rpitchUV+(delta-h+wwd)/2,
										(bwd-delta+h-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(delta-h-wwd)/2+h/2*rpitchUV,
									RightFramepV+(delta-h+wwd/2)/2+h/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+h/2*lpitchUV+(delta-h+wwd)/2,
									RightFramepV+h/2*rpitchUV+(delta-h+wwd)/2,
										(bwd-delta+h-wwd)/2);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramepU+(bwd-wwd)/2+h/2*rpitchUV,
									RightFramepU+(bwd-wwd/2)/2+h/2*lpitchUV,
									wwd/2,0);
					}


				}
		
			}
		}

		if((*dir=='n' || *dir=='N')&& (*(dir+1)=='e' || *(dir+1)=='E'))
			
		{			//LeftFrame peels se wards unmasking RightFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB32(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB32(LeftFramep+4*(delta-h-wwd)+h*rpitch,
									RightFramep+4*(delta-h+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineRGB32(LeftFramep+h*lpitch+4*(delta-h+wwd),
									RightFramep+h*rpitch+4*(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB32(LeftFramep+4*(bwd-wwd)+h*rpitch,
									RightFramep+4*(bwd-wwd/2)+h*lpitch,
									wwd,shade);
					}


				}
	
			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineRGB24(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineRGB24(LeftFramep+3*(delta-h-wwd)+h*rpitch,
									RightFramep+3*(delta-h+wwd/2)+h*lpitch,
									wwd,shade);
						CopyHLineRGB24(LeftFramep+h*lpitch+3*(delta-h+wwd),
									RightFramep+h*rpitch+3*(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineRGB24(LeftFramep+3*(bwd-wwd)+h*rpitch,
									RightFramep+3*(bwd-wwd/2)+h*lpitch,
									wwd,shade);
					}


				}
			
			}

			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYUY2(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYUY2(LeftFramep+2*(delta-h-wwd)+(bht-1-h)*rpitch,
									RightFramep+2*(delta-h+wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
						CopyHLineYUY2(LeftFramep+(bht-1-h)*lpitch+2*(delta-h+wwd),
									RightFramep+(bht-1-h)*rpitch+2*(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYUY2(LeftFramep+2*(bwd-wwd)+(bht-1-h)*rpitch,
									RightFramep+2*(bwd-wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
					}


				}
			
			}
			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<8)
						CopyHLineYV12(LeftFramep+(bht-1-h)*lpitch,
									RightFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramep+(delta-h-wwd)+(bht-1-h)*rpitch,
									RightFramep+(delta-h+wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
						CopyHLineYV12(LeftFramep+(bht-1-h)*lpitch+(delta-h+wwd),
									RightFramep+(bht-1-h)*rpitch+(delta-h+wwd),
										bwd-delta+h-wwd);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramep+(bwd-wwd)+(bht-1-h)*rpitch,
									RightFramep+(bwd-wwd/2)+(bht-1-h)*lpitch,
									wwd,shade);
					}

					if(delta-h<8)
					{
						CopyHLineYV12(LeftFramepU+(bht-1-h)/2*lpitchUV,
									RightFramepU+(bht-1-h)/2*rpitchUV,
										bwd/2);
						CopyHLineYV12(LeftFramepV+(bht-1-h)/2*lpitchUV,
									RightFramepV+(bht-1-h)/2*rpitchUV,
										bwd/2);
					}

					if(delta-h>=8 && delta-h<bwd-maxroll/2)
					{
						wwd=(delta-h)/2;
						if	(wwd>maxroll/2)
							wwd=maxroll/2;
						PeelzHLineYV12(LeftFramepU+(delta-h-wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepU+(delta-h+wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepU+(bht-1-h)/2*lpitchUV+(delta-h+wwd)/2,
									RightFramepU+(bht-1-h)/2*rpitchUV+(delta-h+wwd)/2,
										(bwd-delta+h-wwd)/2);
						PeelzHLineYV12(LeftFramepV+(delta-h-wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepV+(delta-h+wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
						CopyHLineYV12(LeftFramepV+(bht-1-h)/2*lpitchUV+(delta-h+wwd)/2,
									RightFramepV+(bht-1-h)/2*rpitchUV+(delta-h+wwd)/2,
										(bwd-delta+h-wwd)/2);
					}


					if(delta-h>bwd-maxroll/2 && delta-h<bwd-8)
					{
						wwd=(bwd-delta+h);
						PeelzHLineYV12(LeftFramepU+(bwd-wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepU+(bwd-wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
						PeelzHLineYV12(LeftFramepV+(bwd-wwd)/2+(bht-1-h)/2*rpitchUV,
									RightFramepV+(bwd-wwd/2)/2+(bht-1-h)/2*lpitchUV,
									wwd/2,0);
					}



				}
			
			}

		}
					

			return RightFrame;	
		
	
}

/********** 



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransSlantRollOut(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	
				
	return new TransSlantRollOut(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("se"),	// Transition direction ne, se,nw,sw
						args[4].AsInt(80),	// diameter of roll
						args[5].AsInt(127),	// shade. 0 noshading, -255 to 255
											// -ve edges shaded more, +ve center shaded more
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransSlantRollOut", "cci[dir]s[rollDia]i[shade]i", Create_TransSlantRollOut, 0);
				
    return "TransSlantRollOut  Plugin";
}
*/
