/******************************************************************************
TransSlantWipe filter plugin for Avisynth by V.C.Mohan
The RightClip  Rolls In onto the  LefttClip  in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also 
used partly adapted function coding 
from Avisynth and simple sample. Roll Diameter and
 shading along roll length are other params
 Modified on 24-9-04
********************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "LineZ.cpp"
*/
class TransSlantWipe : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;

		int overlap;
		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		
		 
  public:
							//Definition of function
    TransSlantWipe(PClip _child, PClip _RightClip, 
					int _overlap,const char* _dir,	  
					IScriptEnvironment* env) ;	
			
 	~TransSlantWipe();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
	// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransSlantWipe::TransSlantWipe(PClip _child, PClip _RightClip,
							   int _overlap,const char* _dir,
								IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
						overlap(_overlap),dir(_dir)
			
{
	
const VideoInfo& rvi= RightClip->GetVideoInfo();

char * Tname="TransSlantWipe";


if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransSlantWipe:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransSlantWipe: This color format is not supported here");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("TransSlantWipe: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransSlantWipe: Height/width of frame are not multiple of 4");
if(!(*dir=='n') && !(*dir=='N') && !(*dir=='s') && !(*dir=='S') &&
	!(*(dir+1)=='e') && !(*(dir+1)=='E') && !(*(dir+1)=='w') && !(*(dir+1)=='W'))
	env->ThrowError("TransSlantWipe:ne, se, nw and sw only  are valid directions");
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);

	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}

	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransSlantWipe::~TransSlantWipe() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/

void TransSlantWipe::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransSlantWipe::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}

//-------------------------------------------------------------------------------------	
	PVideoFrame __stdcall TransSlantWipe::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
	
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		int delta =bwd+bht-((n+1)*(bwd+bht))/(overlap+1);
		delta = delta & 0xFFFFFFFC;
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		int  wht, wwd;
		env->MakeWritable(&LeftFrame);
		const unsigned char* RightFramep= RightFrame->GetReadPtr();
		unsigned char * LeftFramep= LeftFrame->GetWritePtr();
		const unsigned char * rpU, *rpV;
		unsigned char *lpU, *lpV;

		if(vi.IsYV12())
		{

			rpU = RightFrame->GetReadPtr(PLANAR_U);
			rpV = RightFrame->GetReadPtr(PLANAR_V);
			lpU = LeftFrame->GetWritePtr(PLANAR_U);
			lpV = LeftFrame->GetWritePtr(PLANAR_V);
		}

		
		if(delta<2)
			return RightFrame;
		
		if((*dir=='n' || *dir=='N')&& (*(dir+1)=='e' || *(dir+1)=='E'))
			
		{			//RightFrame wipes ne wards masking LeftFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB32(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd);
					if(delta-h>=2 && delta-h<bwd)					
						CopyHLineRGB32(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd-delta+h);
				}
				return LeftFrame;
			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB24(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd);
					if(delta-h>=2 && delta-h<bwd)						
						CopyHLineRGB24(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd-delta+h);
					
				}
				return LeftFrame;
			}
			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineYUY2(RightFramep+h*lpitch,
									LeftFramep+h*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)
					
						CopyHLineYUY2(RightFramep+h*lpitch,
									LeftFramep+h*rpitch,
										bwd-delta+h);

				}

				return LeftFrame;
			}

			if(vi.IsYV12())
			{
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);

				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
					{
						CopyHLineYV12(RightFramep+h*rpitch,
									LeftFramep+h*lpitch,
										bwd);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+h/2*rpitchUV,
									lpU+h/2*lpitchUV,
										bwdUV);
						CopyHLineYV12(rpV+h/2*rpitchUV,
									lpV+h/2*lpitchUV,
										bwdUV);
					}

					if(delta-h>=2 && delta-h<bwd)
					{
					
						CopyHLineYV12(RightFramep+h*rpitch,
									LeftFramep+h*lpitch,
										bwd-delta+h);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+h/2*rpitchUV,
									lpU+h/2*lpitchUV,
										(bwd-delta+h)/2);
						CopyHLineYV12(rpV+h/2*rpitchUV,
									lpV+h/2*lpitchUV,
										(bwd-delta+h)/2);
					}

				}

				return LeftFrame;
			}

			
		}

		if((*dir=='s' || *dir=='S')&& (*(dir+1)=='e' || *(dir+1)=='E'))
			
		{			//LeftFrame peels se wards unmasking RightFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB32(RightFramep+(h)*lpitch,
									LeftFramep+(h)*rpitch,
										bwd);
					if(delta-h>=2 && delta-h<bwd)				

						CopyHLineRGB32(RightFramep+(h)*lpitch,
									LeftFramep+(h)*rpitch,
										bwd-delta+h);
				
				}
				return LeftFrame;
			}

			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineYUY2(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)					
						CopyHLineYUY2(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd-delta+h);
				}
				return LeftFrame;
			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB24(RightFramep+(h)*lpitch,
									LeftFramep+(h)*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)
						CopyHLineRGB24(RightFramep+(h)*lpitch,
									LeftFramep+(h)*rpitch,
										bwd-delta+h);
				}
				return LeftFrame;
			}
			if(vi.IsYV12())
			{
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
					{
						CopyHLineYV12(RightFramep+(bht-1-h)*rpitch,
									LeftFramep+(bht-1-h)*lpitch,
										bwd);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+(bht-1-h)/2*rpitchUV,
									lpU+(bht-1-h)/2*lpitchUV,
										bwdUV);
						CopyHLineYV12(rpV+(bht-1-h)/2*rpitchUV,
									lpV+(bht-1-h)/2*lpitchUV,
										bwdUV);
					}


					if(delta-h>=2 && delta-h<bwd)
					{
						CopyHLineYV12(RightFramep+(bht-1-h)*rpitch,
									LeftFramep+(bht-1-h)*lpitch,
										bwd-delta+h);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+(bht-1-h)/2*rpitchUV,
									lpU+(bht-1-h)/2*lpitchUV,
										(bwd-delta+h)/2);
						CopyHLineYV12(rpV+(bht-1-h)/2*rpitchUV,
									lpV+(bht-1-h)/2*lpitchUV,
										(bwd-delta+h)/2);
					}
				}
				return LeftFrame;
			}
		}
		if((*dir=='n' || *dir=='N')&& (*(dir+1)=='w' || *(dir+1)=='W'))
			
		{			//RightFrame rolls nw wards masking LeftFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB32(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)
						CopyHLineRGB32(RightFramep+(bht-1-h)*rpitch+4*(delta-h),
									LeftFramep+(bht-1-h)*lpitch+4*(delta-h),
										bwd-delta+h);
				}
				return LeftFrame;
			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB24(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)

						CopyHLineRGB24(RightFramep+(bht-1-h)*rpitch+3*(delta-h),
									LeftFramep+(bht-1-h)*lpitch+3*(delta-h),
										bwd-delta+h);
				}
				return LeftFrame;
			}

			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineYUY2(RightFramep+h*lpitch,
									LeftFramep+h*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)
					
						CopyHLineYUY2(RightFramep+h*rpitch+2*(delta-h),
									LeftFramep+h*lpitch+2*(delta-h),
										bwd-delta+h);
				}
				return LeftFrame;
			}

			if(vi.IsYV12())
			{
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
					{
						CopyHLineYV12(RightFramep+h*rpitch,
									LeftFramep+h*lpitch,
										bwd);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+h/2*rpitchUV,
									lpU+h/2*lpitchUV,
										bwdUV);
						CopyHLineYV12(rpV+h/2*rpitchUV,
									lpV+h/2*lpitchUV,
										bwdUV);
					}

					if(delta-h>=2 && delta-h<bwd)
					{
					
						CopyHLineYV12(RightFramep+h*rpitch+(delta-h),
									LeftFramep+h*lpitch+(delta-h),
										bwd-delta+h);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+h/2*rpitchUV+(delta-h)/2,
									lpU+h/2*lpitchUV+(delta-h)/2,
										(bwd-delta+h)/2);
						CopyHLineYV12(rpV+h/2*rpitchUV+(delta-h)/2,
									lpV+h/2*lpitchUV+(delta-h)/2,
										(bwd-delta+h)/2);
					}
				}
				return LeftFrame;
			}
		}

		if((*dir=='s' || *dir=='S')&& (*(dir+1)=='w' || *(dir+1)=='W'))
			
		{			//RightFrame rolls sw wards masking LeftFrame

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB32(RightFramep+h*lpitch,
									LeftFramep+h*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)
					
						CopyHLineRGB32(RightFramep+h*lpitch+4*(delta-h),
									LeftFramep+h*rpitch+4*(delta-h),
										bwd-delta+h);
				}
				return LeftFrame;
			}

			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineRGB24(RightFramep+h*lpitch,
									LeftFramep+h*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)
						CopyHLineRGB24(RightFramep+h*lpitch+3*(delta-h),
									LeftFramep+h*rpitch+3*(delta-h),
										bwd-delta+h);
				}
				return LeftFrame;
			}

			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
						CopyHLineYUY2(RightFramep+(bht-1-h)*lpitch,
									LeftFramep+(bht-1-h)*rpitch,
										bwd);

					if(delta-h>=2 && delta-h<bwd)
						CopyHLineYUY2(RightFramep+(bht-1-h)*lpitch+2*(delta-h),
									LeftFramep+(bht-1-h)*rpitch+2*(delta-h),
										bwd-delta+h);
				}
				return LeftFrame;
			}
			if(vi.IsYV12())
			{
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
				for(int h=0;h<bht-1;h++)
				{

					if(delta-h<2)
					{
						CopyHLineYV12(RightFramep+(bht-1-h)*rpitch,
									LeftFramep+(bht-1-h)*lpitch,
										bwd);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+(bht-1-h)/2*rpitchUV,
									lpU+(bht-1-h)/2*lpitchUV,
										bwdUV);
						CopyHLineYV12(rpV+(bht-1-h)/2*rpitchUV,
									lpV+(bht-1-h)/2*lpitchUV,
										bwdUV);
					}

					if(delta-h>=2 && delta-h<bwd)
					{
						CopyHLineYV12(RightFramep+(bht-1-h)*rpitch+(delta-h),
									LeftFramep+(bht-1-h)*lpitch+(delta-h),
										bwd-delta+h);
						if((h & 1)==1 )
							continue;
						CopyHLineYV12(rpU+(bht-1-h)/2*rpitchUV+(delta-h)/2,
									lpU+(bht-1-h)/2*lpitchUV+(delta-h)/2,
										(bwd-delta+h)/2);
						CopyHLineYV12(rpV+(bht-1-h)/2*rpitchUV+(delta-h)/2,
									lpV+(bht-1-h)/2*lpitchUV+(delta-h)/2,
										(bwd-delta+h)/2);
					}
				}
				return LeftFrame;
			}

		}
					

			
		
	
}

/********** 



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransSlantWipe(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransSlantWipe(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("se"),	// Transition direction ne, se,nw,sw
						
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransSlantWipe", "cci[dir]s", Create_TransSlantWipe, 0);
				
    return "TransSlantWipe  Plugin";
}
*/
