/******************************************************************************
TransSlideIn filter plugin for Avisynth by V.C.Mohan
This RightClip gradually Masks  a stationary LeftClip  in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
Modified 24-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
*/

class TransSlideIn : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;
		int overlap;
		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransSlideIn(PClip _child, PClip _RightClip, 
					int _overlap,const char* _dir,	  
					IScriptEnvironment* env) ;	
			
 	~TransSlideIn();				//destructor
//	copywindow(const unsigned char* _R,unsigned char* _L, int _w, int _h, int _rp, int _lp);
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransSlideIn::TransSlideIn(PClip _child, PClip _RightClip,
							   int _overlap,const char* _dir,
								IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
						overlap(_overlap),dir(_dir) 
			
{
	
const VideoInfo& rvi= RightClip->GetVideoInfo();

	char * Tname="TransSlideIn";

	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransSlideIn:Clips have differing pixel_types");
	if (vi.width != rvi.width || vi.height != rvi.height)
		env->ThrowError("%s:Clips have differing frame dimensions", Tname);

	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransSlideIn: This color format is not supported here");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransSlideIn::~TransSlideIn() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/

void TransSlideIn::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransSlideIn::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//---------------------------------------------------------------------------------	
	PVideoFrame __stdcall TransSlideIn::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		PVideoFrame ResultFrame;
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		
		const	int bwd = lvi.width; 
        const	int bht = LeftFrame->GetHeight();	
		int deltay =((n+1)*bht)/(overlap+1);
		int deltax = ((n+1)*bwd)/(overlap+1);
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		deltax= deltax & 0xFFFFFFFE;
		deltay= deltay & 0xFFFFFFFE;
		int LeftFramex,LeftFramey, wht,wwd,RightFramex,RightFramey ;

		if( strcmp(dir, "north")==0)
			
				{
					// RightFrame moves north gradually covering LeftFrame
				LeftFramex =0;
				LeftFramey = bht-deltay;
				wht = deltay;
				wwd = bwd;
				RightFramex=0;
				RightFramey=0;
				}

		if(strcmp(dir, "south")==0)
		
				{
					// RightFrame moves south gradually covering LeftFrame
				LeftFramex =0;
				LeftFramey = 0;
				wht = deltay;
				wwd = bwd;
				RightFramex=0;
				RightFramey=bht-deltay;
				
				}

		if(strcmp(dir, "east")==0)

				{
					// RightFrame moves east gradually covering LeftFrame
				LeftFramex =0;
				LeftFramey = 0;
				wht = bht;
				wwd = deltax;
				RightFramex=wwd-deltax;
				RightFramey=0;
				
				}

		if(strcmp(dir, "west")==0)

				{
					// RightFrame moves west masking LeftFrame
					LeftFramex =bwd-deltax;
					LeftFramey = 0;
					wht = bht;
					wwd = deltax;
					RightFramex=0;
					RightFramey=0;
				
				}

		if(strcmp(dir, "ne")==0)

				{
					// RightFrame moves north east gradually masking LeftFrame
				LeftFramex =0;
				LeftFramey = bht-deltay;
				wht = deltay;
				wwd = deltax;
				RightFramex=bwd-deltax;
				RightFramey=0;
				
				}

		if(strcmp(dir, "se")==0)
				{
					// RightFrame moves south east covering LeftFrame
				LeftFramex =0;
				LeftFramey =0;
				wht = deltay;
				wwd = deltax;
				RightFramex=bwd-deltax;
				RightFramey=bht-deltay;
				
				}

		if(strcmp(dir, "nw")==0)

				{
					// RightFrame moves north west gradually masking LeftFrame
				LeftFramex =bwd-deltax;
				LeftFramey = bht-deltay;
				wht = deltay;
				wwd = deltax;
				RightFramex=0;
				RightFramey=0;
				
				}

		if(strcmp(dir, "sw")==0)

				{
					// RightFrame moves south west gradually masking LeftFrame
				LeftFramex = bwd-deltax;
				LeftFramey = 0;
				wht =deltay;
				wwd =deltax;
				RightFramex=0;
				RightFramey=bht-deltay;
				
				}

		if(strcmp(dir, "center")==0)

				{
					// Right emerges from center of LeftFrame in all directions
				LeftFramex =(bwd-deltax)/2;
				LeftFramey =(bht- deltay)/2;
				wht =deltay;
				wwd =deltax;
				RightFramex= (bwd-deltax)/2;
				RightFramey= (bht-deltay)/2;				
				}

			wwd=wwd & 0xFFFFFFFE; //  make even number of pixels
			if(lvi.IsYV12())
				wht=wht & 0xFFFFFFFE; // for YUV2 format 
			if (wwd==0 || wht==0)
				return LeftFrame; // no work to be done

			env->MakeWritable(&LeftFrame);
			unsigned char* LeftFramep= LeftFrame->GetWritePtr();
			const unsigned char* RightFramep= RightFrame->GetReadPtr();
			
			if(lvi.IsRGB24())
			{		
				// RGB image is stored in a vertically flipped fashion, So from bottom up copy.
				// 3 bytes per pixel format.
				RightFramep= RightFramep+3*RightFramex+(bht-wht-RightFramey)*rpitch;
				LeftFramep= LeftFramep+3*LeftFramex+(bht-wht-LeftFramey)*lpitch;
				for (int h=0; h<wht; h++)
					{

					for (int w=0; w< 3*wwd; w++)
						{
						*(LeftFramep +w)=*(RightFramep+w);
						}
					LeftFramep=LeftFramep+lpitch;
					RightFramep=RightFramep+rpitch;
					}
				return LeftFrame;
			}
					
			if(lvi.IsRGB32())
			{
				// RGB frame is stored in memory vertically flipped
				// 4 bytes or one 32 bit word per pixel. so faster to treat in word size
				// copy bottom up of window

				RightFramep= RightFramep+4*RightFramex+(bht-wht-RightFramey)*rpitch;
				LeftFramep= LeftFramep+4*LeftFramex+(bht-wht-LeftFramey)*lpitch;
				for(int h=0; h<wht; h++)
				{
					for (int w=0; w< wwd; w++)
					{

						*((unsigned int *)LeftFramep+ w) = *((unsigned int *)RightFramep+w);
					}
					LeftFramep=LeftFramep+lpitch;
					RightFramep= RightFramep+rpitch;
				}
			}

			if(lvi.IsYUY2())
			{
				// This code deals with YUY2 colourspace where each 4 byte sequence represents
				// 2 pixels, (Y1, U, Y2 and then V). So always deal with even number of pixels.

				RightFramep= RightFramep+2*RightFramex+RightFramey*rpitch;
				LeftFramep= LeftFramep+2*LeftFramex+LeftFramey*lpitch; // positioned window top left
				for (int h=0; h<wht; h++)
				{
					for  (int w=0; w<wwd/2; w++)	// not very clear why divide by 4 inplace of 2
						// because the width is for each color plane. So for each 4 bits.
					{
						// copy two pixels at a time as a 32 bit word
						*((unsigned int *)LeftFramep+ w) = *((unsigned int *)RightFramep+w);
					}
					LeftFramep=LeftFramep+lpitch;
					RightFramep= RightFramep+rpitch;
				}

			}


			if(lvi.IsYV12())	// planar
						// This code deals with YV12 colourspace where the Y, U and V information are
						// stored in completely separate memory areas
		// This colourspace is the most memory efficient but usually requires 3 separate loops
		// However, it can actually be easier to deal with than YUY2 depending on your filter algorithim		
		// So first of all deal with the Y Plane
			{
				RightFramep= RightFramep+RightFramex+RightFramey*rpitch;
				LeftFramep= LeftFramep+LeftFramex+LeftFramey*lpitch; // position top left Y plane
				for (int h=0; h<wht; h++)
				{
					for  (int w=0; w<wwd; w++)
					{
						*(LeftFramep+w)= *(RightFramep+w); //copy each byte of window from source to left frame
					}
					LeftFramep=LeftFramep+lpitch;
					RightFramep= RightFramep+rpitch;
				}

						// This section of code deals with the U and V planes of planar formats (e.g. YV12)
		// So first of all we have to get the additional info on the U and V planes

		const int LeftFrame_pitchUV = LeftFrame->GetPitch(PLANAR_U);	// The pitch,height and width information
		const int LeftFrame_widthUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
		const int LeftFrame_heightUV = LeftFrame->GetHeight(PLANAR_U);	// the U and V planes so we only the U
		const int RightFrame_pitchUV = RightFrame->GetPitch(PLANAR_U);	// plane values and use them for V as
		const int RightFrame_widthUV = RightFrame->GetRowSize(PLANAR_U);	// well
		const int RightFrame_heightUV = RightFrame->GetHeight(PLANAR_U);	//
		
		//Copy U plane window
				RightFramep = RightFrame->GetReadPtr(PLANAR_U);
				LeftFramep = LeftFrame->GetWritePtr(PLANAR_U);
				RightFramep=RightFramep+RightFramex/2+RightFramey/2*RightFrame_pitchUV;
				LeftFramep=LeftFramep+LeftFramex/2+LeftFramey/2*LeftFrame_pitchUV;

				for ( h=0; h < wht/2;h++)	// as the U plane height is half the Y plane
				{

					for (int w = 0; w <wwd/2; w++)	// only scans half of width bytes of a line because  U=Y/2
					{
						*(LeftFramep + w) = *(RightFramep + w);
					}
					RightFramep = RightFramep + RightFrame_pitchUV;
					LeftFramep = LeftFramep + LeftFrame_pitchUV;
				}
		// end of U plane window copying 

			//Copy V plane window
				RightFramep = RightFrame->GetReadPtr(PLANAR_V);
				LeftFramep = LeftFrame->GetWritePtr(PLANAR_V);
				RightFramep=RightFramep+RightFramex/2+RightFramey/2*RightFrame_pitchUV;
				LeftFramep=LeftFramep+LeftFramex/2+LeftFramey/2*LeftFrame_pitchUV;

				for ( h=0; h < wht/2;h++)	// as the V plane height is half the Y plane
				{

					for (int w = 0; w <wwd/2; w++)	// only scans half of width bytes of a line because  V=Y/2
					{
						*(LeftFramep + w) = *(RightFramep + w);
					}
					RightFramep = RightFramep + RightFrame_pitchUV;
					LeftFramep = LeftFramep + LeftFrame_pitchUV;
				}
		// end of V plane window copying 

				
			}
			return LeftFrame;
	}
//_______________________________________________________________________________________
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransSlideIn(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransSlideIn(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("center"),	// Transition direction 
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransSlideIn", "cci[dir]s", Create_TransSlideIn, 0);
				
    return "TransSlideIn Transition Mask Plugin";
}
*/
