/******************************************************************************
TransSprite filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
Modified 24-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
#include "FastResizer.cpp"
*/
class TransSprite : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;
		int overlap;
		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransSprite(PClip _child, PClip _RightClip, 
					int _overlap,const char* _dir,	  
					IScriptEnvironment* env) ;	
			
 	~TransSprite();				//destructor
	copywindow(unsigned char* R,const unsigned char* L, int w, int h, int rp, int lp);
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransSprite::TransSprite(PClip _child, PClip _RightClip,
							 int _overlap, const char* _dir,
							 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
						overlap(_overlap),dir(_dir)
{
char * Tname="TransSprite";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("Sprite:Clips have differing pixel_types");

if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("Sprite: This color format is not yet supported ");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("Sprite: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("Sprite: Height/width of frame are not multiple of 4");
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransSprite::~TransSprite() 
{

// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
void TransSprite::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransSprite::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//---------------------------------------------------------------------------------	

	PVideoFrame __stdcall TransSprite::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();

		int kb=vi.BitsPerPixel()/8;
		int kby=vi.IsYUY2()?1:kb;
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		int deltay =((n+1)*bht)/(overlap+1);
		int deltax = ((n+1)*bwd)/(overlap+1);
		deltay = deltay & 0xFFFFFFFC;
		deltax=deltax & 0xFFFFFFFC;

		if(*dir=='u' || *dir=='U'||*dir=='d' || *dir=='D')
		{
			if(deltay<2)
				return LeftFrame;
			else if(bht-deltay<2)
				return RightFrame;
		}
		else if(*dir=='l' || *dir=='L'||*dir=='r' || *dir=='R')
		{
			if(deltax<2)
				return LeftFrame;
			else if(bwd-deltax<2)
				return RightFrame;
		}
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();

		int LeftFramex, LeftFramey, Rp,Lp,rht,rwd,lht,lwd, RightFramex, RightFramey;

		if( *dir=='u' || *dir=='U')
			
		{			//RightFrame turns upward turning out LeftFrame
			
			rht=deltay;
			rwd=bwd;
			lht = bht-deltay;
			lwd= bwd;			
			RightFramex= 0;
			RightFramey =lht;
			LeftFramex=0;
			LeftFramey=0;
		}
			

		if( *dir=='d' || *dir=='D')
			
		{			//RightFrame turning downward turns out LeftFrame
			rht=deltay;
			rwd=bwd;
			lht = bht-deltay;
			lwd= bwd;			
			RightFramex= 0;
			RightFramey =0;
			LeftFramex=0;
			LeftFramey=rht;
		}
			
		if( *dir=='r' || *dir=='R')
			
		{		//RightFrame turning right turns out LeftFrame
			rht=bht;
			rwd=deltax;
			lht = bht;
			lwd= bwd-deltax;			
			RightFramex= 0;
			RightFramey= 0;
			LeftFramex=rwd;
			LeftFramey=0;
		}
			

		if( *dir=='l' || *dir=='L')
			
		{		//RightFrame turning left turns out LeftFrame
			rht = bht;
			rwd= deltax;
			lht= bht;
			lwd=bwd-deltax;
			RightFramex= lwd;
			RightFramey =0;
			LeftFramex=0;
			LeftFramey=0;
		}
			
			
		
		PVideoFrame work = env->NewVideoFrame(vi);
		unsigned char* workp =work->GetWritePtr();
		int wpitch=work->GetPitch();
		const unsigned char* RightFramep= RightFrame->GetReadPtr();
		const unsigned char * LeftFramep= LeftFrame->GetReadPtr();
			// sides blackening for depth effect. Min(zero) taper when fully visible 
		int tratio =10;
		int rwtaper = (double)(bht-rht)/tratio;
		int lwtaper = (double)(bht-lht)/tratio;
		int rhtaper = (double)(bwd-rwd)/tratio;
		int lhtaper = (double)(bwd-lwd)/tratio;
		if(vi.IsRGB32() || vi.IsRGB24())
		{
			Rp= kb*RightFramex+(bht-rht-RightFramey)*rpitch;
			Lp= kb*LeftFramex+(bht-lht-LeftFramey)*lpitch;
		}
		if(vi.IsYUY2() || vi.IsYV12())
		{
			Rp= kb*RightFramex+(RightFramey)*rpitch;
			Lp= kb*LeftFramex+(LeftFramey)*lpitch;
		}
		if(bht==lht && bht==rht)
		{
							// no vert resizing
							// resize left frame
			if(lvi.IsRGB24())
			{
				HFastResizerRGB24(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
									lpitch,wpitch,false);
							// resize Right Frame
				HFastResizerRGB24(RightFramep,workp+Rp,bht,bwd,rht,rwd,
									rpitch,wpitch,false);
			}
			if(lvi.IsRGB32())
			{
				HFastResizerRGB32(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
									lpitch,wpitch,false);
							// resize Right Frame
				HFastResizerRGB32(RightFramep,workp+Rp,bht,bwd,rht,rwd,
									rpitch,wpitch,false);
			}
			if(lvi.IsYUY2())
			{
				HFastResizerYUY2(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
									lpitch,wpitch,false);
							// resize Right Frame
				HFastResizerYUY2(RightFramep,workp+Rp,bht,bwd,rht,rwd,
									rpitch,wpitch,false);
			}
			if(lvi.IsYV12())
			{
				HFastResizerYV12(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
									lpitch,wpitch,false);
							// resize Right Frame
				HFastResizerYV12(RightFramep,workp+Rp,bht,bwd,rht,rwd,
									rpitch,wpitch,false);
				const unsigned char *rpU =RightFrame->GetReadPtr(PLANAR_U);
				unsigned char *wpU = work->GetWritePtr(PLANAR_U);
				const unsigned char *lpU =LeftFrame->GetReadPtr(PLANAR_U);
				unsigned char *wpV = work->GetWritePtr(PLANAR_V);
				const unsigned char *rpV = RightFrame->GetReadPtr(PLANAR_V);
				const unsigned char *lpV =LeftFrame->GetReadPtr(PLANAR_V);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				const int wpitchUV=work->GetPitch(PLANAR_U);
				int RpUV=RightFramex/2+(RightFramey/2)*rpitchUV;
				int LpUV= LeftFramex/2+(LeftFramey/2)*lpitchUV;

				HFastResizerYV12(lpU,wpU+LpUV,bhtUV,bwdUV,lht/2,lwd/2,
									lpitchUV,wpitchUV,false);
							// resize Right Frame
				HFastResizerYV12(rpU,wpU+RpUV,bhtUV,bwdUV,rht/2,rwd/2,
									rpitchUV,wpitchUV,false);
				HFastResizerYV12(lpV,wpV+LpUV,bhtUV,bwdUV,lht/2,lwd/2,
									lpitchUV,wpitchUV,false);
							// resize Right Frame
				HFastResizerYV12(rpV,wpV+RpUV,bhtUV,bwdUV,rht/2,rwd/2,
									rpitchUV,wpitchUV,false);
			}
						// sides blackening in a taper for depth effect
			if( *dir=='r' || *dir=='R')
			{
				for(int w=0;w<rwd;w++)
					for(int h=0; h<(rwd-w)*rhtaper/rwd;h++)
						for(int k=0; k<kby; k++)
						{
							*(workp+Rp+h*wpitch+kb*(w)+k)=0;
							*(workp+Rp+(bht-1-h)*wpitch+kb*(w)+k)=0;
						}
				for( w=0;w<lwd;w++)
					for(int h=0; h<(lwd-w)*lhtaper/lwd;h++)
						for(int k=0; k<kby; k++)
						{
							*(workp+Lp+h*wpitch+kb*(lwd-1-w)+k)=0;
							*(workp+Lp+(bht-1-h)*wpitch+kb*(lwd-1-w)+k)=0;
						}
			}
			if( *dir=='l' || *dir=='L')
			{
				for(int w=0;w<rwd;w++)
					for(int h=0; h<(rwd-w)*rhtaper/rwd;h++)
						for(int k=0; k<kby; k++)
						{
							*(workp+Rp+h*wpitch+kb*(rwd-1-w)+k)=0;
							*(workp+Rp+(bht-1-h)*wpitch+kb*(rwd-1-w)+k)=0;
						}
				for( w=0;w<lwd;w++)
					for(int h=0; h<(lwd-w)*lhtaper/lwd;h++)
						for(int k=0; k<kby; k++)
						{
							*(workp+Lp+h*wpitch+kb*(w)+k)=0;
							*(workp+Lp+(bht-1-h)*wpitch+kb*(w)+k)=0;
						}
			}
			return work;
				
		}
		if(bwd==rwd && bwd==lwd)
		{
			if(lvi.IsRGB24())
			{
					// no horizontal resizing required
				VFastResizerRGB24(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
								lpitch,wpitch,false);
				VFastResizerRGB24(RightFramep,workp+Rp,bht,bwd,rht,rwd,
								lpitch,wpitch,false);
			}
			if(lvi.IsRGB32())
			{
					// no horizontal resizing required
				VFastResizerRGB32(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
								lpitch,wpitch,false);
				VFastResizerRGB32(RightFramep,workp+Rp,bht,bwd,rht,rwd,
								lpitch,wpitch,false);
			}
			if(lvi.IsYUY2())
			{
					// no horizontal resizing required
				VFastResizerYUY2(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
								lpitch,wpitch,false);
				VFastResizerYUY2(RightFramep,workp+Rp,bht,bwd,rht,rwd,
								lpitch,wpitch,false);
			}
			if(lvi.IsYV12())
			{
					// no horizontal resizing required
				VFastResizerYV12(LeftFramep,workp+Lp,bht,bwd,lht,lwd,
								lpitch,wpitch,false);
				VFastResizerYV12(RightFramep,workp+Rp,bht,bwd,rht,rwd,
								lpitch,wpitch,false);
				const unsigned char *rpU =RightFrame->GetReadPtr(PLANAR_U);
				unsigned char *wpU = work->GetWritePtr(PLANAR_U);
				const unsigned char *lpU =LeftFrame->GetReadPtr(PLANAR_U);
				unsigned char *wpV = work->GetWritePtr(PLANAR_V);
				const unsigned char *rpV = RightFrame->GetReadPtr(PLANAR_V);
				const unsigned char *lpV =LeftFrame->GetReadPtr(PLANAR_V);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				const int wpitchUV=work->GetPitch(PLANAR_U);
				int RpUV=RightFramex/2+(RightFramey/2)*rpitchUV;
				int LpUV= LeftFramex/2+(LeftFramey/2)*lpitchUV;

				VFastResizerYV12(lpU,wpU+LpUV,bhtUV,bwdUV,lht/2,lwd/2,
									lpitchUV,wpitchUV,false);
				VFastResizerYV12(rpU,wpU+RpUV,bhtUV,bwdUV,rht/2,rwd/2,
									rpitchUV,wpitchUV,false);
				VFastResizerYV12(lpV,wpV+LpUV,bhtUV,bwdUV,lht/2,lwd/2,
									lpitchUV,wpitchUV,false);
				VFastResizerYV12(rpV,wpV+RpUV,bhtUV,bwdUV,rht/2,rwd/2,
									rpitchUV,wpitchUV,false);
			}
			if((( *dir=='u' || *dir=='U')&& (vi.IsRGB24() || vi.IsRGB32()))
				|| (( *dir=='d' || *dir=='D')&& (vi.IsYUY2() || vi.IsYV12())))
				
			{
				for(int h=0;h<rht;h++)
					for(int w=0; w<(rht-h)*rwtaper/rht;w++)
						for(int k=0;k<kby;k++)
						{
							*(workp+Rp+h*wpitch+kb*w+k)=0;
							*(workp+Rp+h*wpitch+kb*(bwd-1-w)+k)=0;
						}
				for( h=0;h<lht;h++)
					for(int w=0; w<(lht-h)*lwtaper/lht;w++)
						for(int k=0;k<kby;k++)
						{
							*(workp+Lp+(lht-1-h)*wpitch+kb*w+k)=0;
							*(workp+Lp+(lht-1-h)*wpitch+kb*(bwd-1-w)+k)=0;
						}
			}
			if((( *dir=='d' || *dir=='D')&& (vi.IsRGB24() || vi.IsRGB32()))
				|| (( *dir=='u' || *dir=='U')&& (vi.IsYUY2() || vi.IsYV12())))
				
			{
				for(int h=0;h<rht;h++)
					for(int w=0; w<(rht-h)*rwtaper/rht;w++)
						for(int k=0;k<kby;k++)
						{
							*(workp+Rp+(rht-1-h)*wpitch+kb*w+k)=0;
							*(workp+Rp+(rht-1-h)*wpitch+kb*(bwd-1-w)+k)=0;
						}
				for( h=0;h<lht;h++)
					for(int w=0; w<(lht-h)*lwtaper/lht;w++)
						for(int k=0;k<kby;k++)
						{
							*(workp+Lp+(h)*wpitch+kb*w+k)=0;
							*(workp+Lp+(h)*wpitch+kb*(bwd-1-w)+k)=0;
						}
			}

			return work;
		}
	}


/***************************************************************/
// This is the function that created the filter, when the filter has been called.

AVSValue __cdecl Create_TransSprite(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
			
	return new TransSprite(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("left"),	// Transition direction Left, Right,up, down
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransSprite", "cci[dir]s", Create_TransSprite, 0);
				
    return "TransSprite Transition Mask Plugin";
}
*/
