/******************************************************************************
TransSwing filter plugin for Avisynth by V.C.Mohan
4 or 2 or one segments of  LeftClip  unmasks a stationary RightClip clip by swinging out doors or
4 or 2 or one segments of  Right Clip masks  the Left clip by swinging in.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
Modified on 24-9-04
*************************************************************************************************/  
/*#include "windows.h"
#include "avisynth.h"
#include "FastDiscoRotate.cpp"
*/
class TransSwing : public GenericVideoFilter {
		PClip RightClip;
		const bool out;
		const int ndoors;
		const int corner;
		int overlap;
		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransSwing(PClip _child, PClip _RightClip,int _overlap, 
				const bool _out,const int _ndoors,const int _corner,  
				IScriptEnvironment* env) ;	
			
 	~TransSwing();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
bool __stdcall GetParity(int n);  

};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/

//Here is the acutal constructor code used

TransSwing::TransSwing(PClip _child, PClip _RightClip,int _overlap, 
						  const bool _out, const int _ndoors,const int _corner,
						 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),overlap(_overlap), out(_out), ndoors(_ndoors), corner(_corner) 
			
{
char * Tname="TransSwing";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransSwing:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransSwing: This color format is not yet supported ");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("TransSwing: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransSwing: Height/width of frame are not multiple of 4");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransSwing::~TransSwing() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
/***************************************************************/	
void TransSwing::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransSwing::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//---------------------------------------------------------------------------------	
		PVideoFrame __stdcall TransSwing::GetFrame(int en, IScriptEnvironment* env)
		{
			if (en < video_fade_start)
				return child->GetFrame(en, env);
			if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
			int n=en-video_fade_start;
		
			PVideoFrame LeftFrame = child->GetFrame(en, env);
			PVideoFrame RightFrame = RightClip->GetFrame(n, env);
			PVideoFrame door, base;
			const VideoInfo& lvi = child->GetVideoInfo();
			const VideoInfo& rvi= RightClip->GetVideoInfo();
		
			const	int bwd = lvi.width;
			const	int bht = LeftFrame->GetHeight();
			const	int lpitch = LeftFrame->GetPitch();	
			const	int rpitch = RightFrame->GetPitch();		
			int angle=((n+1)*90)/(overlap+1);
			if(!out)
			{
				base=LeftFrame;
				door=RightFrame;			
				angle=90-((n+1)*90)/(overlap+1);
				
				
			}
			else
			{
				
				door=LeftFrame;
				base= RightFrame;
				angle=((n+1)*90)/(overlap+1);
				
			}
				env->MakeWritable(&base);
				unsigned char*	basep=base->GetWritePtr();
				const unsigned char*doorp= door->GetReadPtr();
				const int bpitch=base->GetPitch();
				const int dpitch=door->GetPitch();
			
			
			int rad=sqrt(bwd*bwd+bht*bht)/2;
			
			
/**
bool FastDiscoRotateRGB32(const unsigned char *srcp,unsigned char*dstp,
					  int radius,int srcx,int srcy,int srch,int srcw,
					  int dstx,int dsty,int dsth,int dstw,int spitch, int dpitch,int degree);
bool FastDiscoRotateRGB24(const unsigned char *srcp,unsigned char*dstp,
					  int radius,int srcx,int srcy,int srch,int srcw,
					  int dstx,int dsty,int dsth,int dstw,int spitch, int dpitch,int degree);
**/
	
			if(vi.IsRGB24())
			{
				if(ndoors==4)
				{
					if(corner==1)
					{

						FastDiscoRotateRGB24(doorp,basep,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+3*bwd/2,basep+3*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2+3*bwd/2,
								basep+bpitch*bht/2+3*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==2)
					{
						FastDiscoRotateRGB24(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+3*bwd/2,basep+3*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2+3*bwd/2,
								basep+bpitch*bht/2+3*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==3)
					{
						FastDiscoRotateRGB24(doorp,basep,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB24(doorp+3*bwd/2,basep+3*bwd/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2+3*bwd/2,
								basep+bpitch*bht/2+3*bwd/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateRGB24(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB24(doorp+3*bwd/2,basep+3*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2+3*bwd/2,
								basep+bpitch*bht/2+3*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB24(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
				}
				if(ndoors==2)
				{
					if(corner==1)
					{
						FastDiscoRotateRGB24(doorp,basep,
									2*rad-2,0,0,bht/2,bwd,
									0,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,bwd-1,bht/2-1,bht/2,bwd,
									bwd-1,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==2)
					{
						FastDiscoRotateRGB24(doorp,basep,
									2*rad-2,bwd-1,0,bht/2,bwd,
									bwd-1,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,0,bht/2-1,bht/2,bwd,
									0,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==3)
					{
						FastDiscoRotateRGB24(doorp,basep,
									2*rad-2,0,0,bht,bwd/2,
									0,0,bht,bwd/2,dpitch, bpitch,-angle);
						FastDiscoRotateRGB24(doorp+3*bwd/2,basep+3*bwd/2,
									2*rad-2,bwd/2-1,bht-1,bht,bwd/2,
									bwd/2-1,bht-1,bht,bwd/2,dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateRGB24(doorp+3*bwd/2,basep+3*bwd/2,
									2*rad-2,bwd/2-1,0,bht,bwd/2,
									bwd/2-1,0,bht,bwd/2,dpitch, bpitch,angle);
						FastDiscoRotateRGB24(doorp,basep,
									2*rad-2,0,bht-1,bht,bwd/2,
									0,bht-1,bht,bwd/2,dpitch, bpitch,angle);
					}
				}
				if(ndoors==1)
				{
					if(corner==1)
						FastDiscoRotateRGB24(doorp,basep,
									2*rad,0,0,bht,bwd,
									0,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==2)
						FastDiscoRotateRGB24(doorp,basep,
									2*rad,bwd-1,0,bht,bwd,
									bwd-1,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==3)
						FastDiscoRotateRGB24(doorp,basep,
									2*rad,bwd-1,bht-1,bht,bwd,
									bwd-1,bht-1,bht,bwd,dpitch, bpitch,-angle);
					if(corner==4)
						FastDiscoRotateRGB24(doorp,basep,
									2*rad,0,bht-1,bht,bwd,
									0,bht-1,bht,bwd,dpitch, bpitch,-angle);
				}

			}
		

//***************************************************************************************
			
			if(lvi.IsRGB32())
			{
				if(ndoors==4)
				{
					if(corner==1)
					{

						FastDiscoRotateRGB32(doorp,basep,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+4*bwd/2,basep+4*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2+4*bwd/2,
								basep+bpitch*bht/2+4*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==2)
					{
						FastDiscoRotateRGB32(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+4*bwd/2,basep+4*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2+4*bwd/2,
								basep+bpitch*bht/2+4*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==3)
					{
						FastDiscoRotateRGB32(doorp,basep,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB32(doorp+4*bwd/2,basep+4*bwd/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2+4*bwd/2,
								basep+bpitch*bht/2+4*bwd/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateRGB32(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB32(doorp+4*bwd/2,basep+4*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2+4*bwd/2,
								basep+bpitch*bht/2+4*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateRGB32(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
				}
				if(ndoors==2)
				{
					if(corner==1)
					{
						FastDiscoRotateRGB32(doorp,basep,
									2*rad-2,0,0,bht/2,bwd,
									0,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,bwd-1,bht/2-1,bht/2,bwd,
									bwd-1,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==2)
					{
						FastDiscoRotateRGB32(doorp,basep,
									2*rad-2,bwd-1,0,bht/2,bwd,
									bwd-1,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,0,bht/2-1,bht/2,bwd,
									0,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==3)
					{
						FastDiscoRotateRGB32(doorp,basep,
									2*rad-2,0,0,bht,bwd/2,
									0,0,bht,bwd/2,dpitch, bpitch,-angle);
						FastDiscoRotateRGB32(doorp+4*bwd/2,basep+4*bwd/2,
									2*rad-2,bwd/2-1,bht-1,bht,bwd/2,
									bwd/2-1,bht-1,bht,bwd/2,dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateRGB32(doorp+4*bwd/2,basep+4*bwd/2,
									2*rad-2,bwd/2-1,0,bht,bwd/2,
									bwd/2-1,0,bht,bwd/2,dpitch, bpitch,angle);
						FastDiscoRotateRGB32(doorp,basep,
									2*rad-2,0,bht-1,bht,bwd/2,
									0,bht-1,bht,bwd/2,dpitch, bpitch,angle);
					}
				}

				if(ndoors==1)
				{
					if(corner==1)
						FastDiscoRotateRGB32(doorp,basep,
									2*rad,0,0,bht,bwd,
									0,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==2)
						FastDiscoRotateRGB32(doorp,basep,
									2*rad,bwd-1,0,bht,bwd,
									bwd-1,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==3)
						FastDiscoRotateRGB32(doorp,basep,
									2*rad,bwd-1,bht-1,bht,bwd,
									bwd-1,bht-1,bht,bwd,dpitch, bpitch,-angle);
					if(corner==4)
						FastDiscoRotateRGB32(doorp,basep,
									2*rad,0,bht-1,bht,bwd,
									0,bht-1,bht,bwd,dpitch, bpitch,-angle);
				}
			}

			if(lvi.IsYUY2())
			{
				if(ndoors==4)
				{

				if(corner==1)
					{

						FastDiscoRotateYUY2(doorp,basep,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+2*bwd/2,basep+2*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2+2*bwd/2,
								basep+bpitch*bht/2+2*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYUY2(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+2*bwd/2,basep+2*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2+2*bwd/2,
								basep+bpitch*bht/2+2*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYUY2(doorp,basep,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYUY2(doorp+2*bwd/2,basep+2*bwd/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2+2*bwd/2,
								basep+bpitch*bht/2+2*bwd/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYUY2(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYUY2(doorp+2*bwd/2,basep+2*bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2+2*bwd/2,
								basep+bpitch*bht/2+2*bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYUY2(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
				}
				if(ndoors==2)
				{
					if(corner==1)
					{
						FastDiscoRotateYUY2(doorp,basep,
									2*rad-2,0,0,bht/2,bwd,
									0,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,bwd-1,bht/2-1,bht/2,bwd,
									bwd-1,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYUY2(doorp,basep,
									2*rad-2,bwd-1,0,bht/2,bwd,
									bwd-1,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,0,bht/2-1,bht/2,bwd,
									0,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYUY2(doorp,basep,
									2*rad-2,0,0,bht,bwd/2,
									0,0,bht,bwd/2,dpitch, bpitch,-angle);
						FastDiscoRotateYUY2(doorp+2*bwd/2,basep+2*bwd/2,
									2*rad-2,bwd/2-1,bht-1,bht,bwd/2,
									bwd/2-1,bht-1,bht,bwd/2,dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYUY2(doorp+2*bwd/2,basep+2*bwd/2,
									2*rad-2,bwd/2-1,0,bht,bwd/2,
									bwd/2-1,0,bht,bwd/2,dpitch, bpitch,angle);
						FastDiscoRotateYUY2(doorp,basep,
									2*rad-2,0,bht-1,bht,bwd/2,
									0,bht-1,bht,bwd/2,dpitch, bpitch,angle);
					}
				}

				if(ndoors==1)
				{
					if(corner==1)
						FastDiscoRotateYUY2(doorp,basep,
									2*rad,0,0,bht,bwd,
									0,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==2)
						FastDiscoRotateYUY2(doorp,basep,
									2*rad,bwd-1,0,bht,bwd,
									bwd-1,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==3)
						FastDiscoRotateYUY2(doorp,basep,
									2*rad,bwd-1,bht-1,bht,bwd,
									bwd-1,bht-1,bht,bwd,dpitch, bpitch,-angle);
					if(corner==4)
						FastDiscoRotateYUY2(doorp,basep,
									2*rad,0,bht-1,bht,bwd,
									0,bht-1,bht,bwd,dpitch, bpitch,-angle);
				}
			}

			if(vi.IsYV12())
			{
				const unsigned char *doorpU = door->GetReadPtr(PLANAR_U);
				unsigned char *basepU = base->GetWritePtr(PLANAR_U);
				const unsigned char *doorpV = door->GetReadPtr(PLANAR_V);
				unsigned char *basepV = base->GetWritePtr(PLANAR_V);
				const int bwdUV=base->GetRowSize(PLANAR_U);
				const int bhtUV=base->GetHeight(PLANAR_U);
				const int bpitchUV=base->GetPitch(PLANAR_U);
				const int dpitchUV=door->GetPitch(PLANAR_U);
				

				if(ndoors==4)
				{

					if(corner==1)
					{

						FastDiscoRotateYV12(doorp,basep,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+bwd/2,basep+bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2+bwd/2,
								basep+bpitch*bht/2+bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYV12(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+bwd/2,basep+bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2+bwd/2,
								basep+bpitch*bht/2+bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYV12(doorp,basep,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYV12(doorp+bwd/2,basep+bwd/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2+bwd/2,
								basep+bpitch*bht/2+bwd/2,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYV12(doorp,basep,
								rad,0,bht/2-1,bht/2,bwd/2,
								0,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYV12(doorp+bwd/2,basep+bwd/2,
								rad,bwd/2-1,bht/2-1,bht/2,bwd/2,
								bwd/2-1,bht/2-1,bht/2,bwd/2,
								dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2+bwd/2,
								basep+bpitch*bht/2+bwd/2,
								rad,bwd/2-1,0,bht/2,bwd/2,
								bwd/2-1,0,bht/2,bwd/2,
								dpitch, bpitch,angle);
						FastDiscoRotateYV12(doorp+dpitch*bht/2,
								basep+bpitch*bht/2,
								rad,0,0,bht/2,bwd/2,
								0,0,bht/2,bwd/2,
								dpitch, bpitch,-angle);
					}
				}
				if(ndoors==2)
				{
					if(corner==1)
					{
						FastDiscoRotateYV12(doorp,basep,
									2*rad-2,0,0,bht/2,bwd,
									0,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,bwd-1,bht/2-1,bht/2,bwd,
									bwd-1,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYV12(doorp,basep,
									2*rad-2,bwd-1,0,bht/2,bwd,
									bwd-1,0,bht/2,bwd,dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+bpitch*bht/2,basep+bpitch*bht/2,
									2*rad-2,0,bht/2-1,bht/2,bwd,
									0,bht/2-1,bht/2,bwd,dpitch, bpitch,-angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYV12(doorp,basep,
									2*rad-2,0,0,bht,bwd/2,
									0,0,bht,bwd/2,dpitch, bpitch,-angle);
						FastDiscoRotateYV12(doorp+bwd/2,basep+bwd/2,
									2*rad-2,bwd/2-1,bht-1,bht,bwd/2,
									bwd/2-1,bht-1,bht,bwd/2,dpitch, bpitch,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYV12(doorp+bwd/2,basep+bwd/2,
									2*rad-2,bwd/2-1,0,bht,bwd/2,
									bwd/2-1,0,bht,bwd/2,dpitch, bpitch,angle);
						FastDiscoRotateYV12(doorp,basep,
									2*rad-2,0,bht-1,bht,bwd/2,
									0,bht-1,bht,bwd/2,dpitch, bpitch,angle);
					}
				}

				if(ndoors==1)
				{
					if(corner==1)
						FastDiscoRotateYV12(doorp,basep,
									2*rad,0,0,bht,bwd,
									0,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==2)
						FastDiscoRotateYV12(doorp,basep,
									2*rad,bwd-1,0,bht,bwd,
									bwd-1,0,bht,bwd,dpitch, bpitch,-angle);
					if(corner==3)
						FastDiscoRotateYV12(doorp,basep,
									2*rad,bwd-1,bht-1,bht,bwd,
									bwd-1,bht-1,bht,bwd,dpitch, bpitch,-angle);
					if(corner==4)
						FastDiscoRotateYV12(doorp,basep,
									2*rad,0,bht-1,bht,bwd,
									0,bht-1,bht,bwd,dpitch, bpitch,-angle);
				}

				if(ndoors==4)
				{

					if(corner==1)
					{

						FastDiscoRotateYV12(doorpU,basepU,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+bwdUV/2,basepU+bwdUV/2,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2+bwdUV/2,
								basepU+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2,
								basepU+bpitchUV*bhtUV/2,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYV12(doorpU,basepU,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+bwdUV/2,basepU+bwdUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2+bwdUV/2,
								basepU+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2,
								basepU+bpitchUV*bhtUV/2,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYV12(doorpU,basepU,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpU+bwdUV/2,basepU+bwdUV/2,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2+bwdUV/2,
								basepU+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2,
								basepU+bpitchUV*bhtUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYV12(doorpU,basepU,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpU+bwdUV/2,basepU+bwdUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2+bwdUV/2,
								basepU+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpU+dpitchUV*bhtUV/2,
								basepU+bpitchUV*bhtUV/2,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
					}
				}
				if(ndoors==2)
				{
					if(corner==1)
					{
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2-1,0,0,bhtUV/2,bwdUV,
									0,0,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+bpitchUV*bhtUV/2,basepU+bpitchUV*bhtUV/2,
									2*rad/2-1,bwdUV-1,bhtUV/2-1,bhtUV/2,bwdUV,
									bwdUV-1,bhtUV/2-1,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2-1,bwdUV-1,0,bhtUV/2,bwdUV,
									bwdUV-1,0,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+bpitchUV*bhtUV/2,basepU+bpitchUV*bhtUV/2,
									2*rad/2-1,0,bhtUV/2-1,bhtUV/2,bwdUV,
									0,bhtUV/2-1,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2-2,0,0,bhtUV,bwdUV/2,
									0,0,bhtUV,bwdUV/2,dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpU+bwdUV/2,basepU+bwdUV/2,
									2*rad/2-1,bwdUV/2-1,bhtUV-1,bhtUV,bwdUV/2,
									bwdUV/2-1,bhtUV-1,bhtUV,bwdUV/2,dpitchUV, bpitchUV,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYV12(doorpU+bwdUV/2,basepU+bwdUV/2,
									2*rad/2-1,bwdUV/2-1,0,bhtUV,bwdUV/2,
									bwdUV/2-1,0,bhtUV,bwdUV/2,dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2-1,0,bhtUV-1,bhtUV,bwdUV/2,
									0,bhtUV-1,bhtUV,bwdUV/2,dpitchUV, bpitchUV,angle);
					}
				}

				if(ndoors==1)
				{
					if(corner==1)
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2,0,0,bhtUV,bwdUV,
									0,0,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
					if(corner==2)
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2,bwdUV-1,0,bhtUV,bwdUV,
									bwdUV-1,0,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
					if(corner==3)
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2,bwdUV-1,bhtUV-1,bhtUV,bwdUV,
									bwdUV-1,bhtUV-1,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
					if(corner==4)
						FastDiscoRotateYV12(doorpU,basepU,
									2*rad/2,0,bhtUV-1,bhtUV,bwdUV,
									0,bhtUV-1,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
				}

				if(ndoors==4)
				{

					if(corner==1)
					{

						FastDiscoRotateYV12(doorpV,basepV,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+bwdUV/2,basepV+bwdUV/2,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2+bwdUV/2,
								basepV+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2,
								basepV+bpitchUV*bhtUV/2,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYV12(doorpV,basepV,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+bwdUV/2,basepV+bwdUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2+bwdUV/2,
								basepV+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2,
								basepV+bpitchUV*bhtUV/2,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYV12(doorpV,basepV,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpV+bwdUV/2,basepV+bwdUV/2,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2+bwdUV/2,
								basepV+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2,
								basepV+bpitchUV*bhtUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYV12(doorpV,basepV,
								rad/2,0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								0,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpV+bwdUV/2,basepV+bwdUV/2,
								rad/2,bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								bwdUV/2-1,bhtUV/2-1,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2+bwdUV/2,
								basepV+bpitchUV*bhtUV/2+bwdUV/2,
								rad/2,bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								bwdUV/2-1,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpV+dpitchUV*bhtUV/2,
								basepV+bpitchUV*bhtUV/2,
								rad/2,0,0,bhtUV/2,bwdUV/2,
								0,0,bhtUV/2,bwdUV/2,
								dpitchUV, bpitchUV,-angle);
					}
				}
				if(ndoors==2)
				{
					if(corner==1)
					{
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2-1,0,0,bhtUV/2,bwdUV,
									0,0,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+bpitchUV*bhtUV/2,basepV+bpitchUV*bhtUV/2,
									2*rad/2-1,bwdUV-1,bhtUV/2-1,bhtUV/2,bwdUV,
									bwdUV-1,bhtUV/2-1,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
					}
					if(corner==2)
					{
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2-1,bwdUV-1,0,bhtUV/2,bwdUV,
									bwdUV-1,0,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+bpitchUV*bhtUV/2,basepV+bpitchUV*bhtUV/2,
									2*rad/2-1,0,bhtUV/2-1,bhtUV/2,bwdUV,
									0,bhtUV/2-1,bhtUV/2,bwdUV,dpitchUV, bpitchUV,-angle);
					}
					if(corner==3)
					{
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2-1,0,0,bhtUV,bwdUV/2,
									0,0,bhtUV,bwdUV/2,dpitchUV, bpitchUV,-angle);
						FastDiscoRotateYV12(doorpV+bwdUV/2,basepV+bwdUV/2,
									2*rad/2-1,bwdUV/2-1,bhtUV-1,bhtUV,bwdUV/2,
									bwdUV/2-1,bhtUV-1,bhtUV,bwdUV/2,dpitchUV, bpitchUV,-angle);
					}
					if(corner==4)
					{
						FastDiscoRotateYV12(doorpV+bwdUV/2,basepV+bwdUV/2,
									2*rad/2-1,bwdUV/2-1,0,bhtUV,bwdUV/2,
									bwdUV/2-1,0,bhtUV,bwdUV/2,dpitchUV, bpitchUV,angle);
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2-1,0,bhtUV-1,bhtUV,bwdUV/2,
									0,bhtUV-1,bhtUV,bwdUV/2,dpitchUV, bpitchUV,angle);
					}
				}

				if(ndoors==1)
				{
					if(corner==1)
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2,0,0,bhtUV,bwdUV,
									0,0,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
					if(corner==2)
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2,bwdUV-1,0,bhtUV,bwdUV,
									bwdUV-1,0,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
					if(corner==3)
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2,bwdUV-1,bhtUV-1,bhtUV,bwdUV,
									bwdUV-1,bhtUV-1,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
					if(corner==4)
						FastDiscoRotateYV12(doorpV,basepV,
									2*rad/2,0,bhtUV-1,bhtUV,bwdUV,
									0,bhtUV-1,bhtUV,bwdUV,dpitchUV, bpitchUV,-angle);
				}


			}
				



			
			return base;
			
		}



/***************************************************************/
// This is the function that created the filter, when the filter has been called.

AVSValue __cdecl Create_TransSwing(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	
	if(!(args[4].AsInt(2)==1) && !(args[4].AsInt(2)==2) && !(args[4].AsInt(2)==4))
		env->ThrowError("TransSwing: number of doors can be 1, 2 or4 only");
	if(args[5].AsInt(1)<1 || args[5].AsInt(1)>4)
		env->ThrowError("TransSwing: corner can be 1, 2, 3 or 4 only");

			
	return new TransSwing(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsBool(true),	// true: swing out doors, false swing in Doors
						args[4].AsInt(2),	// number of doors 1 or 2 or 4
						args[5].AsInt(1),	// corner number 1, 2, 3 or 4 from which the door(s) swings
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransSwing", "cci[out]b[ndoors]i[corner]i", Create_TransSwing, 0);
				
    return "TransSwing Transition  Plugin";
}
*/
