
/**************************************************************************************
Trans Swirl . Left frame swirls, then RightFrame swirls over,
 then Right Swirl windows  out
 Modified on 24-9-04
**************************************************************************************/
class TransSwirl : public GenericVideoFilter {
		PClip RightClip;
		const char * dir;
		const int thick;
		 int radius;
		 int sx;
		 int sy;
		 int ex;
		 int ey;

		 int overlap;
		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransSwirl(PClip _child,PClip _Right,
				int _overlap,const char * _dir,const int _thick, 
				
				IScriptEnvironment* env) ;	
			
 	~TransSwirl();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/

//Here is the acutal constructor code used

TransSwirl::TransSwirl(PClip _child, PClip _Right,
					   int _overlap,const char* _dir,const int _thick,   
						 IScriptEnvironment* env) :
	GenericVideoFilter(_child),RightClip (_Right),
				overlap(_overlap), dir(_dir), thick(_thick) 
			
{
		const VideoInfo& rvi= RightClip->GetVideoInfo();
	char * Tname="TransSwirl";
	
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransSwirl: This color format is not yet supported ");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransSwirl: Height/width of frame are not multiple of 4");
if(vi.pixel_type!=rvi.pixel_type)
	env->ThrowError("TransSwirl: The two clips have different color formats");
if(vi.width!=rvi.width || vi.height != rvi.height)
	env->ThrowError("TransSwirl: The two clips have different dimensions");
sx=ex=vi.width/2;
sy=ey=vi.height/2;
radius=sqrt(vi.height*vi.height+vi.width*vi.width)/2;
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransSwirl::~TransSwirl() {
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
void TransSwirl::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransSwirl::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//---------------------------------------------------------------------------------	

	PVideoFrame __stdcall TransSwirl::GetFrame(int en, IScriptEnvironment* env)
		{
			if (en < video_fade_start)
				return child->GetFrame(en, env);
			if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
			int n=en-video_fade_start;
		
			PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n,env);
		const	int bwd = vi.width; 
        const	int bht = LeftFrame->GetHeight();	// LeftClip->GetHeight()?
		const	int lpitch = LeftFrame->GetPitch();
		const	int rpitch = RightFrame->GetPitch();
		int nframes= overlap;
		unsigned char* dstp, *dstpU, *dstpV;
		const unsigned char* srcp,*srcpU,*srcpV;
		if(3*n<2*nframes)
		{
			env->MakeWritable(&LeftFrame);
			dstp= LeftFrame->GetWritePtr();
			srcp=RightFrame->GetReadPtr();
			if(vi.IsYV12())
			{
				
				dstpU = LeftFrame->GetWritePtr(PLANAR_U);
				dstpV = LeftFrame->GetWritePtr(PLANAR_V);
				srcpU = RightFrame->GetReadPtr(PLANAR_U);
				srcpV = RightFrame->GetReadPtr(PLANAR_V);

			}
		}
		else
		{
			env->MakeWritable(&RightFrame);
			dstp = RightFrame->GetWritePtr();
			srcp=LeftFrame->GetReadPtr();
			if(vi.IsYV12())
			{
				
				srcpU = LeftFrame->GetReadPtr(PLANAR_U);
				srcpV = LeftFrame->GetReadPtr(PLANAR_V);
				dstpU = RightFrame->GetWritePtr(PLANAR_U);
				dstpV = RightFrame->GetWritePtr(PLANAR_V);

			}
		}
		
		int deg;
		int radmax=3*n<(nframes)?(3*n*radius)/nframes:radius;
		radmax=radmax & 0xfffffffc;
		int rad=3*n<nframes? thick:((3*n-nframes)*radius)/nframes;
		if(3*n>=2*nframes)
			rad=((-2*nframes+3*n)*radius*3)/nframes;
//			deg=(2*nframes-3*n);
		rad= rad & 0xfffffffc;
		int one=(*dir=='a' || *dir=='A')?-1:1;
		
		if(vi.IsRGB24())
		{
				
			if(3*n<2*nframes)
			{
					// produce left frame swirls on leftframe increasing upto 1/3 nframes
					// to full radius. Therafter till 2/3 nframes increase RightFrame
					// swirls to full radius.
				for (int i=radmax; i>rad ; i-=thick)
					FastORingRotateRGB24(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								lpitch, lpitch,one*(4*n+i));
				for ( i=rad; i>thick ; i-=thick)
							FastORingRotateRGB24(srcp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, lpitch,-one*(4*n+i));
			}
			else
						// decrease rightframe swirls from full to zero
				for (int i=radmax; i>rad ; i-=thick)
							FastORingRotateRGB24(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, rpitch,-one*(4*n+i));

		}
		

		if(vi.IsRGB32())
		{
			if(3*n<2*nframes)
			{
			
				for (int i=radmax; i>rad ; i-=thick)
					FastORingRotateRGB32(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								lpitch, lpitch,one*(4*n+i));
				for ( i=rad; i>thick ; i-=thick)
							FastORingRotateRGB32(srcp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, lpitch,one*(4*n+i));
			}
			else
				for (int i=radmax; i>rad ; i-=thick)
							FastORingRotateRGB32(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, rpitch,one*(4*n+i));


		}
		if(vi.IsYUY2())
		{
			if(3*n<2*nframes)
			{
			
				for (int i=radmax; i>rad ; i-=thick)
					FastORingRotateYUY2(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								lpitch, lpitch,one*(4*n+i));
				for ( i=rad; i>thick ; i-=thick)
							FastORingRotateYUY2(srcp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, lpitch,one*(4*n+i));
			}
			else
				for (int i=radmax; i>rad ; i-=thick)
							FastORingRotateYUY2(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, rpitch,one*(4*n+i));

		}

		if(vi.IsYV12())
		{
			
			const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
			const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
			const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
			const int bhtUV = LeftFrame->GetHeight(PLANAR_U);
			if(3*n<2*nframes)
			{
			
				for (int i=radmax; i>rad ; i-=thick)
					FastORingRotateYV12(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								lpitch, lpitch,one*(4*n+i));
				for ( i=rad; i>thick ; i-=thick)
							FastORingRotateYV12(srcp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, lpitch,one*(4*n+i));
						// U and V planes
				for (i=radmax/2; i>rad/2 ; i-=thick/2)
				{
					FastORingRotateYV12(dstpU,dstpU,
								i,thick/2,sx/2,sy/2,bhtUV,bwdUV,
								sx/2,sy/2,bhtUV,bwdUV,
								lpitchUV, lpitchUV,one*(4*n+i));
					FastORingRotateYV12(dstpV,dstpV,
								i,thick/2,sx/2,sy/2,bhtUV,bwdUV,
								sx/2,sy/2,bhtUV,bwdUV,
								lpitchUV, lpitchUV,one*(4*n+i));
				}
				for ( i=rad/2; i>thick/2 ; i-=thick/2)
				{
					FastORingRotateYV12(srcpU,dstpU,
								i,thick/2,sx/2,sy/2,bhtUV,bwdUV,
								sx/2,sy/2,bhtUV,bwdUV,
								rpitchUV, lpitchUV,one*(4*n+i));
					FastORingRotateYV12(srcpV,dstpV,
								i,thick/2,sx/2,sy/2,bhtUV,bwdUV,
								sx/2,sy/2,bhtUV,bwdUV,
								rpitchUV, lpitchUV,one*(4*n+i));
				}
			}
			else
			{
				for (int i=radmax; i>rad ; i-=thick)
							FastORingRotateYV12(dstp,dstp,
								i,thick,sx,sy,bht,bwd,
								sx,sy,bht,bwd,
								rpitch, rpitch,one*(4*n+i));
				for (i=radmax/2; i>rad/2 ; i-=thick/2)
				{
					FastORingRotateYV12(dstpU,dstpU,
								i,thick/2,sx/2,sy/2,bhtUV,bwdUV,
								sx/2,sy/2,bhtUV,bwdUV,
								rpitchUV, rpitchUV,one*(4*n+i));
					FastORingRotateYV12(dstpV,dstpV,
								i,thick/2,sx/2,sy/2,bhtUV,bwdUV,
								sx/2,sy/2,bhtUV,bwdUV,
								rpitchUV, rpitchUV,one*(4*n+i));
				}
			}

		}
			
		if(3*n<2*nframes)
			return LeftFrame;
		return RightFrame;
	}
	


AVSValue __cdecl Create_TransSwirl(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	if(args[4].AsInt(10)<4 || args[4].AsInt(10)>64)
		env->ThrowError("TransSwirl: Swirl step to be between 4 and 64 only");

			
	return new TransSwirl(args[0].AsClip(),	// Leftclip as LeftClip
						args[1].AsClip(),		// Right Clip
						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("clock"),// Swirling direction clockwise/anticlockwise 						
						args[4].AsInt(10),		// step of swirl. 4(fine) t0 64 (coarse)
						env);
// Calls the constructor with the arguments provied.
}
// The following  is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    
	env->AddFunction("TransSwirl", "cci[dir]s[step]i", Create_TransSwirl, 0);			
    return "Transition Ripple Plugin";
}

*/



