/******************************************************************************
TransVenetianBlinds filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip in the pattern specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also used partly adapted function coding 
from Avisynth and simple sample
Modified on 25-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
*/
class TransVenetianBlinds : public GenericVideoFilter {
		PClip RightClip;		// define the second clip
		const int cslatw;	// slat width
		const char* type;	// type vert/hor/cheqer/diamond
		int slatw;

		int overlap;
		void * abuf;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransVenetianBlinds(PClip _child,PClip _RightClip,
					int _overlap,int _slatw, const char * _type,  
			IScriptEnvironment* env) ;	
			
 	~TransVenetianBlinds();		//destructor
//	copywindow(unsigned char* R,const unsigned char* L, int w, int h, int rp, int lp);
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransVenetianBlinds::TransVenetianBlinds(PClip _child,PClip _RightClip,
										 int _overlap,int _slatw, const char * _type,  IScriptEnvironment* env) :
	GenericVideoFilter(_child),RightClip(_RightClip),
					overlap( _overlap),cslatw(_slatw), type(_type) 
			
{
	char * Tname="TransVenetianBlinds";
	slatw=cslatw & 0xfffffffc;
	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransVenetianBlinds:Clips have differing pixel_types");
	if (vi.width != rvi.width || vi.height != rvi.height)
		env->ThrowError("%s:Clips have differing frame dimensions", Tname);
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransVenetianBlinds: This color format is not yet supported");
	if(!(*type=='V') && !(*type=='H') && !(*type=='D') && !(*type=='v')
		&& !(*type=='h') && !(*type=='d') && !(*type=='c') && !(*type=='C'))
		env->ThrowError("TransVenetianBlinds: Vertical, Horizontal, cheqs and Diamonds only available");
	if(slatw<4 ||slatw>vi.width/4 || slatw>vi.height/4)
		env->ThrowError("TransVenetianBlinds: Slatw must be between 4 and 1/4th width or height");

	
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}

/*************************************************************************/	

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
//************************************************************************
// This is where any actual destructor code used goes
TransVenetianBlinds::~TransVenetianBlinds() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/
void TransVenetianBlinds::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransVenetianBlinds::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//---------------------------------------------------------------------------------	

	PVideoFrame __stdcall TransVenetianBlinds::GetFrame(int en, IScriptEnvironment* env)
		{
			if (en < video_fade_start)
				return child->GetFrame(en, env);
			if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
			int n=en-video_fade_start;
		
			PVideoFrame LeftFrame = child->GetFrame(en, env);
			PVideoFrame RightFrame = RightClip->GetFrame(n, env);
			const	int bwd = vi.width;
			const	int bht = LeftFrame->GetHeight();
			const	int lpitch = LeftFrame->GetPitch();
			const	int rpitch = RightFrame->GetPitch();
			const	int nframes = overlap;
			env->MakeWritable(&LeftFrame);
			unsigned char* lp= LeftFrame->GetWritePtr();
			const unsigned char * rp= RightFrame->GetReadPtr();
			int NWindowh=bht/slatw;
			int NWindoww = bwd/slatw;
			int dwidth=1+(slatw*(n+1))/(nframes+1);
			int dheight=1+0.8*dwidth;
		
			if(vi.IsRGB24())
			{
				if(*type =='H'|| *type =='h')
				{
					for(int i=0; i<NWindowh;i++)
						for(int h=0; h<dwidth-1; h++)												
							for(int w=0;w<bwd;w++)
								for(int k=0;k<3;k++)
									*(lp+(i*slatw+h)*lpitch+3*w+k)=*(rp+(i*slatw+h)*lpitch+3*w+k);
					
				}

				if(*type =='V'|| *type =='v')
				{
					
					for(int i=0; i<NWindoww;i++)
						for(int w=0;w<dwidth-1;w++)
							for(int h=0; h<bht; h++)
								for(int k=0; k<3;k++)
									*(lp+h*lpitch+3*(i*slatw+w)+k)=*(rp+h*lpitch+3*(i*slatw+w)+k);
				
				}

				if(*type=='C' || *type=='c')
				{
					for(int i=0;i<NWindowh+1;i++)
						for(int h=0;h<dwidth-1;h++)
							for(int k=0;k<NWindoww+1;k++)
								for(int w=0;w<dwidth-1;w++)
									if(i*slatw+h<bht && k*slatw+w<bwd)
										for(int p=0;p<3;p++)
											*(lp+(i*slatw+h)*lpitch+3*(k*slatw+w)+p)
											=*(rp+(i*slatw+h)*lpitch+3*(k*slatw+w)+p);
			
				}
			
				if(*type=='D' || *type=='d')
				{
					for(int e=0;e<2;e++)
						for(int i=-e*0.8*slatw;i<bht+0.8*slatw;i+=1.6*slatw)
							for(int k=e*slatw/2;k<bwd;k+=slatw)
								for(int h=0;h<dheight;h++)							
									for(int w=0;w<((dheight-h)*dwidth)/dheight/2;w++)
										for(int p=0; p<3; p++)
										{
											if(i+h>0 && i+h<bht)
											{
												if( k+w>0 && k+w<bwd)											
													*(lp+(i+h)*lpitch+3*(k+w)+p)
														=*(rp+(i+h)*rpitch+3*(k+w)+p);
												if( k-w>0 && k-w<bwd)
													*(lp+(i+h)*lpitch+3*(k-w)+p)
														=*(rp+(i+h)*rpitch+3*(k-w)+p);
											}
											if(i-h>0 && i-h<bht)
											{
												if( k-w>0 && k-w<bwd)											
													*(lp+(i-h)*lpitch+3*(k-w)+p)
														=*(rp+(i-h)*rpitch+3*(k-w)+p);
												if(k+w>=0 && (k+w)<bwd)
													*(lp+(i-h)*lpitch+3*(k+w)+p)
														=*(rp+(i-h)*rpitch+3*(k+w)+p);
											}
									
										}

				}
			}

			
//***************************************************************************************
			
			if(vi.IsRGB32())
			{
				if(*type =='H'|| *type =='h')
				{
					for(int i=0; i<NWindowh;i++)
						for(int h=0; h<dwidth-1; h++)												
							for(int w=0;w<bwd;w++)
								*((int*)(lp+(i*slatw+h)*lpitch+4*w))=*((int*)(rp+(i*slatw+h)*lpitch+4*w));
	
				}

				if(*type =='V'|| *type =='v')
				{
					
					for(int i=0; i<NWindoww;i++)
						for(int w=0;w<dwidth-1;w++)
							for(int h=0; h<bht; h++)							
								*((int*)(lp+h*lpitch+4*(i*slatw+w)))=*((int*)(rp+h*lpitch+4*(i*slatw+w)));
		
				}

				if(*type=='C'|| *type=='c')
				{
					for(int i=0;i<NWindowh+1;i++)
						for(int h=0;h<dwidth-1;h++)
							for(int k=0;k<NWindoww+1;k++)
								for(int w=0;w<dwidth-1;w++)
									if(i*slatw+h<bht && k*slatw+w<bwd)
										*((int *)(lp+(i*slatw+h)*lpitch+4*(k*slatw+w)))
											=*((int *)(rp+(i*slatw+h)*lpitch+4*(k*slatw+w)));
			
				}
				if(*type=='D' || *type=='d')
					for(int e=0;e<2;e++)
						for(int i=-e*0.8*slatw;i<bht+0.8*slatw;i+=1.6*slatw)
							for(int k=e*slatw/2;k<bwd;k+=slatw)
								for(int h=0;h<dheight;h++)							
									for(int w=0;w<((dheight-h)*dwidth)/dheight/2;w++)
									{
										if(i+h>0 && i+h<bht)
										{
											if( k+w>0 && k+w<bwd)
												*((int*)(lp+(i+h)*lpitch+4*(k+w)))
													=*((int*)(rp+(i+h)*rpitch+4*(k+w)));
											if( k-w>0 && k-w<bwd)
										
												*((int*)(lp+(i+h)*lpitch+4*(k-w)))
													=*((int*)(rp+(i+h)*rpitch+4*(k-w)));
										}
										if(i-h>0 && i-h<bht)
										{
											if( k-w>0 && k-w<bwd)
												*((int*)(lp+(i-h)*lpitch+4*(k-w)))
													=*((int*)(rp+(i-h)*rpitch+4*(k-w)));
											if(k+w>=0 && (k+w)<bwd)
												*((int*)(lp+(i-h)*lpitch+4*(k+w)))
													=*((int*)(rp+(i-h)*rpitch+4*(k+w)));
										}
									
									}
	

				
							
			}

			if(vi.IsYUY2())
			{
				if(*type =='H'|| *type =='h')
				{
					for(int i=0; i<NWindowh;i++)
						for(int h=0; h<dwidth-1; h++)												
							for(int w=0;w<bwd;w+=2)
								*((int*)(lp+(i*slatw+h)*lpitch+2*w))=*((int*)(rp+(i*slatw+h)*lpitch+2*w));
	
				}

				if(*type =='V'|| *type =='v')
				{
					
					for(int i=0; i<NWindoww;i++)
						for(int w=0;w<dwidth-1;w+=2)
							for(int h=0; h<bht; h++)							
								*((int*)(lp+h*lpitch+2*(i*slatw+w)))=*((int*)(rp+h*lpitch+2*(i*slatw+w)));

				}

				if(*type=='C'|| *type=='c')
				{
					for(int i=0;i<NWindowh+1;i++)
						for(int h=0;h<dwidth-1;h++)
							for(int k=0;k<NWindoww+1;k++)
								for(int w=0;w<dwidth-1;w++)
									if(i*slatw+h<bht && k*slatw+w<bwd)
										*((int *)(lp+(i*slatw+h)*lpitch+2*(k*slatw+w)))
											=*((int *)(rp+(i*slatw+h)*lpitch+2*(k*slatw+w)));
				
				}
				if(*type=='D' || *type=='d')
					for(int e=0;e<2;e++)
						for(int i=-e*0.8*slatw;i<bht+0.8*slatw;i+=1.6*slatw)
							for(int k=e*slatw/2;k<bwd;k+=slatw)
								for(int h=0;h<dheight;h++)							
									for(int w=0;w<((dheight-h)*dwidth)/dheight/2;w+=2)
									{
										if(i+h>0 && i+h<bht)
										{
											if( k+w>0 && k+w<bwd)
												*((int*)(lp+(i+h)*lpitch+2*(k+w)))
													=*((int*)(rp+(i+h)*rpitch+2*(k+w)));
											if( k-w>0 && k-w<bwd)
										
												*((int*)(lp+(i+h)*lpitch+2*(k-w)))
													=*((int*)(rp+(i+h)*rpitch+2*(k-w)));
										}
										if(i-h>0 && i-h<bht)
										{
											if( k-w>0 && k-w<bwd)
												*((int*)(lp+(i-h)*lpitch+2*(k-w)))
													=*((int*)(rp+(i-h)*rpitch+2*(k-w)));
											if(k+w>=0 && (k+w)<bwd)
												*((int*)(lp+(i-h)*lpitch+2*(k+w)))
													=*((int*)(rp+(i-h)*rpitch+2*(k+w)));
										}
									
									}
	




			}
			if(vi.IsYV12())
			{
				const unsigned char *rpU =RightFrame->GetReadPtr(PLANAR_U);
				unsigned char *lpU = LeftFrame->GetWritePtr(PLANAR_U);
				const unsigned char *rpV = RightFrame->GetReadPtr(PLANAR_V);
				unsigned char *lpV = LeftFrame->GetWritePtr(PLANAR_V);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				if(*type =='H'|| *type =='h')
				{
					for(int i=0; i<NWindowh;i++)
						for(int h=0; h<dwidth-1; h++)												
							for(int w=0;w<bwd;w++)
							{
								*(lp+(i*slatw+h)*lpitch+w)=*(rp+(i*slatw+h)*lpitch+w);
									
								if((h & 1)==1 || ( w &1)==1)
									continue;
										// U and V planes
								*(lpU+(i*slatw+h)/2*lpitchUV+w/2)
									=*(rpU+(i*slatw+h)/2*lpitchUV+w/2);
								*(lpV+(i*slatw+h)/2*lpitchUV+w/2)
									=*(rpV+(i*slatw+h)/2*lpitchUV+w/2);
							}

				
				}

				if(*type =='V'|| *type =='v')
				{
					
					for(int i=0; i<NWindoww;i++)
						for(int w=0;w<dwidth-1;w++)
							for(int h=0; h<bht; h++)
							{
								*(lp+h*lpitch+(i*slatw+w))=*(rp+h*lpitch+(i*slatw+w));
								if((h & 1)==1 || ( w &1)==1)
									continue;
										// U and V planes
								*(lpU+h/2*lpitchUV+(i*slatw+w)/2)=*(rpU+h/2*lpitchUV+(i*slatw+w)/2);
								*(lpV+h/2*lpitchUV+(i*slatw+w)/2)=*(rpV+h/2*lpitchUV+(i*slatw+w)/2);
							}
					
				}

				if(*type=='C'|| *type=='c')
				{
					for(int i=0;i<NWindowh+1;i++)
						for(int h=0;h<dwidth-1;h++)
							for(int k=0;k<NWindoww+1;k++)
								for(int w=0;w<dwidth-1;w++)
									if(i*slatw+h<bht && k*slatw+w<bwd)
									{
										*(lp+(i*slatw+h)*lpitch+(k*slatw+w))
											=*(rp+(i*slatw+h)*lpitch+(k*slatw+w));
										if((h & 1)==1 || ( w &1)==1)
											continue;
											// U and V planes
										*(lpU+(i*slatw+h)/2*lpitchUV+(k*slatw+w)/2)
											=*(rpU+(i*slatw+h)/2*lpitchUV+(k*slatw+w)/2);
										*(lpV+(i*slatw+h)/2*lpitchUV+(k*slatw+w)/2)
											=*(rpV+(i*slatw+h)/2*lpitchUV+(k*slatw+w)/2);
									}
					
				}
				if(*type=='D' || *type=='d')
					for(int e=0;e<2;e++)
						for(int i=-e*0.8*slatw;i<bht+0.8*slatw;i+=1.6*slatw)
							for(int k=-e*slatw/2;k<bwd+slatw/2;k+=slatw)
								for(int h=0;h<dheight;h++)							
									for(int w=0;w<(((dheight-h)*dwidth)/dheight)/2;w++)
									{
										if(i+h>0 && i+h<bht)
										{
											if( k+w>0 && k+w<bwd)
											{
												*(lp+(i+h)*lpitch+(k+w))
													=*(rp+(i+h)*rpitch+(k+w));
												if((h & 1)==1 || ( w &1)==1)
													continue;
														// U and V planes
												*(lpU+(i+h)/2*lpitchUV+(k+w)/2)
													=*(rpU+(i+h)/2*rpitchUV+(k+w)/2);
												*(lpV+(i+h)/2*lpitchUV+(k+w)/2)
													=*(rpV+(i+h)/2*rpitchUV+(k+w)/2);
											}
											if( k-w>0 && k-w<bwd)
											{
												*(lp+(i+h)*lpitch+(k-w))
													=*(rp+(i+h)*rpitch+(k-w));
												if((h & 1)==1 || ( w &1)==1)
													continue;
														// U and V planes
												*(lpU+(i+h)/2*lpitchUV+(k-w)/2)
													=*(rpU+(i+h)/2*rpitchUV+(k-w)/2);
												*(lpV+(i+h)/2*lpitchUV+(k-w)/2)
													=*(rpV+(i+h)/2*rpitchUV+(k-w)/2);
											}
										}
										if(i-h>0 && i-h<bht)
										{
											if( k-w>0 && k-w<bwd)			
											{
												*(lp+(i-h)*lpitch+(k-w))
													=*(rp+(i-h)*rpitch+(k-w));
												if((h & 1)==1 || ( w &1)==1)
													continue;
														// U and V planes
												*(lpU+(i-h)/2*lpitchUV+(k-w)/2)
													=*(rpU+(i-h)/2*rpitchUV+(k-w)/2);
												*(lpV+(i-h)/2*lpitchUV+(k-w)/2)
													=*(rpV+(i-h)/2*rpitchUV+(k-w)/2);
											}
											if(k+w>=0 && (k+w)<bwd)
											{						
												*(lp+(i-h)*lpitch+(k+w))
													=*(rp+(i-h)*rpitch+(k+w));
												if((h & 1)==1 || ( w &1)==1)
													continue;
														// U and V planes
												*(lpU+(i-h)/2*lpitchUV+(k+w)/2)
													=*(rpU+(i-h)/2*rpitchUV+(k+w)/2);
												*(lpV+(i-h)/2*lpitchUV+(k+w)/2)
													=*(rpV+(i-h)/2*rpitchUV+(k+w)/2);
											}
										}
									
									}

				
					




			}
			return LeftFrame;	
				
	}

/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransVenetianBlinds(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransVenetianBlinds(args[0].AsClip(),	// clip as LeftClip
								args[1].AsClip(), // as right clip
								args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
								args[3].AsInt(40), // width of slat/chequer
								args[4].AsString("Cheq"), // V = vertical, H = Horizontal, C chequers, D=diamonds
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransVenetianBlinds", "cci[width]i[type]s", Create_TransVenetianBlinds, 0);
				
    return "TransVenetianBlinds Transition Mask Plugin";
}
*/
