/******************************************************************************
TransWeave filter plugin for Avisynth by V.C.Mohan
The RightClip  covers  LeftClip either as slats moving from opposite, or JigSaw in hor & ver.
 The clips should have identical lengths, height, width and same pixel type 
Modified 25-9-04
*************************************************************************************************/  
/*
#include "windows.h"
#include "avisynth.h"
*/
class TransWeave : public GenericVideoFilter {
		PClip RightClip;		// define the second clip
		
		const char* type;	// type vert/hor/weave
		int slatw,slath;

		int overlap;
		void * abuf;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransWeave(PClip _child,PClip _RightClip, 
				int _overlap,const char * _type,  
			IScriptEnvironment* env) ;	
			
 	~TransWeave();		//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
	// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
//Here is the acutal constructor code used

TransWeave::TransWeave(PClip _child,PClip _RightClip,
					   int _overlap,const char * _type,  IScriptEnvironment* env) :
	GenericVideoFilter(_child),RightClip(_RightClip),
					overlap(_overlap),type(_type) 
			
{
	char * Tname="TransWeave";
	slatw=16;  //(vi.IsYUY2()? cslatw & 0xfffffffc:cslatw);
	slath=slatw;
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("TransWeave:Clips have differing pixel_types");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("TransWeave: This color format is not yet supported");
	if (vi.width != rvi.width || vi.height != rvi.height)
		env->ThrowError("%s:Clips have differing frame dimensions", Tname);
	if(!(*type=='V') && !(*type=='H') && !(*type=='J') && !(*type=='v')
		&& !(*type=='h') && !(*type=='j')  && !(*type=='W') && !(*type=='w'))
		env->ThrowError("TransWeave: Vertical, Horizontal, weave only available");
	if(slatw<4 ||(slatw>vi.width/2 && (*type=='v' || *type=='V'))
		|| (slatw>vi.height/2 && (*type=='h' || *type=='H')))
		env->ThrowError("TransWeave: Strip width is either less than 4 or too large");
	if((slatw>vi.width/16 || slatw>vi.height/16) && (*type=='j' || *type=='J'
		||*type=='h' || *type=='H'))
		env->ThrowError("TransWeave:Strip width not to exceed for this clip %d ",vi.width<vi.height?vi.width/16:vi.height/16);
		
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransWeave::~TransWeave() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/
void TransWeave::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransWeave::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//---------------------------------------------------------------------------------	

	PVideoFrame __stdcall TransWeave::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
			return child->GetFrame(en, env);
		if (en > video_fade_end)
			return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const	int bwd = vi.width;
        const	int bht = LeftFrame->GetHeight();
		const	int lpitch = LeftFrame->GetPitch();
		const	int rpitch = RightFrame->GetPitch();
		const	int nframes = overlap;
		env->MakeWritable(&LeftFrame);
		unsigned char* lp= LeftFrame->GetWritePtr();
		const unsigned char * rp= RightFrame->GetReadPtr();
		
		int deltax=((n+1)*bwd)/(nframes+1) & 0xfffffffe;
		int deltay = ((n+1)*bht)/(nframes+1) & 0xfffffffe;
		if(deltax <4 || deltay<4)
			return LeftFrame;
		int kb= vi.BitsPerPixel()/8;
			
				if(*type =='H'|| *type =='h')
				{
					for(int sh=0; sh<bht; sh+=2*slatw)
						for(int sw=0; sw<slatw; sw++)
							for(int w=0; w<deltax; w++)
								for(int k=0; k<kb; k++)
									*(lp+(sh+sw)*lpitch+kb*w+k)
									=*(rp+(sh+sw)*rpitch+kb*(bwd-deltax+w)+k);
					for( sh=slatw; sh<bht; sh+=2*slatw)
						for(int sw=0; sw<slatw; sw++)
							for(int w=0; w<deltax; w++)
								for(int k=0; k<kb; k++)
									*(lp+(sh+sw)*rpitch+kb*(bwd-deltax+w)+k)
									=*(rp+(sh+sw)*lpitch+kb*w+k);


				
				}

				if(*type =='V'|| *type =='v')
				{
					for(int ws=0; ws<=bwd; ws+=2*slatw)
						for(int sw=0; sw<slatw; sw++)
							for(int h=0; h<deltay; h++)
								for(int k=0; k<kb; k++)
									*(lp+h*lpitch+kb*(ws+sw)+k)
									=*(rp+(bht-deltay+h)*rpitch+kb*(ws+sw)+k);
					for( ws=slatw; ws<bwd; ws+=2*slatw)
						for(int sw=0; sw<slatw; sw++)
							for(int h=0; h<deltay; h++)
								for(int k=0; k<kb; k++)
									*(lp+(bht-deltay+h)*rpitch+kb*(ws+sw)+k)
									=*(rp+h*lpitch+kb*(ws+sw)+k);
					
				
				
				}

				if(*type=='J' || *type=='j')
				{

					for(int sh=0; sh<deltay-slatw; sh+=2*slatw)
						for(int h=0; h<slatw-1; h++)
							for(int sw=0; sw<deltax-slatw; sw+=2*slatw)
								for(int w=0; w<slatw-1; w++)										
									for(int k=0; k<kb; k++)
										*(lp+(sh+h)*lpitch+kb*(sw+w)+k)
											=*(rp+(bht-deltay+sh+h)*rpitch+kb*(bwd-deltax+sw+w)+k);
					for( sh=slatw; sh<deltay-slatw; sh+=2*slatw)
						for(int h=0; h<slatw-1; h++)							
							for(int sw=slatw; sw<deltax-slatw; sw+=2*slatw)
								for(int w=0; w<slatw-1; w++)										
									for(int k=0; k<kb; k++)
										*(lp+(sh+h)*lpitch+kb*(bwd-deltax+sw+w)+k)
											=*(rp+(bht-deltay+sh+h)*rpitch+kb*(bwd-deltax+sw+w)+k);
										
					for( sh=slatw; sh<deltay-slatw; sh+=2*slatw)
						for(int h=0; h<slatw-1; h++)							
							for(int sw=0; sw<deltax-slatw; sw+=2*slatw)
								for(int w=0; w<slatw-1; w++)										
									for(int k=0; k<kb; k++)
										*(lp+(bht-deltay+sh+h)*lpitch+kb*(bwd-deltax+sw+w)+k)
											=*(rp+(sh+h)*rpitch+kb*(bwd-deltax+sw+w)+k);
					for( sh=0; sh<deltay-slatw; sh+=2*slatw)
						for(int h=0; h<slatw-1; h++)							
							for(int sw=slatw; sw<deltax-slatw; sw+=2*slatw)
								for(int w=0; w<slatw-1; w++)										
									for(int k=0; k<kb; k++)
										*(lp+(bht-deltay+sh+h)*lpitch+kb*(sw+w)+k)
											=*(rp+(sh+h)*rpitch+kb*(bwd-deltax+sw+w)+k);

				
				}

				if(*type=='W' || *type=='w')
				{
				
						// fill the four quarters with four different quadrants of input
					deltax=deltax & 0xfffffff0;
					deltay=deltay & 0xfffffff0;
					
						
						// copy alternate squares from alternate strips
						// squares from horizontally moving blocks
					for(int sh=slath/2;sh<bht/2-slath; sh+=3*slath/2)
						for(int h=sh;h<sh+slath-1; h++)
							for(int sw=0;sw<deltax-2*slatw;sw+=3*slatw)
								for(int w=sw;w<sw+2*slatw-1;w++)
									for(int k=0; k<kb;k++)
									{
										*(lp+(h)*lpitch+kb*w+k)
											=*(rp+(h)*rpitch+kb*(bwd-deltax+w)+k);
										*(lp+(bht/2+h)*lpitch+kb*(bwd-deltax+w)+k)
											=*(rp+(bht/2+h)*rpitch+kb*w+k);
									}
					for(sh=slath/2;sh<bht/2-slath; sh+=3*slath/2)
						for(int h=sh;h<sh+slath-1; h++)
							for(int sw=slatw;sw<deltax-2*slatw;sw+=3*slatw)
								for(int w=sw;w<sw+2*slatw-1;w++)
									for(int k=0; k<kb;k++)
									{
										*(lp+(h)*lpitch+kb*w+k)
											=*(rp+(h)*rpitch+kb*(bwd-deltax+w)+k);
										*(lp+(bht/2+h)*lpitch+kb*(bwd-deltax+w)+k)
											=*(rp+(bht/2+h)*rpitch+kb*w+k);
									}

						// squares from vertically moving blocks. these should be adjacent to h moving
					for(int sw=slatw/2;sw<bwd/2-slatw;sw+=3*slatw/2)
						for(int w=sw;w<sw+slatw-1;w++)
							for(int sh=slath; sh<deltay-2*slath;sh+=3*slath)
								for(int h=sh; h<sh+2*slath-1;h++)
									for(int k=0; k<kb; k++)
									{
										*(lp+h*lpitch+kb*(bwd/2+w)+k)
											=*(rp+(bht-deltay+h)*rpitch+kb*(bwd/2+w)+k);
										*(lp+(bht-deltay+h)*lpitch+kb*(w)+k)
											=*(rp+h*rpitch+kb*(w)+k);
									}
					for( sw=slatw/2;sw<bwd/2-slatw;sw+=3*slatw/2)
						for(int w=sw;w<sw+slatw-1;w++)
							for(int sh=0; sh<deltay-2*slath;sh+=3*slath)
								for(int h=sh; h<sh+2*slath-1;h++)
									for(int k=0; k<kb; k++)
									{
										*(lp+h*lpitch+kb*(bwd/2+w)+k)
											=*(rp+(bht-deltay+h)*rpitch+kb*(bwd/2+w)+k);
										*(lp+(bht-deltay+h)*lpitch+kb*(w)+k)
											=*(rp+h*rpitch+kb*(w)+k);
									}
				
				}

			if(vi.IsYV12())
			{
				// Only U V planes . Y plane was done above in common code
				const unsigned char *rpU =RightFrame->GetReadPtr(PLANAR_U);
				unsigned char *lpU = LeftFrame->GetWritePtr(PLANAR_U);
				const unsigned char *rpV = RightFrame->GetReadPtr(PLANAR_V);
				unsigned char *lpV = LeftFrame->GetWritePtr(PLANAR_V);
				const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
				const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
				const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
				const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
				if(*type =='H'|| *type =='h')
				{
					for(int sh=0; sh<bhtUV; sh+=2*slatw/2)
						for(int sw=0; sw<slatw/2; sw++)
							for(int w=0; w<deltax/2; w++)
							{
								
								*(lpU+(sh+sw)*lpitchUV+w)
									=*(rpU+(sh+sw)*rpitchUV+(bwdUV-deltax/2+w));
								*(lpV+(sh+sw)*lpitchUV+w)
									=*(rpV+(sh+sw)*rpitchUV+(bwdUV-deltax/2+w));
							}
					for( sh=slatw/2; sh<bhtUV; sh+=2*slatw/2)
						for(int sw=0; sw<slatw/2; sw++)
							for(int w=0; w<deltax/2; w++)
							{	
								*(lpU+(sh+sw)*rpitchUV+(bwdUV-deltax/2+w))
									=*(rpU+(sh+sw)*lpitchUV+w);
								*(lpV+(sh+sw)*rpitchUV+(bwdUV-deltax/2+w))
									=*(rpV+(sh+sw)*lpitchUV+w);
							}				
				}

				if(*type =='V'|| *type =='v')
				{
					for(int ws=0; ws<=bwdUV; ws+=2*slatw/2)
						for(int sw=0; sw<slatw/2; sw++)
							for(int h=0; h<deltay/2; h++)
							{	
								*(lpU+h*lpitchUV+(ws+sw))
									=*(rpU+(bhtUV-deltay/2+h)*rpitchUV+(ws+sw));
								*(lpV+h*lpitchUV+(ws+sw))
									=*(rpV+(bhtUV-deltay/2+h)*rpitchUV+(ws+sw));
							}
					for( ws=slatw/2; ws<bwdUV; ws+=2*slatw/2)
						for(int sw=0; sw<slatw/2; sw++)
							for(int h=0; h<deltay/2; h++)
							{	
								*(lpU+(bhtUV-deltay/2+h)*rpitchUV+(ws+sw))
									=*(rpU+h*lpitchUV+(ws+sw));
								*(lpV+(bhtUV-deltay/2+h)*rpitchUV+(ws+sw))
									=*(rpV+h*lpitchUV+(ws+sw));
							}				
				}

				if(*type=='J' || *type=='j')
				{

					for(int sh=0; sh<deltay/2-slatw/2; sh+=2*slatw/2)
						for(int h=0; h<slatw/2-1; h++)
							for(int sw=0; sw<deltax/2-slatw/2; sw+=2*slatw/2)
								for(int w=0; w<slatw/2-1; w++)										
								{	
									*(lpU+(sh+h)*lpitchUV+(sw+w))
										=*(rpU+(bhtUV-deltay/2+sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
									*(lpV+(sh+h)*lpitchUV+(sw+w))
										=*(rpV+(bhtUV-deltay/2+sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
								}
					for( sh=slatw/2; sh<deltay/2-slatw/2; sh+=2*slatw/2)
						for(int h=0; h<slatw/2-1; h++)							
							for(int sw=slatw/2; sw<deltax/2-slatw/2; sw+=2*slatw/2)
								for(int w=0; w<slatw/2-1; w++)										
								{	
									*(lpU+(sh+h)*lpitchUV+(bwdUV-deltax/2+sw+w))
										=*(rpU+(bhtUV-deltay/2+sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
									*(lpV+(sh+h)*lpitchUV+(bwdUV-deltax/2+sw+w))
										=*(rpV+(bhtUV-deltay/2+sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
								}
										
					for( sh=slatw/2; sh<deltay/2-slatw/2; sh+=2*slatw/2)
						for(int h=0; h<slatw/2-1; h++)							
							for(int sw=0; sw<deltax/2-slatw/2; sw+=2*slatw/2)
								for(int w=0; w<slatw/2-1; w++)										
								{	
									*(lpU+(bhtUV-deltay/2+sh+h)*lpitchUV+(bwdUV-deltax/2+sw+w))
										=*(rpU+(sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
									*(lpV+(bhtUV-deltay/2+sh+h)*lpitchUV+(bwdUV-deltax/2+sw+w))
										=*(rpV+(sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
								}
					for( sh=0; sh<deltay/2-slatw/2; sh+=2*slatw/2)
						for(int h=0; h<slatw/2-1; h++)							
							for(int sw=slatw/2; sw<deltax/2-slatw/2; sw+=2*slatw/2)
								for(int w=0; w<slatw/2-1; w++)										
								{	
									*(lpU+(bhtUV-deltay/2+sh+h)*lpitchUV+(sw+w))
										=*(rpU+(sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
									*(lpV+(bhtUV-deltay/2+sh+h)*lpitchUV+(sw+w))
										=*(rpV+(sh+h)*rpitchUV+(bwdUV-deltax/2+sw+w));
								}				
				}

				if(*type=='W' || *type=='w')
				{
					int delta1x=(deltax & 0xfffffff0)>>1;
					int delta1y=(deltay & 0xfffffff0)>>1;
					int slat1h=slath/2;
					int slat1w=slatw/2;
					
						
							// copy alternate squares from alternate strips
						// squares from horizontally moving blocks
					for(int sh=slat1h/2;sh<bhtUV/2-slat1h; sh+=3*slat1h/2)
						for(int h=sh;h<sh+slat1h-1; h++)
							for(int sw=0;sw<delta1x-2*slat1w;sw+=3*slat1w)
								for(int w=sw;w<sw+2*slat1w-1;w++)
								
									{
										*(lpU+(h)*lpitchUV+w)
											=*(rpU+(h)*rpitchUV+(bwdUV-delta1x+w));
										*(lpU+(bhtUV/2+h)*lpitchUV+(bwdUV-delta1x+w))
											=*(rpU+(bhtUV/2+h)*rpitchUV+w);
										*(lpV+(h)*lpitchUV+w)
											=*(rpV+(h)*rpitchUV+(bwdUV-delta1x+w));
										*(lpV+(bhtUV/2+h)*lpitchUV+(bwdUV-delta1x+w))
											=*(rpV+(bhtUV/2+h)*rpitchUV+w);
									}
					for(sh=slat1h/2;sh<bhtUV/2-slat1h; sh+=3*slat1h/2)
						for(int h=sh;h<sh+slat1h-1; h++)
							for(int sw=slat1w;sw<delta1x-2*slat1w;sw+=3*slat1w)
								for(int w=sw;w<sw+2*slat1w-1;w++)
									
									{
										*(lpU+(h)*lpitchUV+w)
											=*(rpU+(h)*rpitchUV+(bwdUV-delta1x+w));
										*(lpU+(bhtUV/2+h)*lpitchUV+(bwdUV-delta1x+w))
											=*(rpU+(bhtUV/2+h)*rpitchUV+w);
										*(lpV+(h)*lpitchUV+w)
											=*(rpV+(h)*rpitchUV+(bwdUV-delta1x+w));
										*(lpV+(bhtUV/2+h)*lpitchUV+(bwdUV-delta1x+w))
											=*(rpV+(bhtUV/2+h)*rpitchUV+w);
									}

						// squares from vertically moving blocks. these should be adjacent to h moving
					for(int sw=slat1w/2;sw<bwdUV/2-slat1w;sw+=3*slat1w/2)
						for(int w=sw;w<sw+slat1w-1;w++)
							for(int sh=slat1h; sh<delta1y-2*slat1h;sh+=3*slat1h)
								for(int h=sh; h<sh+2*slat1h-1;h++)
									
									{
										*(lpU+h*lpitchUV+(bwdUV/2+w))
											=*(rpU+(bhtUV-delta1y+h)*rpitchUV+(bwdUV/2+w));
										*(lpU+(bhtUV-delta1y+h)*lpitchUV+(w))
											=*(rpU+h*rpitchUV+(w));
										*(lpV+h*lpitchUV+(bwdUV/2+w))
											=*(rpV+(bhtUV-delta1y+h)*rpitchUV+(bwdUV/2+w));
										*(lpV+(bhtUV-delta1y+h)*lpitchUV+(w))
											=*(rpV+h*rpitchUV+(w));
									}
					for( sw=slat1w/2;sw<bwdUV/2-slat1w;sw+=3*slat1w/2)
						for(int w=sw;w<sw+slat1w-1;w++)
							for(int sh=0; sh<delta1y-2*slat1h;sh+=3*slat1h)
								for(int h=sh; h<sh+2*slat1h-1;h++)
									
									{
										*(lpU+h*lpitchUV+(bwdUV/2+w))
											=*(rpU+(bhtUV-delta1y+h)*rpitchUV+(bwdUV/2+w));
										*(lpU+(bhtUV-delta1y+h)*lpitchUV+(w))
											=*(rpU+h*rpitchUV+(w));
										*(lpV+h*lpitchUV+(bwdUV/2+w))
											=*(rpV+(bhtUV-delta1y+h)*rpitchUV+(bwdUV/2+w));
										*(lpV+(bhtUV-delta1y+h)*lpitchUV+(w))
											=*(rpV+h*rpitchUV+(w));
									}
					}
					
			}

			
			return LeftFrame;
			

			

				
	}

/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransWeave(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransWeave(args[0].AsClip(),	// clip as LeftClip
								args[1].AsClip(), // as right clip
								args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
								args[3].AsString("weave"), // Ver = vertical Interleave, Hor = Horizontal, jigsaw, weave
							//	args[3].AsInt(20), // breadth of the slats
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransWeave", "cci[type]s", Create_TransWeave, 0);
				
    return "TransWeave Transition  Plugin";
}
*/
