/******************************************************************************
TransWipe filter plugin for Avisynth by V.C.Mohan
The LeftClip  unmasks a stationary RightClip clip in the direction specified.
 The clips should have identical lengths, height, width and same pixel type 
The doom9 forum has been helpful and I have also 
used partly adapted function coding 
from Avisynth and simple sample. Roll Diameter and
 shading along roll length are other params
********************************************************************************/  
/*#include "windows.h"
#include "avisynth.h"
#include "LineZ.cpp"
*/
class TransWipe : public GenericVideoFilter {
		PClip RightClip;
		const char* dir;

		int overlap;
		void * abuf;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
		
		 
  public:
							//Definition of function
    TransWipe(PClip _child, PClip _RightClip,
				int _overlap, const char* _dir,
				  
				IScriptEnvironment* env) ;	
			
 	~TransWipe();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
bool __stdcall GetParity(int n);

};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
void TransWipe::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
  
 
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransWipe::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


//Here is the acutal constructor code used

TransWipe::TransWipe(PClip _child, PClip _RightClip,
					 int _overlap,const char* _dir,
					 
					 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
					overlap(_overlap),dir(_dir)
						
			
{
char * Tname="TransWipe";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransWipe:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
	env->ThrowError("TransWipe: This color format is not supported here");
if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
	env->ThrowError("TransWipe: The heights/widths of clips are unequal");
if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
	env->ThrowError("TransWipe: Height/width of frame are not multiple of 4");
if(!(*dir=='l') && !(*dir=='L') && !(*dir=='r') && !(*dir=='R') &&
	!(*dir=='u') && !(*dir=='U') && !(*dir=='d') && !(*dir=='D'))
	env->ThrowError("TransWipe:up, down,left or right are valid directions");
	if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);
	if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);

	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransWipe::~TransWipe() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
	PVideoFrame __stdcall TransWipe::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
			return child->GetFrame(en, env);
		if (en > video_fade_end)
			return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
				 
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		int deltay = ((n+1)*bht)/(overlap+1);
		deltay = deltay & 0xFFFFFFFC;
		int deltax = ((n+1)*bwd)/(overlap+1);
		deltax=deltax & 0xFFFFFFFC;
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		int  wht, wwd;
		env->MakeWritable(&RightFrame);
		unsigned char* RightFramep= RightFrame->GetWritePtr();
		const unsigned char * LeftFramep= LeftFrame->GetReadPtr();
		const unsigned char * lpU, *lpV;
		unsigned char *rpU, *rpV;
		const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
		const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		if(vi.IsYV12())
		{
				rpU = RightFrame->GetWritePtr(PLANAR_U);
				lpU = LeftFrame->GetReadPtr(PLANAR_U);
				rpV = RightFrame->GetWritePtr(PLANAR_V);
				lpV = LeftFrame->GetReadPtr(PLANAR_V);
		}
				

		if(*dir=='u' || *dir=='U')
			
		{			//LeftFrame peels upward unmasking RightFrame
			if(deltay<2)
				return LeftFrame;
			

			if(vi.IsRGB32())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineRGB32(LeftFramep+deltay*lpitch+4*w,
									RightFramep+deltay*rpitch+4*w,
									bht-1-deltay,lpitch,rpitch);
				}
				
			}
			if(vi.IsRGB24())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineRGB24(LeftFramep+deltay*lpitch+3*w,
									RightFramep+deltay*rpitch+3*w,
									bht-1-deltay,lpitch,rpitch);
				}
			
			}
			if(vi.IsYUY2())
			{
				for(int w=0;w<bwd;w+=2)
				{
					CopyVLineYUY2(LeftFramep+(bht-1-deltay)*lpitch+2*w,
									RightFramep+(bht-1-deltay)*rpitch+2*w,
									deltay,lpitch,rpitch);
				}
		
			}
			if(vi.IsYV12())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineYV12(LeftFramep+(bht-1-deltay)*lpitch+w,
									RightFramep+(bht-1-deltay)*rpitch+w,
									deltay,lpitch,rpitch);
					if((w & 1)==1)
						continue;
					CopyVLineYV12(lpU+(bht-1-deltay)/2*lpitchUV+w/2,
									rpU+(bht-1-deltay)/2*rpitchUV+w/2,
									deltay/2,lpitchUV,rpitchUV);
					CopyVLineYV12(lpV+(bht-1-deltay)/2*lpitchUV+w/2,
									rpV+(bht-1-deltay)/2*rpitchUV+w/2,
									deltay/2,lpitchUV,rpitchUV);
				}
	
			}

		}
			

		if(*dir=='d' || *dir=='D')
			
		{			//LeftFrame peels down unmasking RightFrame
				
			if(deltay<2)
				return LeftFrame;
			
			if(vi.IsRGB32())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineRGB32(LeftFramep+4*w,
									RightFramep+4*w,
									(bht-deltay),lpitch,rpitch);
				}
			
			}
			if(vi.IsRGB24())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineRGB24(LeftFramep+3*w,
									RightFramep+3*w,
									(bht-deltay),lpitch,rpitch);
				}
			
			}
			if(vi.IsYUY2())
			{
				for(int w=0;w<bwd;w+=2)
				{
					CopyVLineYUY2(LeftFramep+deltay*lpitch+2*w,
									RightFramep+deltay*rpitch+2*w,
									(bht-deltay),lpitch,rpitch);
				}
			
			}
			if(vi.IsYV12())
			{
				for(int w=0;w<bwd;w++)
				{
					CopyVLineYV12(LeftFramep+deltay*lpitch+w,
									RightFramep+deltay*rpitch+w,
									(bht-deltay),lpitch,rpitch);
					if((w & 1)==1)
						continue;
					CopyVLineYV12(lpU+deltay/2*lpitchUV+w/2,
									rpU+deltay/2*rpitchUV+w/2,
									(bht-deltay)/2,lpitchUV,rpitchUV);
					CopyVLineYV12(lpV+deltay/2*lpitchUV+w/2,
									rpV+deltay/2*rpitchUV+w/2,
									(bht-deltay)/2,lpitchUV,rpitchUV);
				}
		
			}

		}
			
		if(*dir=='r' || *dir=='R')
			
		{		//LeftFrame peels out Right unmasking RightFrame
			if(deltax<2)
				return LeftFrame;
		
			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)				
				{
					CopyHLineRGB32(LeftFramep+h*lpitch+4*deltax,
									RightFramep+h*rpitch+4*deltax,
										bwd-deltax);
				}
	
			}
			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineRGB24(LeftFramep+h*lpitch+3*deltax,
									RightFramep+h*rpitch+3*deltax,
										bwd-deltax);
				}
	
			}
			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)				
				{
					CopyHLineYUY2(LeftFramep+h*lpitch+2*deltax,
									RightFramep+h*rpitch+2*deltax,
										bwd-deltax);
				}

			}
			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)				
				{
					CopyHLineYV12(LeftFramep+h*lpitch+deltax,
									RightFramep+h*rpitch+deltax,
										bwd-deltax);
					if((h & 1)==1)
						continue;
					CopyHLineYV12(lpU+h/2*lpitchUV+deltax/2,
									rpU+h/2*rpitchUV+deltax/2,
										(bwd-deltax)/2);
					CopyHLineYV12(lpV+h/2*lpitchUV+deltax/2,
									rpV+h/2*rpitchUV+deltax/2,
										(bwd-deltax)/2);
				}

			}

		}
			

		if(*dir=='l' || *dir=='L')
			
		{			//LeftFrame moves westward unmasking RightFrame
			if(deltax<2)
				return LeftFrame;
			

			if(vi.IsRGB32())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineRGB32(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax);
				}
			
			}
			if(vi.IsRGB24())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineRGB24(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax);
				}
		
			}
			if(vi.IsYUY2())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineYUY2(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax);
				}
	
			}
			if(vi.IsYV12())
			{
				for(int h=0;h<bht-1;h++)
				{
					CopyHLineYV12(LeftFramep+h*lpitch,
									RightFramep+h*rpitch,
										bwd-deltax);
					if((h & 1)==1)
						continue;
					CopyHLineYV12(lpU+h/2*lpitchUV,
									rpU+h/2*rpitchUV,
										(bwd-deltax)/2);
					CopyHLineYV12(lpV+h/2*lpitchUV,
									rpV+h/2*rpitchUV,
										(bwd-deltax)/2);
				}
	
			}
		}
		return RightFrame;
		
	}

/********** 



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransWipe(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
				
	return new TransWipe(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// Clip as RightClip   
 						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
						args[3].AsString("left"),	// Transition direction
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransWipe", "cci[dir]s", Create_TransWipe, 0);
				
    return "TransWipe  Plugin";
}
*/
