
/******************************************************************************
The TransMarbles Filter copies  circular disks from source frame to destination frame
after magnifying them
Modified 23-9-04
*************************************************************************************************/  

class TransMarbles : public GenericVideoFilter 
{
		PClip RightClip;
		int  rad;
		int imag;
		bool drop;

		int overlap;

		void * abuf;
//		bool first,laudio,raudio;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_fade_end ;
		__int64 audio_fade_start;
		__int64 abufsize;
		
  public:
							//Definition of function
    TransMarbles(PClip _child, PClip _RightClip,
				int _overlap,int _rad,int _mag, bool _drop,  
			IScriptEnvironment* env) ;	
			
 	~TransMarbles();				//destructor
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
	void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
	bool __stdcall GetParity(int n);
};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/


TransMarbles::TransMarbles(PClip _child, PClip _RightClip,
						   int _overlap,int _rad,int _mag,bool _drop,  IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
				overlap(_overlap),rad(_rad),imag(_mag),drop(_drop)  
			
{
	char * Tname="TransMarbles";	
const VideoInfo& rvi= RightClip->GetVideoInfo();
if (!vi.pixel_type == rvi.pixel_type)
	env->ThrowError("TransMarbles:Clips have differing pixel_types");
if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2()&& !vi.IsYV12())
	env->ThrowError("TransMarbles: This color format is not supported here");
if (vi.width != rvi.width || vi.height != rvi.height)
	env->ThrowError("%s:Clips have differing frame dimensions", Tname);
if(rad>sqrt(vi.width*vi.width+vi.height*vi.height)/2)
	env->ThrowError("TransMarbles: radius must not be more than %d",sqrt(vi.width*vi.width+vi.height*vi.height)/2);
if(overlap<2 && overlap>=0)
		env->ThrowError("%s: Overlap should be atleast 2 frames ", Tname);	
if(overlap<0)

		overlap=-(overlap*vi.fps_numerator)/vi.fps_denominator;// number of seconds convert to frames 
	if(overlap>vi.num_frames || overlap> rvi.num_frames)
		env->ThrowError("%s: Clip is shorter than overlap ", Tname); 

  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  // where the following variables gets defined:
  // PClip child;   // Contains the LeftClip clip.
  // VideoInfo vi;  // Contains videoinfo on the LeftClip clip.
	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}
	if(vi.HasAudio() ^ rvi.HasAudio())
		env->ThrowError("%s:One of the clips has audio and other does not", Tname);


	video_fade_start = vi.num_frames - overlap;
	video_fade_end = vi.num_frames - 1;

	audio_fade_end = vi.num_audio_samples-1;
	audio_fade_start = vi.AudioSamplesFromFrames(video_fade_start);

	vi.num_frames = video_fade_start + rvi.num_frames;
	vi.num_audio_samples = audio_fade_start + rvi.num_audio_samples;
	abufsize=0;
}
// This is where any actual destructor code used goes
TransMarbles::~TransMarbles() {
	
// This is where you can deallocate any memory you might have used.
}
//_____________________________________________________________________
void TransMarbles::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_fade_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_fade_end)
	{
		RightClip->GetAudio(buf, start -audio_fade_start, count, env);
		return;
	}
	const int bytes = vi.BytesFromAudioSamples(count);
    if (abufsize < bytes)
	{
		if(abufsize>0)
			delete[] abuf;
		abuf = new char[bytes];
		abufsize = bytes;
	}

	if(start<audio_fade_start ) // case 
	{
		void* tabs=(char *)buf+(audio_fade_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_fade_end)
		{
			
			__int64 tcount=count-(audio_fade_start-start);	// right clip samples
			child->GetAudio(buf, start,count, env);
			RightClip->GetAudio(abuf, 0, tcount, env);
			MixAudio(tabs,0,tcount, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		if(start+count>=audio_fade_end)
		{
			__int64 tcount=start+count-audio_fade_end-1;		// Right clip samples beyond transition

			child->GetAudio(buf, start, audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, 0, audio_fade_end-audio_fade_start+1, env);
			MixAudio(tabs,0,audio_fade_end-audio_fade_start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start+1);
			if(tcount>0)
				RightClip->GetAudio(
							(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
							audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;
			abufsize=0;
		}
	}
	if(start>audio_fade_start )
	{
		
		if(start+count<audio_fade_end)
		{
		
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(abuf, start-audio_fade_start, count, env);
			MixAudio(buf,start-audio_fade_start,count, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);

		}
		else
		{

			__int64 tcount=count-(audio_fade_end-start+1);

			child->GetAudio(buf, start,audio_fade_end-start+1, env);
			RightClip->GetAudio(abuf, start-audio_fade_start+1,audio_fade_end-start+1, env);
			MixAudio(buf,start-audio_fade_start,audio_fade_end-start+1, abuf, vi.sample_type,
				vi.nchannels,audio_fade_end-audio_fade_start);
			if(tcount>0)
				RightClip->GetAudio(
					(char*)buf+(audio_fade_end-start+1)*vi.BytesPerAudioSample(),
					audio_fade_end-audio_fade_start+1, start+count-audio_fade_end-1, env);
			delete[] abuf;

		}
	}

}
bool TransMarbles::GetParity(int n) 
{
  return (n < video_fade_start) ? child->GetParity(n) : RightClip->GetParity(n - video_fade_start);
}


	
/***************************************************************/	
	PVideoFrame __stdcall TransMarbles::GetFrame(int en, IScriptEnvironment* env)
	{
		if (en < video_fade_start)
				return child->GetFrame(en, env);
		if (en > video_fade_end)
				return RightClip->GetFrame(en - video_fade_start, env);
		int n=en-video_fade_start;
		
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		PVideoFrame Frame;
		
		const	int bwd = vi.width; 
        const	int bht = LeftFrame->GetHeight();	// LeftClip->GetHeight()?
		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();
		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		const int bwdUV = LeftFrame->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
		const int bhtUV = LeftFrame->GetHeight(PLANAR_U);
		
		int radius;
		unsigned char* dstp, *dstpU, *dstpV;
		const unsigned char * srcp, *srcpU, *srcpV;
		env->MakeWritable(&LeftFrame);
			dstp= LeftFrame->GetWritePtr();
			srcp= RightFrame->GetReadPtr();
			if(vi.IsYV12())
			{
				dstpU = LeftFrame->GetWritePtr(PLANAR_U);
				dstpV = LeftFrame->GetWritePtr(PLANAR_V);
				srcpU = RightFrame->GetReadPtr(PLANAR_U);
				srcpV = RightFrame->GetReadPtr(PLANAR_V);
			}
//		Frame=LeftFrame;
		double mag=(double)imag;
		double mag1=mag;
		int kb= vi.BitsPerPixel()/8;

		if(imag <2)
		{
			mag1=1;		
			
			radius=((n+1)*rad)/(overlap+1);
		}

		
		if(imag>1)
		{
			if(2*n<=overlap)
			{

				radius=(rad*2*(n+1))/(overlap+1);
				if(drop)
					mag1=mag+mag;
			}
		
			if(2*n>overlap)
			{
				radius=rad-(double)(2*n-overlap)*rad/overlap;
				if(!drop)
					radius=rad;

				mag1=1+mag-(double)(2*n-overlap)*mag/overlap;
				if(drop)
					mag1=2*mag-2.0*(2*n-overlap)*mag/overlap;

					// copy right frame on to left frame
				for(int h=0; h<bht; h++)
					for(int w=0;w<kb*bwd;w++)
						*(dstp+h*lpitch+w)=*(srcp+h*rpitch+w);
				if(vi.IsYV12())
					for(int h=0; h<bhtUV; h++)
						for(int w=0;w<bwdUV;w++)
						{
							*(dstpU+h*lpitchUV+w)=*(srcpU+h*rpitchUV+w);
							*(dstpV+h*lpitchUV+w)=*(srcpV+h*rpitchUV+w);
						}
			}
		}
		
		


		int nx=bwd/(2*rad);
		if(nx<1)
			nx=1;
		int ny = bht/(2*rad);
		if(ny<1)
			ny=1;
		int starty=bht/(2*ny)& 0xfffffffe;
		int startx = bwd/(2*nx)& 0xfffffffe;

		if(vi.IsRGB32())
		{
			for( int r=startx; r<bwd;r+=1.6*rad )
				for(int c =starty; c<bht;c+=1.6*rad)
					FastDiscoRGB32( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								rpitch,lpitch, mag1);
		}
		if(vi.IsRGB24())
		{
			for( int r=startx; r<bwd;r+=1.6*rad )
				for(int c =starty; c<bht;c+=1.6*rad)
					FastDiscoRGB24( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								rpitch,lpitch, mag1);
		}
		if(vi.IsYUY2())
		{
			for( int r=startx; r<bwd;r+=1.6*rad )
				for(int c =starty; c<bht;c+=1.6*rad)
					FastDiscoYUY2( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								rpitch,lpitch, mag1);
	
		}
		if(vi.IsYV12())
		{
			for( int r=startx; r<bwd;r+=1.6*rad )
				for(int c =starty; c<bht;c+=1.6*rad)
					FastDiscoYV12( srcp, dstp, radius,
								r, c, bht, bwd,
								r, c, bht, bwd,
								rpitch,lpitch, mag1);
			for(r=startx/2; r<bwdUV;r+=1.6*rad/2 )
				for(int c =starty/2; c<bhtUV;c+=1.6*rad/2)
				{
					FastDiscoYV12( srcpU, dstpU, radius/2,
								r, c, bhtUV, bwdUV,
								r, c, bhtUV, bwdUV,
								rpitchUV,lpitchUV, mag1);
					FastDiscoYV12( srcpV, dstpV, radius/2,
								r, c, bhtUV, bwdUV,
								r, c, bhtUV, bwdUV,
								rpitchUV,lpitchUV, mag1);
				}
	
		}
		return LeftFrame;

		
	}

/**************
DiscoRGB32(const char* srcp,  char* dstp, int radius, int srcx, int srcy, int srch, int srcw,
		int dstx, int dsty, int dsth, int dstw, int spitch, int dpitch, bool mag, bool drop);
DiscoRGB24(const char* srcp,  char* dstp, int radius, int srcx, int srcy, int srch, int srcw,
		int dstx, int dsty, int dsth, int dstw, int spitch, int dpitch, bool mag, bool drop);
***********************/
  	 
/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_TransMarbles(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
/******************************************************************************************/

/**********************************************************************************************/
	
		if(args[3].AsInt(16)<8 || args[4].AsInt(16)<2 || args[4].AsInt(16)>24)
		env->ThrowError("TransMarbles: radius must be minimum 8 and mag between 2 and 24");
	


	return new TransMarbles(args[0].AsClip(),	// clip as LeftClip
						args[1].AsClip(),	// clip as RightClip
						args[2].AsInt(),	//overlap of clips. -ve time in seconds, +ve frames
 						args[3].AsInt(16),	// radius of circle
						args[4].AsInt(16),	// magnification x
						args[5].AsBool(true),				//(default) true for water drop effect
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("TransMarbles", "cci[radius]i[mag]i[drop]b", Create_TransMarbles, 0);
				
    return "TransMarbles Transition TransMarbles Plugin";
}

*/