/*********************************************************************************************
LineZ contain a set of functions that operate on a single line (vertical or Horizontal)
These are 1 Copy, 2 Flip and magnify linear or as a lens or constant value; Shading
can also be applied. These functions do not check the parameters and assume them to be correct.
Input and out pointers to the middle(except for copy and plain flip functions)
 of window, magnification, window winh( height). pitch for vertical functions  flip, shade are 
some of the parameters needed. 
**********************************************************************************************
// Definitions of Horizontal line functions
// source pointer at start destination pointer at start,  Segment width in pixels
inline void CopyHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw);
inline void CopyHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw);
inline void FlipHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw);
inline void FlipHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw);
// ResizeHLineRGB32 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw, double factor);
// ResizeHLineRGB24 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw, double factor);
// FlipResizeHLineRGB32 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// FlipResizeHLineRGB24 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// PeelzHLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally.
inline void PeelzHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw, int shade=31);
// PeelzHLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally.
inline void PeelzHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw, int shade=31);
//ShadeHLineRGB24 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw,int shade);
//ShadeHLineRGB32 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,int shade);
//*************************************************************************************************
***************************************************************************************************
// Definitions of Vertical line functions
// source pointer at end pixel, destination pointer at end pixel,  Segment width in pixels
inline void CopyVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
inline void CopyVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
inline void FlipVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
inline void FlipVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
// ResizeVLineRGB32 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// ResizeVLineRGB24 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// FlipResizeVLineRGB32 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// FlipResizeVLineRGB24 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// PeelzVLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh.
//The line is flipped horizontally.
inline void PeelzVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch, int shade=31);
// PeelzVLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh.
//The line is flipped horizontally.
inline void PeelzVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch, int shade=31);
//ShadeVLineRGB24 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade);
//ShadeVLineRGB32 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,int spitch, int dpitch,int shade);

********************************************************************************************************************
//YV12 functions
inline void FlipHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw);
inline void CopyHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw);
// ResizeHLineYV12 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// FlipResizeHLineYV12 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// FlipLenzHLineYV12 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void FlipLenzHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// LenzHLineYV12 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void LenzHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// PeelzHLineYV12 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally. Shading can be done optionally
inline void PeelzHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,int shade);
//ShadeHLineYV12 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,int shade);
// Vertical Functions:-
//FlipVLineYV12 expects the srcp and dstp to be at 32 bit boundary of the start point of the segment to be copied. At a time
// two adjacent vertical lines are flipped.  
inline void FlipVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
// CopyVLineYV12 expects the srcp and dstp to be at the start point of the segment to be copied. Two adjacent
//pixells are copied at a time. So two vertical lines are copied 
inline void CopyVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
// FlipResizeVLineYV12 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// ResizeVLineYV12 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// PeelzVLineYV122 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh, and divisible by 4.
//The line is flipped vertically. Shading can be done optionally  shade 0 will have no shading, -255 to 255
inline void PeelzVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade);
//ShadeVLineYV12 adds shading to a line. -255 to 255 Center gets whiter compared to edges.
// the pointers point the center of window. Two lines are copied/shaded at a time 
inline void ShadeVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade);
//***********************************************************************************
//YUV2 Format Functions
// Vertical Line Functions:-
//FlipVLineYUY2 expects the srcp and dstp to be at 32 bit boundary of the start point of the segment to be copied. At a time
// two adjacent vertical lines are flipped.  
inline void FlipVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
// CopyVLineYUY2 expects the srcp and dstp to be at the start point of the segment to be copied. Two adjacent
//pixells are copied at a time. So two vertical lines are copied 
inline void CopyVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch);
// FlipResizeVLineYUY2 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// ResizeVLineYUY2 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor);
// PeelzVLineYUY22 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh, and divisible by 4.
//The line is flipped vertically. Shading can be done optionally  shade 0 will have no shading, -255 to 255
inline void PeelzVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade);
//ShadeVLineYUY2 adds shading to a line. -255 to 255 Center gets whiter compared to edges.
// the pointers point the center of window. Two lines are copied/shaded at a time 
inline void ShadeVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade);
************************************************************************************************************
//Horizontal line functions
inline void FlipHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw);
inline void CopyHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw);
// ResizeHLineYUY2 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// FlipResizeHLineYUY2 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// FlipLenzHLineYUY2 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void FlipLenzHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// LenzHLineYUY2 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void LenzHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor);
// PeelzHLineYUY2 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally. Shading can be done optionally
inline void PeelzHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,int shade);
//ShadeHLineYUY2 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,int shade);

//**************************************************************************************************/
#ifndef Linez_V_C_MOHAN
#define Linez_V_C_MOHAN
#include "LineZ.h"
//Functions for Horizontal lines 

inline void CopyHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0;w<winw;w++)
		*((int*)(dstp+4*w))=*((int *)(srcp+4*w));
}

inline void CopyHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0;w<3*winw;w++)
		*((dstp+w))=*((srcp+w));
}

inline void CopyHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0;w<winw/2;w++)
		*((int*)(dstp+4*w))=*((int *)(srcp+4*w));
}

inline void FlipHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0; w<winw; w++)
		*((int*)(dstp+4*w))=*((int *)(srcp+4*(winw-1-w)));
}

inline void FlipHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0; w<winw; w++)
		for(int k=0;k<3;k++)
			*(dstp+3*w+k)=*(srcp+3*(winw-1-w)+k);
}
inline void FlipHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0; w<winw/2; w++)
		*((int*)(dstp+4*w))=(((*((int *)(srcp+4*(winw-1-w))))& 0xff000000)>>16) |
							(((*((int *)(srcp+4*(winw-1-w))))& 0xff00)<<16) |
							((*((int *)(srcp+4*(winw-1-w))))& 0xff00ff);
}

// ResizeHLineRGB32 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation	
	for(int w=0; w<winw/2; w++)
	{
		
		
		*((int *)(dstp+4*w))=*((int *)(srcp+4*((int)(factor*w))));
		*((int *)(dstp-4*w))=*((int *)(srcp-4*((int)(factor*w))));
		
	}
}

// ResizeHLineYUY2 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation. 2 points at a time. So bad approx	
	for(int w=0; w<winw/2; w+=2)
	{
		
		
		*((int *)(dstp+2*w))=*((int *)(srcp+2*(((int)(factor*w))& 0xFFFFFFFE)));
		*((int *)(dstp-2*w))=*((int *)(srcp-2*(((int)(factor*w))& 0xFFFFFFFE)));
		
	}
}
// ResizeHLineRGB24 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void ResizeHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation	
	for(int w=0; w<winw/2; w++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+3*w+k)=*(srcp+3*((int)(factor*w))+k);
			*(dstp-3*w+k)=*(srcp-3*((int)(factor*w))+k);
		}
	}
}

// FlipResizeHLineRGB32 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation	
	for(int w=0; w<winw/2; w++)
	{
		*((int *)(dstp+4*w))=*((int *)(srcp-4*((int)(factor*w))));
		*((int *)(dstp-4*w))=*((int *)(srcp+4*((int)(factor*w))));
	}
}

// FlipResizeHLineYUY2 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation. two pointse. Bad approx	
	for(int w=0; w<winw/2; w++)
	{
		*((int *)(dstp+2*w))=*((int *)(srcp-2*(((int)(factor*w))& 0xFFFFFFFE)));
		*((int *)(dstp-2*w))=*((int *)(srcp+2*(((int)(factor*w))& 0xFFFFFFFE)));
	}
}
// FlipResizeHLineRGB24 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation	
	for(int w=0; w<winw/2; w++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+3*w+k)=*(srcp-3*((int)(factor*w))+k);
			*(dstp-3*w+k)=*(srcp+3*((int)(factor*w))+k);
		}
	}
}

//

// FlipLenzHLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void FlipLenzHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+3*w+k)=*(srcp-3*((int)((factor+(1-factor)*w*2/winw)*w))+k);
			*(dstp-3*w+k)=*(srcp+3*((int)((factor+(1-factor)*w*2/winw)*w))+k);
		}
	}
}


// LenzHLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void LenzHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+3*w+k)=*(srcp+3*((int)((factor+(1-factor)*w*2/winw)*w))+k);
			*(dstp-3*w+k)=*(srcp-3*((int)((factor+(1-factor)*w*2/winw)*w))+k);
		}
	}
}

// FlipLenzHLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void FlipLenzHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		
		*((int *)(dstp+4*w))=*((int *)(srcp-4*((int)((factor+(1-factor)*w*2/winw)*w))));
		*((int *)(dstp-4*w))=*((int *)(srcp+4*((int)((factor+(1-factor)*w*2/winw)*w))));
		
	}
}


// LenzHLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void LenzHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		
		*((int *)(dstp+4*w))=*((int *)(srcp+4*((int)((factor+(1-factor)*w*2/winw)*w))));
		*((int *)(dstp-4*w))=*((int *)(srcp-4*((int)((factor+(1-factor)*w*2/winw)*w))));
		
	}
}

// FlipLenzHLineYUY2 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void FlipLenzHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		
		*((int *)(dstp+2*w))=*((int *)(srcp-2*((int)((factor+(1-factor)*w*2/winw)*w))));
		*((int *)(dstp-2*w))=*((int *)(srcp+2*((int)((factor+(1-factor)*w*2/winw)*w))));
		
	}
}


// LenzHLineYUY2 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void LenzHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		
		*((int *)(dstp+2*w))=*((int *)(srcp+2*((int)((factor+(1-factor)*w*2/winw)*w))));
		*((int *)(dstp-2*w))=*((int *)(srcp-2*((int)((factor+(1-factor)*w*2/winw)*w))));
		
	}
}

// PeelzHLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally. Shading can be done optionally
inline void PeelzHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,int shade)
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		
		*((int *)(dstp+4*w))=(*((int *)(srcp-4*((int)(w*(1+(double)(2*w/winw)))))))
				| 0x01010101*((shade*(wwd-2*w))/winw);
		*((int *)(dstp-4*w))=(*((int *)(srcp+4*((int)(w*(1+(double)(2*w/winw))))))) 
				| 0x01010101*((shade*(wwd-2*w))/winw);
		
	}
}
// PeelzHLineYUY2 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally. Shading can be done optionally
inline void PeelzHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,int shade)
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		
		*((int *)(dstp+2*w))=(*((int *)(srcp-2*((int)(w*(1+(double)(2*w/winw)))))))
				| ((shade*(wwd-2*w))/winw);
		*((int *)(dstp-2*w))=(*((int *)(srcp+2*((int)(w*(1+(double)(2*w/winw))))))) 
				| ((shade*(wwd-2*w))/winw);
		
	}
}

// PeelzHLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally.
inline void PeelzHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw, int shade )
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+3*w+k)=(*(srcp-3*((int)(w*(1+(double)(2*w/winw))))+k))
				| (unsigned char)((shade*(wwd-2*w))/winw);
			*(dstp-3*w+k)=(*(srcp+3*((int)(w*(1+(double)(2*w/winw))))+k)) 
				| (unsigned char)((shade*(wwd-2*w))/winw);
		}
	}
}
//ShadeHLineRGB24 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeHLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winw,int shade)
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+3*w+k)=(*(srcp+3*w+k)) | (unsigned char)((shade*(wwd-2*w))/winw);
			*(dstp-3*w+k)=(*(srcp-3*w+k)) | (unsigned char)((shade*(wwd-2*w))/winw);
		}
	}
}

//ShadeHLineRGB32 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeHLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winw,int shade)
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		
				
			*((int*)(dstp+4*w))=(*((int*)(srcp+4*w))) | 0x01010101*((shade*(wwd-2*w))/winw);
			*((int*)(dstp-4*w))=(*((int*)(srcp-4*w))) | 0x01010101*((shade*(wwd-2*w))/winw);
		
		
	}
}


//ShadeHLineYUY2 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeHLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winw,int shade)
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		
				
			*(dstp+2*w)=(*(srcp+2*w)) | ((shade*(wwd-2*w))/winw);
			*(dstp-2*w)=(*(srcp-2*w)) | ((shade*(wwd-2*w))/winw);
		
		
	}
}
//***************************************************************************************************************
//***************************************************************************************************************
//Functions for Vertical lines 
// CopyVLineRGB32 expects the srcp and dstp to be at the end point of the segment to be copied 
inline void CopyVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0;h<winh;h++)
		*((int*)(dstp+dpitch*h))=*((int *)(srcp+spitch*h));
}
// CopyVLineRGB24 expects the srcp and dstp to be at the end point of the segment to be copied 
inline void CopyVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0;h<winh;h++)
		for(int k=0;k<3;k++)
			*(dstp+dpitch*h+k)=*(srcp+spitch*h+k);
}
// CopyVLineYUY2 expects the srcp and dstp to be at the start point of the segment to be copied. Two adjacent
//pixells are copied at a time. So two vertical lines are copied
 
inline void CopyVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0;h<winh;h++)
		*((int*)(dstp+dpitch*h))=*((int *)(srcp+spitch*h));
}
//FlipVLineRGB32 expects the srcp and dstp to be at the start point of the segment to be copied 
inline void FlipVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0; h<winh; h++)
		*((int*)(dstp+dpitch*h))=*((int *)(srcp+spitch*(winh-1-h)));
}
//FlipVLineRGB24 expects the srcp and dstp to be at the start point of the segment to be copied 
inline void FlipVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0; h<winh; h++)
		for(int k=0; k<3; k++)
			*(dstp+dpitch*h+k)=*(srcp+spitch*(winh-1-h)+k);
}
//FlipVLineYUY2 expects the srcp and dstp to be at 32 bit boundary of the start point of the segment to be copied. At a time
// two adjacent vertical lines are flipped.  
inline void FlipVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0; h<winh; h++)
		*((int*)(dstp+dpitch*h))=*((int *)(srcp+spitch*(winh-1-h)));
}

// ResizeVLineRGB32 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{
	
		*((int *)(dstp+dpitch*h))=*((int *)(srcp+spitch*((int)(factor*h))));
		*((int *)(dstp-dpitch*h))=*((int *)(srcp-spitch*((int)(factor*h))));
		
	}
}

// ResizeVLineRGB24 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+dpitch*h+k)=*(srcp+spitch*((int)(factor*h))+k);
			*(dstp-dpitch*h+k)=*(srcp-spitch*((int)(factor*h))+k);
		}
	}
}
// ResizeVLineYUY2 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{
	
		*((int *)(dstp+dpitch*h))=*((int *)(srcp+spitch*((int)(factor*h))));
		*((int *)(dstp-dpitch*h))=*((int *)(srcp-spitch*((int)(factor*h))));
		
	}
}

// FlipResizeVLineRGB32 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{

		*((int *)(dstp+dpitch*h))=*((int *)(srcp-spitch*((int)(factor*h))));
		*((int *)(dstp-dpitch*h))=*((int *)(srcp+spitch*((int)(factor*h))));
		
	}
}
// FlipResizeVLineRGB24 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+dpitch*h+k)=*(srcp-spitch*((int)(factor*h))+k);
			*(dstp-dpitch*h+k)=*(srcp+spitch*((int)(factor*h))+k);
		}
	}
}
// FlipResizeVLineYUY2 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{

		*((int *)(dstp+dpitch*h))=*((int *)(srcp-spitch*((int)(factor*h))));
		*((int *)(dstp-dpitch*h))=*((int *)(srcp+spitch*((int)(factor*h))));
		
	}
}

//

// FlipLenzVLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winh
inline void FlipLenzVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
	for(int h=0; h<winh/2; h++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+dpitch*h+k)=*(srcp-spitch*((int)((factor+(1-factor)*h*2/winh)*h))+k);
			*(dstp-dpitch*h+k)=*(srcp+spitch*((int)((factor+(1-factor)*h*2/winh)*h))+k);
		}
	}
}


// LenzVLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window height/destination window height winh
inline void LenzVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
	for(int h=0; h<winh/2; h++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+dpitch*h+k)=*(srcp+spitch*((int)((factor+(1-factor)*h*2/winh)*h))+k);
			*(dstp-dpitch*h+k)=*(srcp-spitch*((int)((factor+(1-factor)*h*2/winh)*h))+k);
		}
	}
}

// FlipLenzVLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winh
inline void FlipLenzVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
	for(int h=0; h<winh/2; h++)
	{
		
		*((int *)(dstp+dpitch*h))=*((int *)(srcp-spitch*((int)((factor+(1-factor)*h*2/winh)*h))));
		*((int *)(dstp-dpitch*h))=*((int *)(srcp+spitch*((int)((factor+(1-factor)*h*2/winh)*h))));
		
	}
}


// LenzVLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winh
inline void LenzVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
	for(int h=0; h<winh/2; h++)
	{
		*((int *)(dstp+dpitch*h))=*((int *)(srcp+spitch*((int)((factor+(1-factor)*h*2/winh)*h))));
		*((int *)(dstp-dpitch*h))=*((int *)(srcp-spitch*((int)((factor+(1-factor)*h*2/winh)*h))));
	}
}


// PeelzVLineRGB32 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh.
//The line is flipped vertically. Shading can be done optionally  shade= Zero will have no shading
inline void PeelzVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade)
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
			*((int*)(dstp+dpitch*h))=*((int *)(srcp-spitch*((int)(h*(1+(double)(2*h/winh))))))
				| 0x01010101*((shade*(wwd-2*h))/winh);
			*((int*)(dstp-dpitch*h))=*((int *)(srcp+spitch*((int)(h*(1+(double)(2*h/winh)))))) 
				| 0x01010101*((shade*(wwd-2*h))/winh);
	}
}


// PeelzVLineRGB24 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh.
//The line is flipped vertically.shading can be done. shade= Zero will have no shading
inline void PeelzVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch, int shade )
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+dpitch*h+k)=(*(srcp-spitch*((int)(h*(1+(double)(2*h/winh))))+k))
				| (unsigned char)((shade*(wwd-2*h))/winh);
			*(dstp-dpitch*h+k)=(*(srcp+spitch*((int)(h*(1+(double)(2*h/winh))))+k)) 
				| (unsigned char)((shade*(wwd-2*h))/winh);
		}
	}
}
// PeelzVLineYUY22 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh, and divisible by 4.
//The line is flipped vertically. Shading can be done optionally  shade= Zero will have no shading
inline void PeelzVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade)
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
			*((int*)(dstp+dpitch*h))=*((int *)(srcp-spitch*((int)(h*(1+(double)(2*h/winh))))))
				| 0x01000100*((shade*(wwd-2*h))/winh);
			*((int*)(dstp-dpitch*h))=*((int *)(srcp+spitch*((int)(h*(1+(double)(2*h/winh)))))) 
				| 0x01000100*((shade*(wwd-2*h))/winh);
	}
}
//ShadeVLineRGB24 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeVLineRGB24( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade)
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
		for(int k=0; k<3; k++)
		{
			*(dstp+dpitch*h+k)=(*(srcp+spitch*h+k)) | (unsigned char)((shade*(wwd-2*h))/winh);
			*(dstp-dpitch*h+k)=(*(srcp-spitch*h+k)) | (unsigned char)((shade*(wwd-2*h))/winh);
		}
	}
}


//ShadeVLineRGB32 adds shading to a line. -255 to 255 Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeVLineRGB32( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade)
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
		
		*((int*)(dstp+dpitch*h))=(*((int*)(srcp+spitch*h))) | 0x01010101*((shade*(wwd-2*h))/winh);
		*((int*)(dstp-dpitch*h))=(*((int*)(srcp-spitch*h))) | 0x01010101*((shade*(wwd-2*h))/winh);
		
	}
}
//ShadeVLineYUY2 adds shading to a line. -255 to 255 Center gets whiter compared to edges.
// the pointers point the center of window. Two lines are copied/shaded at a time 
inline void ShadeVLineYUY2( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade)
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
		
		*((int*)(dstp+dpitch*h))=(*((int*)(srcp+spitch*h))) | 0x01010101*((shade*(wwd-2*h))/winh);
		*((int*)(dstp-dpitch*h))=(*((int*)(srcp-spitch*h))) | 0x01010101*((shade*(wwd-2*h))/winh);
		
	}
}

/*******************************************************************************************************************
YV12 functions


**********************************************************************************************************************/
inline void CopyHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0;w<winw;w++)
		*(dstp+w)=*(srcp+w);
}
inline void FlipHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw)
{
	for(int w=0; w<winw; w++)
		*(dstp+w)=*(srcp+(winw-1-w));
}
// ResizeHLineYV12 requires the srcp and dstp to point to middle of window winw/2. The factor is input widow width/winw
inline void ResizeHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation	
	for(int w=0; w<winw/2; w++)
	{
		
		
		*(dstp+w)=*(srcp+(int)(factor*w));
		*(dstp-w)=*(srcp-(int)(factor*w));
		
	}
}
// FlipResizeHLineYV12 requires the srcp and dstp to point to middle of window winw/2. The factor is frame width/winw
inline void FlipResizeHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
						// nearest point interpolation	
	for(int w=0; w<winw/2; w++)
	{
		*(dstp+w)=*(srcp-(int)(factor*w));
		*(dstp-w)=*(srcp+(int)(factor*w));
	}
}
// FlipLenzHLineYV12 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void FlipLenzHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		
			*(dstp+w)=*(srcp-((int)((factor+(1-factor)*w*2/winw)*w)));
			*(dstp-w)=*(srcp+((int)((factor+(1-factor)*w*2/winw)*w)));
		
	}
}
// LenzHLineYV12 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winw
inline void LenzHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,double factor)
{
	for(int w=0; w<winw/2; w++)
	{
		
			*(dstp+w)=*(srcp+((int)((factor+(1-factor)*w*2/winw)*w)));
			*(dstp-w)=*(srcp-((int)((factor+(1-factor)*w*2/winw)*w)));
		
	}
}
// PeelzHLineYV12 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winw.
//The line is flipped horizontally. Shading can be done optionally
inline void PeelzHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,int shade)
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		
		*(dstp+w)=*(srcp-((int)(w*(1+(double)(2*w/winw)))))
				| 0x01010101*((shade*(wwd-2*w))/winw);
		*(dstp-w)=*(srcp+((int)(w*(1+(double)(2*w/winw))))) 
				| 0x01010101*((shade*(wwd-2*w))/winw);
		
	}
}
//ShadeHLineYV12 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window.
inline void ShadeHLineYV12( const unsigned char* srcp, unsigned char* dstp, int winw,int shade)
{
	int wwd = (shade>=0?winw:0);
	for(int w=0; w<winw/2; w++)
	{
		
			*(dstp+w)=(*(srcp+w)) | (unsigned char)((shade*(wwd-2*w))/winw);
			*(dstp-w)=(*(srcp-w)) | (unsigned char)((shade*(wwd-2*w))/winw);
		
	}
}
//***************************************************************************************************************
//Functions for Vertical lines 
// CopyVLineYV12 expects the srcp and dstp to be at the end point of the segment to be copied. 
inline void CopyVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0;h<winh;h++)
		*(dstp+dpitch*h)=*(srcp+spitch*h);
}
//FlipVLineYV12 expects the srcp and dstp to be at the start point of the segment to be copied 
inline void FlipVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch)
{
	for(int h=0; h<winh; h++)
		*(dstp+dpitch*h)=*(srcp+spitch*(winh-1-h));
}
// ResizeVLineYV12 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void ResizeVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{
	
		*(dstp+dpitch*h)=*(srcp+spitch*((int)(factor*h)));
		*(dstp-dpitch*h)=*(srcp-spitch*((int)(factor*h)));
		
	}
}
// FlipResizeVLineYV12 requires the srcp and dstp to point to middle of window winh/2. The factor is frame width/winh
inline void FlipResizeVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
						// nearest point interpolation	
	for(int h=0; h<winh/2; h++)
	{
	
		*(dstp+dpitch*h)=*(srcp-spitch*((int)(factor*h)));
		*(dstp-dpitch*h)=*(srcp+spitch*((int)(factor*h)));
		
	}
}
// FlipLenzVLineYV12 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winh
inline void FlipLenzVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
	for(int h=0; h<winh/2; h++)
	{
		
		*(dstp+dpitch*h)=*(srcp-spitch*((int)((factor+(1-factor)*h*2/winh)*h)));
		*(dstp-dpitch*h)=*(srcp+spitch*((int)((factor+(1-factor)*h*2/winh)*h)));
		
	}
}
// LenzVLineYV12 requires the srcp and dstp to point to middle of source window and destination windows
// The factor is source window width/destination window width winh
inline void LenzVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,double factor)
{
	for(int h=0; h<winh/2; h++)
	{
		
		*(dstp+dpitch*h)=*(srcp+spitch*((int)((factor+(1-factor)*h*2/winh)*h)));
		*(dstp-dpitch*h)=*(srcp-spitch*((int)((factor+(1-factor)*h*2/winh)*h)));
		
	}
}
// PeelzVLineYV12 requires the srcp and dstp to point to middle of source window and destination windows.
// the destw normally should be touching the srcwindow towards peel direction
//source window should be double that of destination window winh.
//The line is flipped vertically. Shading can be done optionally  shade= Zero will have no shading
inline void PeelzVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade)
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
			*(dstp+dpitch*h)=*(srcp-spitch*((int)(h*(1+(double)(2*h/winh)))))
				| 0x01010101*((shade*(wwd-2*h))/winh);
			*(dstp-dpitch*h)=*(srcp+spitch*((int)(h*(1+(double)(2*h/winh))))) 
				| 0x01010101*((shade*(wwd-2*h))/winh);
	}
}
//ShadeVLineYV12 adds shading to a line. Center gets whiter compared to edges.
// the pointers point the center of window. 
inline void ShadeVLineYV12( const unsigned char* srcp, unsigned char* dstp, int winh,int spitch, int dpitch,int shade)
{
	int wwd = (shade>=0?winh:0);
	for(int h=0; h<winh/2; h++)
	{
		
			*(dstp+dpitch*h)=(*(srcp+spitch*h)) | (unsigned char)((shade*(wwd-2*h))/winh);
			*(dstp-dpitch*h)=(*(srcp-spitch*h)) | (unsigned char)((shade*(wwd-2*h))/winh);
		
	}
}
#endif